﻿/*  ----------------------------------------------------------------------------
 *  Emergint Technologies, Inc.
 *  ----------------------------------------------------------------------------
 *  Epi Info™ - Web Analysis & Visualization
 *  ----------------------------------------------------------------------------
 *  File:       IAdminDatasourceDao.cs
 *  Namespace:  Ewav.DAL.Interfaces    
 *
 *  Author(s):  Daniel Shorter, Mohammad Usman, Ninad Date, Sachin Agnihotri 
 *  Created:    26/08/2013    
 *  Summary:	no summary     
 *  ----------------------------------------------------------------------------
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ewav.DTO;
using System.Data;

namespace Ewav.DAL.Interfaces
{
    public interface IAdminDatasourceDao
    {
        /// <summary>
        /// Gets or sets the connection string.
        /// </summary>
        /// <value>The connection string.</value>
        string ConnectionString { get; set; }

        /// <summary>
        /// Gets or sets the name of the table.
        /// </summary>
        /// <value>The name of the table.</value>
        string TableName { get; set; }

        /// <summary>
        /// Tests the DB connection.
        /// </summary>
        /// <param name="connStr">The conn STR.</param>
        /// <returns></returns>
        bool TestDBConnection(string connStr);

        /// <summary>
        /// Tests the data.
        /// </summary>
        /// <param name="sqlOrTableName">Name of the SQL or table.</param>
        /// <returns></returns>
        bool TestData(string sqlOrTableName);

        /// <summary>
        /// Adds the datasource.
        /// </summary>
        /// <param name="dsDto">The ds dto.</param>
        /// <returns></returns>
        bool AddDatasource(DatasourceDto dsDto);

        /// <summary>
        /// Updates the datasource.
        /// </summary>
        /// <param name="dsDto">The ds dto.</param>
        /// <returns></returns>
        bool UpdateDatasource(DatasourceDto dsDto);

        /// <summary>
        /// Removes the datasource.
        /// </summary>
        /// <param name="dsId">The ds id.</param>
        /// <returns></returns>
        bool RemoveDatasource(int dsId);

        /// <summary>
        /// Reads the datasource.
        /// </summary>
        /// <param name="orgId">The org id.</param>
        /// <returns></returns>
        DataSet ReadDatasource(int orgId);

        /// <summary>
        /// Reads the users that are part of a organization    
        /// </summary>
        /// <param name="orgId">The org id.</param>
        /// <returns></returns>
        DataSet ReadAssociatedUsers(int orgId);

        /// <summary>
        ///  Reads the users that are part of a organization and have access to a particular datasource  
        /// </summary>
        /// <param name="dsID">The datasource ID.</param>
        /// <param name="orgId">The organization id.</param>
        /// <returns></returns>
        DataSet ReadAssociatedUsersForDatasource(int dsID, int orgId);

        /// <summary>
        /// Gets the columns for the table or view for a datasource.
        /// </summary>
        /// <returns></returns>
        DataTable GetColumnsForDatasource();

        /// <summary>
        /// Gets the connection string from DB.
        /// </summary>
        /// <param name="DsName">Name of the ds.</param>
        /// <param name="ConnectionString">The connection string.</param>
        /// <returns></returns>
        string GetConnectionStringFromDB(string DsName, string ConnectionString);

    }
}