﻿/*  ----------------------------------------------------------------------------
 *  Emergint Technologies, Inc.
 *  ----------------------------------------------------------------------------
 *  Epi Info™ - Web Analysis & Visualization
 *  ----------------------------------------------------------------------------
 *  File:       CanvasDomainService.cs
 *  Namespace:  Ewav.Web.Services.CanvasDomainService    
 *
 *  Author(s):  Daniel Shorter, Mohammad Usman, Ninad Date, Sachin Agnihotri 
 *  Created:    26/08/2013    
 *  Summary:	no summary     
 *  ----------------------------------------------------------------------------
 */
namespace Ewav.Web.Services.CanvasDomainService
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.DataAnnotations;
    using System.Linq;
    using System.ServiceModel.DomainServices.Hosting;
    using System.ServiceModel.DomainServices.Server;
    using Ewav.DTO;
    using System.Data;


    // TODO: Create methods containing your application logic.
    [EnableClientAccess()]
    public class CanvasDomainService : DomainService
    {
        CanvasManager cm = new CanvasManager();

        /// <summary>
        /// Saves Canvas
        /// </summary>
        /// <param name="canvasDto"></param>
        /// <returns></returns>
        public int SaveCanvas(CanvasDto canvasDto)
        {
            return cm.SaveCanvas(canvasDto);

            // return false;
        }

        /// <summary>
        /// Loads Canvas List for User
        /// </summary>
        /// <param name="user"></param>
        /// <returns></returns>
        public DatatableBag LoadCanvasListForUser(int UserId)
        {
            DataSet ds = cm.LoadCanvasListForUser(UserId);
            DatatableBag dtb = new DatatableBag();
            return dtb.ConvertToDatatableBag(ds.Tables[0]);

        }

        /// <summary>
        /// Loads selected Canvas
        /// </summary>
        /// <param name="canvasId"></param>
        /// <returns></returns>
        public CanvasDto LoadCanvas(int canvasId)
        {
            return cm.LoadCanvas(canvasId);
        }

        /// <summary>
        /// Shares canvas
        /// </summary>
        /// <param name="canvasId"></param>
        /// <param name="SharedUserIdList"></param>
        public bool ShareCanvas(int canvasId, List<int> sharedUserIds)
        {
            cm.ShareCanvas(canvasId, sharedUserIds);
            return false;
        }

        /// <summary>
        /// Deletes canvas
        /// </summary>
        /// <param name="canvasId"></param>
        /// <returns></returns>
        public bool DeleteCanvas(int canvasId)
        {
            cm.DeleteCanvas(canvasId);
            return false;
        }

        /// <summary>
        /// Reads All the users in my org
        /// </summary>
        /// <param name="orgId"></param>
        /// <returns></returns>
        public DatatableBag ReadAllUsersInMyOrg(int orgId)
        {
            DataSet ds = cm.ReadAllUsersInMyOrg(orgId);
            DatatableBag dtb = new DatatableBag();
            return dtb.ConvertToDatatableBag(ds.Tables[0]);
        }

        /// <summary>
        /// Exports the Data in to DatatableBag
        /// </summary>
        /// <param name="dsName"></param>
        /// <param name="tableName"></param>
        /// <param name="ewavDataFilters"></param>
        /// <param name="rules"></param>
        /// <returns></returns>
        public DatatableBag ExportData(string dsName, string tableName,
               IEnumerable<EwavDataFilterCondition> ewavDataFilters, List<EwavRule_Base> rules)
        {
            DatatableBag dtb = new DatatableBag();
            EpiDashboard.GadgetParameters gp = new EpiDashboard.GadgetParameters();
            gp.DatasourceName = dsName;
            gp.TableName = tableName;
            EpiDashboard.DashboardHelper dbHelper = new EpiDashboard.DashboardHelper(gp, ewavDataFilters, rules);
            dbHelper.PopulateDataSet();
            DataView dv = dbHelper.DataSet.Tables[0].DefaultView;
            ///Write code here
            return dtb;
        }
    }
}