﻿/*  ----------------------------------------------------------------------------
 *  Emergint Technologies, Inc.
 *  ----------------------------------------------------------------------------
 *  Epi Info™ - Web Analysis & Visualization
 *  ----------------------------------------------------------------------------
 *  File:       ContextMenuCreator.cs
 *  Namespace:  Ewav    
 *
 *  Author(s):  Daniel Shorter, Mohammad Usman, Ninad Date, Sachin Agnihotri 
 *  Created:    26/08/2013    
 *  Summary:	no summary     
 *  ----------------------------------------------------------------------------
 */
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Ewav.Common;
using Ewav.ContextMenu;
using System.Text;
using System.Collections.Generic;
using System;
using System.Windows.Browser;
using System.Windows.Media;
using Ewav.Web.Services;
using EpiDashboard;
using CommonLibrary;
using System.ComponentModel.Composition;
using System.Linq;
using Ewav.ViewModels;

namespace Ewav
{
    public class ContextMenuCreator
    {
        Menu popupMenu;

        ViewModels.ApplicationViewModel applicationViewModel = ViewModels.ApplicationViewModel.Instance;

        //  public     static  event  MemuPopup  

        public static event EventHandler MenuPopup;

        public Menu PopupMenu
        {
            get
            {
                return popupMenu;
            }
            set
            {
                popupMenu = value;
            }
        }

        IGadget gadget;

        #region Properties

        ClientCommon.Common cmnClass;

        /// <summary>
        /// The list of grids for each strata value. E.g. if stratifying by SEX, there would be (likely)
        /// two grids in this list: One showing the summary statistics for Male and another for Female.
        /// </summary>

        private List<Grid> strataGridList;

        public List<Grid> StrataGridList
        {
            get
            {
                return strataGridList;
            }
            set
            {
                strataGridList = value;
            }
        }

        /// <summary>
        /// Contains the reference for the Selected Gadget.
        /// </summary>
        private IGadget selectedGadget;

        public IGadget SelectedGadget
        {
            get
            {
                return selectedGadget;
            }
            set
            {
                selectedGadget = value;
            }
        }

        /// <summary>
        /// Contains the reference for The Root Canvas.
        /// </summary>
        private Canvas root;

        public Canvas Root
        {
            get
            {
                return root;
            }
            set
            {
                root = value;
            }
        }

        private DragCanvas dgRoot;

        public DragCanvas DgRoot
        {
            get
            {
                return dgRoot;
            }
            set
            {
                dgRoot = value;
            }
        }
        /// <summary>
        /// Intializes the name of Control where the right click was clicked.
        /// </summary>
        private string controlName;

        public string ControlName
        {
            get
            {
                return controlName;
            }
            set
            {
                controlName = value;
            }
        }

        private bool popupDisplayed;

        public bool PopupDisplayed
        {
            get
            {
                return popupDisplayed;
            }
            set
            {
                popupDisplayed = value;
            }
        }

        private Grid layoutRoot;


        private GridCells gridCells;

        public GridCells GridCells
        {
            get
            {
                return gridCells;
            }
            set
            {
                gridCells = value;
            }
        }

        private GadgetParameters gadgetparameters;

        public GadgetParameters Gadgetparameters
        {
            get
            {
                return gadgetparameters;
            }
            set
            {
                gadgetparameters = value;
            }
        }

        private bool displayWithOutDSSelected;

        public bool DisplayWithOutDSSelected
        {
            get { return displayWithOutDSSelected; }
            set { displayWithOutDSSelected = value; }
        }

        #endregion

        #region Constructors

        public ContextMenuCreator()
        {
        }

        private Point contextMenuRightClickCoords;


        /// <summary>
        /// Constructor that instantiats the contextMenu.
        /// </summary>
        /// <param name="controlName"></param>
        /// <param name="StrataList"></param>
        /// <param name="selectedGadget"></param>
        /// <param name="canvas"></param>
        /// <returns></returns>
        public Menu CreateContextMenu(string controlName, Point mouseButtonCoords,
            List<Grid> StrataList, IGadget selectedGadget, Canvas canvas,
            Grid LayoutRoot, GridCells gc, GadgetParameters gp)
        {
            contextMenuRightClickCoords = mouseButtonCoords;

            cmnClass = new ClientCommon.Common();
            popupMenu = new Menu();
            this.StrataGridList = StrataList;
            this.SelectedGadget = selectedGadget;
            this.Root = canvas;
            this.DgRoot = (DragCanvas)canvas;
            this.ControlName = controlName;
            this.GridCells = gc;
            this.Gadgetparameters = gp;
            switch (controlName.ToUpper())
            {
                case "FREQUENCYCONTROL":
                case "MEANSCONTROL":
                    Ewav.ContextMenu.MenuItem mnuCopy = new Ewav.ContextMenu.MenuItem("Copy data to clipboard", "Images/clipboard.png");
                    mnuCopy.MouseLeftButtonDown += new MouseButtonEventHandler(mnuCopy_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem mnuSendDataToHTML = new Ewav.ContextMenu.MenuItem("Send data to web browser", "Images/webexport.png");
                    mnuSendDataToHTML.MouseLeftButtonDown += new MouseButtonEventHandler(mnuSendDataToHTML_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem mnuSendToBack = new Ewav.ContextMenu.MenuItem("Send gadget to back", "Images/sendback.png");
                    mnuSendToBack.MouseLeftButtonDown += new MouseButtonEventHandler(mnuSendToBack_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem mnuClose = new Ewav.ContextMenu.MenuItem("Close this gadget", "Images/closegadget.png");
                    mnuClose.MouseLeftButtonDown += new MouseButtonEventHandler(mnuClose_MouseLeftButtonDown);
                    popupMenu.Items.Add(mnuCopy);
                    popupMenu.Items.Add(mnuSendDataToHTML);
                    popupMenu.Items.Add(mnuSendToBack);
                    popupMenu.Items.Add(mnuClose);
                    break;
                case "ABERRATIONCONTROL":
                case "LOGISTICREGRESSION":
                case "LINEARREGRESSION":
                case "STATCALC2X2":
                case "BINOMIAL":
                case "COHORT":
                case "POPULATION":
                case "UNMATCHED":
                case "CHISQUARE":
                case "POISSON":
                case "LINELIST":
                case "MAPCONTROL":
                    Ewav.ContextMenu.MenuItem mnuSendToBack2 = new Ewav.ContextMenu.MenuItem("Send gadget to back", "Images/sendback.png");
                    mnuSendToBack2.MouseLeftButtonDown += new MouseButtonEventHandler(mnuSendToBack_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem mnuClose2 = new Ewav.ContextMenu.MenuItem("Close this gadget", "Images/closegadget.png");
                    mnuClose2.MouseLeftButtonDown += new MouseButtonEventHandler(mnuClose_MouseLeftButtonDown);
                    popupMenu.Items.Add(mnuSendToBack2);
                    popupMenu.Items.Add(mnuClose2);
                    break;
                case "MXNTABLECONTROL":
                case "TWOXTWOTABLECONTROL":
                    Ewav.ContextMenu.MenuItem mnuCopy3 = new Ewav.ContextMenu.MenuItem("Copy data to clipboard", "Images/clipboard.png");
                    mnuCopy3.MouseLeftButtonDown += new MouseButtonEventHandler(mnuCopy_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem mnuSendDataToHTML3 = new Ewav.ContextMenu.MenuItem("Send data to web browser", "Images/webexport.png");
                    mnuSendDataToHTML3.MouseLeftButtonDown += new MouseButtonEventHandler(mnuSendDataToHTML_MouseLeftButtonDown);

                    Ewav.ContextMenu.MenuItem mnuSendToBack3 = new Ewav.ContextMenu.MenuItem("Send gadget to back", "Images/sendback.png");
                    mnuSendToBack3.MouseLeftButtonDown += new MouseButtonEventHandler(mnuSendToBack_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem mnuClose3 = new Ewav.ContextMenu.MenuItem("Close this gadget", "Images/closegadget.png");
                    mnuClose3.MouseLeftButtonDown += new MouseButtonEventHandler(mnuClose_MouseLeftButtonDown);
                    popupMenu.Items.Add(mnuCopy3);
                    popupMenu.Items.Add(mnuSendDataToHTML3);
                    popupMenu.Items.Add(mnuSendToBack3);
                    popupMenu.Items.Add(mnuClose3);


                    break;
                case "EPICURVE":
                case "SCATTER":
                case "XYCHARTCONTROL":
                    Ewav.ContextMenu.MenuItem mnuSetChartTitle = new ContextMenu.MenuItem("Set Chart Titles", "Images/title.png");
                    mnuSetChartTitle.MouseLeftButtonDown += new MouseButtonEventHandler(mnuSetChartTitle_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem munSaveAsImage = new ContextMenu.MenuItem("Save as Image", "Images/save1.png");
                    munSaveAsImage.MouseLeftButtonDown += new MouseButtonEventHandler(munSaveAsImage_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem mnuSendToBack4 = new Ewav.ContextMenu.MenuItem("Send gadget toback", "Images/sendback.png");
                    mnuSendToBack4.MouseLeftButtonDown += new MouseButtonEventHandler(mnuSendToBack_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem mnuClose4 = new Ewav.ContextMenu.MenuItem("Close this gadget", "Images/closegadget.png");
                    mnuClose4.MouseLeftButtonDown += new MouseButtonEventHandler(mnuClose_MouseLeftButtonDown);
                    popupMenu.Items.Add(mnuSetChartTitle);
                    popupMenu.Items.Add(munSaveAsImage);
                    popupMenu.Items.Add(mnuSendToBack4);
                    popupMenu.Items.Add(mnuClose4);
                    break;
                default:
                    // we can assume this is a right-click    
                    Ewav.ContextMenu.MenuItem GadgetSel = new Ewav.ContextMenu.MenuItem("", ""); //Placeholder to prevent show of submenu on load
                    GadgetSel.IsEnabled = false;
                    GadgetSel.FontSize = 0.2;
                    GadgetSel.Foreground = new SolidColorBrush(Color.FromArgb(255, 0, 0, 0));
                    Ewav.ContextMenu.MenuItem Gadget = new Ewav.ContextMenu.MenuItem("Add Gadgets", "Images/gadget.png");
                    Gadget.IsEnabled = !DisplayWithOutDSSelected;
                    Gadget.MouseLeftButtonDown += new MouseButtonEventHandler(MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem Chart = new Ewav.ContextMenu.MenuItem("Add Charts", "Images/chart.png");
                    Chart.IsEnabled = !DisplayWithOutDSSelected; ;
                    Chart.MouseLeftButtonDown += new MouseButtonEventHandler(MouseLeftButtonDown);

                    Ewav.ContextMenu.MenuDivider MD = new MenuDivider();
                    //Gadget.MouseLeftButtonDown += new MouseButtonEventHandler(Gadget_MouseLeftButtonDown);
                    Ewav.ContextMenu.MenuItem StatCalc = new Ewav.ContextMenu.MenuItem("Add StatCalc Calculator", "Images/calculator.png");
                    StatCalc.MouseLeftButtonDown += new MouseButtonEventHandler(MouseLeftButtonDown);
                    //Ewav.ContextMenu.MenuItem Save = new Ewav.ContextMenu.MenuItem("Save DashBoard", "Images/save1.png");
                    //Ewav.ContextMenu.MenuItem Export = new Ewav.ContextMenu.MenuItem("Export DashBoard", "Images/export1.png");
                    //Ewav.ContextMenu.MenuItem Reset = new Ewav.ContextMenu.MenuItem("Reset DashBoard", "Images/reset1.png");

                    Ewav.ContextMenu.Menu subMenuGadget = new Ewav.ContextMenu.Menu();
                    Ewav.ContextMenu.Menu subMenuChart = new Ewav.ContextMenu.Menu();
                    Ewav.ContextMenu.Menu subMenuStatCalc = new Ewav.ContextMenu.Menu();
                    //mainMenu.Items.Add(subMenu);
                    //mainMenu.Items.Add(subMenu2);

                    //applicationViewModel.MefOrderDictionary.Keys.
                    List<EwavContextMenuItem> listOfItems = new List<EwavContextMenuItem>();
                    listOfItems = applicationViewModel.MefEwavContextMenuItems;
                    if (applicationViewModel.MefOrderDictionary != null)
                    {
                        listOfItems.Clear();
                        var sortedList = from q in applicationViewModel.MefOrderDictionary orderby q.Key ascending select q;
                        foreach (var item in sortedList)
                        {
                            listOfItems.Add(((EwavContextMenuItem)item.Value));
                        }
                    }


                    foreach (EwavContextMenuItem ewavMenuItem in listOfItems)
                    {
                        Ewav.ContextMenu.MenuItem gadgetItem = new ContextMenu.MenuItem(ewavMenuItem); //   , "", 1);


                        gadgetItem.MouseLeftButtonDown += new MouseButtonEventHandler(AddGadget);

                        switch (ewavMenuItem.Type)
                        {
                            case "chart":
                                subMenuChart.Items.Add(gadgetItem);

                                break;
                            case "gadget":
                                subMenuGadget.Items.Add(gadgetItem);
                                break;
                            case "statcalc":
                                subMenuStatCalc.Items.Add(gadgetItem);
                                break;
                            default:
                                break;
                        }


                    }

                    //foreach (Lazy<UserControl, IMapMetaData> gadgetUC in applicationViewModel.MefGetGadgets())
                    //{
                    //    UserControl gadgetU = (UserControl)gadgetUC.Value;
                    //    IEwavGadget uc = gadgetU as IEwavGadget;
                    //    gadgetU.Name = uc.MyControlName;


                    //    Ewav.ContextMenu.MenuItem gadgetItem = new ContextMenu.MenuItem(uc.MyUIName, "");
                    //    gadgetItem.Name = uc.MyControlName;

                    //    gadgetItem.MouseLeftButtonDown += new MouseButtonEventHandler(AddGadget);

                    //    if (gadgetUC.Metadata.type.ToString() == "chart")
                    //    {
                    //        subMenuChart.Items.Add(gadgetItem);
                    //    }
                    //    else
                    //    {
                    //        subMenuGadget.Items.Add(gadgetItem);
                    //    }


                    //}



                    //applicationViewModel.MefCleanup();

                    //Ewav.ContextMenu.Menu subMenu = new Ewav.ContextMenu.Menu();
                    //Ewav.ContextMenu.MenuItem freqGadget = new ContextMenu.MenuItem("Frequency", "");
                    //freqGadget.MouseLeftButtonDown += new MouseButtonEventHandler(AddGadget);
                    //Ewav.ContextMenu.MenuItem meansGadget = new ContextMenu.MenuItem("Means", "");
                    //meansGadget.MouseLeftButtonDown += new MouseButtonEventHandler(AddGadget);
                    //Ewav.ContextMenu.MenuItem logRegGadget = new ContextMenu.MenuItem("Logistic Regression", "");
                    //logRegGadget.MouseLeftButtonDown += new MouseButtonEventHandler(AddGadget);
                    //Ewav.ContextMenu.MenuItem linRegGadget = new ContextMenu.MenuItem("Linear Regression", "");
                    //linRegGadget.MouseLeftButtonDown += new MouseButtonEventHandler(AddGadget);
                    //Ewav.ContextMenu.MenuItem twoxtwoGadget = new ContextMenu.MenuItem("Add 2X2 Control", "");
                    //twoxtwoGadget.MouseLeftButtonDown += new MouseButtonEventHandler(AddGadget);
                    //Ewav.ContextMenu.MenuItem mxnGadget = new ContextMenu.MenuItem("Add 2X2 Control", "");
                    //mxnGadget.MouseLeftButtonDown += new MouseButtonEventHandler(AddGadget);

                    //subMenu.Items.Add(freqGadget);
                    //subMenu.Items.Add(meansGadget);
                    //subMenu.Items.Add(logRegGadget);
                    //subMenu.Items.Add(linRegGadget);
                    //subMenu.Items.Add(twoxtwoGadget);
                    //subMenu.Items.Add(mxnGadget);  

                    Gadget.Content = subMenuGadget;
                    Chart.Content = subMenuChart;
                    StatCalc.Content = subMenuStatCalc;

                    foreach (Ewav.ContextMenu.MenuItem item in popupMenu.Items)
                    {
                        if (item.Text == "Add Gadgets")
                        {
                            popupMenu.Items.Remove(item);
                        }
                    }

                    popupMenu.Items.Add(GadgetSel);
                    popupMenu.Items.Add(Gadget);
                    popupMenu.Items.Add(Chart);
                    popupMenu.Items.Add(MD);
                    popupMenu.Items.Add(StatCalc);
                    //popupMenu.Items.Add(Save);
                    //popupMenu.Items.Add(Export);
                    //popupMenu.Items.Add(Reset);
                    break;
            }

            PopupMenu = popupMenu;

            return popupMenu;
        }

        void MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            HidePopuMenu();
        }

        void munSaveAsImage_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            ((IChartControl)SelectedGadget).SaveAsImage();
            root.Children.Remove((UIElement)((Ewav.ContextMenu.MenuItem)sender).ParentMenu);
            HidePopuMenu();
        }

        void mnuSetChartTitle_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            ((IChartControl)SelectedGadget).SetChartLabels();
            root.Children.Remove((UIElement)((Ewav.ContextMenu.MenuItem)sender).ParentMenu);
            HidePopuMenu();
        }

        public void SetChartName()
        {
            //((IChartControl)SelectedGadget).MyControlName = "";

        }

        void Gadget_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            HidePopuMenu();
        }

        #endregion

        void AddGadget(object sender, MouseButtonEventArgs e)
        {

            Ewav.ContextMenu.MenuItem thisMenuItem = sender as Ewav.ContextMenu.MenuItem;

            //const string gNamespace = "Ewav";

            GadgetManager gm = new GadgetManager();
            UserControl uc = gm.LoadGadget(thisMenuItem.ControlID);

            double mouseHorizontalPosition, mouseVerticalPosition;

            uc.Name = ControlNameGenerator(uc);


            mouseVerticalPosition = contextMenuRightClickCoords.Y - 55;                  // e.GetPosition(null).Y;
            mouseHorizontalPosition = contextMenuRightClickCoords.X;     //   e.GetPosition(null).X;

            if (uc is XYChartControl)
            {
               // ((XYChartControl)uc).ChartName = ((Ewav.ContextMenu.MenuItem)(sender)).Text.Replace(" ","");
                ((XYChartControl)uc).SetChartTitle(((Ewav.ContextMenu.MenuItem)(sender)).Text.Replace(" ", ""));
            }

            cmnClass.AddControlToCanvas(uc, mouseVerticalPosition, mouseHorizontalPosition, applicationViewModel.LayoutRoot);
            MenuPopup(this, new EventArgs());
            //UpdateZOrder(uc, true);

            //if (((Ewav.ContextMenu.MenuItem)sender).Text.ToString().ToLower().Contains("frequency"))
            //{
            //    FrequencyControl frequency = new FrequencyControl();
            //    frequency.Name = ControlNameGenerator(frequency);
            //    this.DgRoot.AddChild(frequency, LayoutRoot);
            //    UpdateZOrder(frequency, true);
            //}
            //else if (((Ewav.ContextMenu.MenuItem)sender).Text.ToString().ToLower().Contains("means"))
            //{
            //    MeansControl means = new MeansControl();
            //    means.Name = ControlNameGenerator(means);
            //    this.DgRoot.AddChild(means, LayoutRoot);
            //    UpdateZOrder(means, true);
            //}
            //else if (((Ewav.ContextMenu.MenuItem)sender).Text.ToString().ToLower().Contains("logistic"))
            //{
            //    LogisticRegression logReg = new LogisticRegression();
            //    logReg.Name = ControlNameGenerator(logReg);
            //    this.DgRoot.AddChild(logReg, LayoutRoot);
            //    UpdateZOrder(logReg, true);
            //}
            //else if (((Ewav.ContextMenu.MenuItem)sender).Text.ToString().ToLower().Contains("linear"))
            //{
            //    LinearRegression linReg = new LinearRegression();
            //    linReg.Name = ControlNameGenerator(linReg);
            //    this.DgRoot.AddChild(linReg, LayoutRoot);
            //    UpdateZOrder(linReg, true);
            //}
            //else if (((Ewav.ContextMenu.MenuItem)sender).Text.ToString().ToLower().Contains("two"))
            //{
            //    TwoxTwoTableControl twoxtwo = new TwoxTwoTableControl();
            //    twoxtwo.Name = ControlNameGenerator(twoxtwo);
            //    this.DgRoot.AddChild(twoxtwo, LayoutRoot);
            //    UpdateZOrder(twoxtwo, true);
            //}
            //else if (((Ewav.ContextMenu.MenuItem)sender).Text.ToString().ToLower().Contains("mxn"))
            //{
            //    TwoxTwoTableControl twoxtwo = new TwoxTwoTableControl();
            //    twoxtwo.Name = ControlNameGenerator(twoxtwo);
            //    this.DgRoot.AddChild(twoxtwo, LayoutRoot);
            //    UpdateZOrder(twoxtwo, true);
            //}

            root.Children.Remove(popupMenu);
        }

        private string ControlNameGenerator(UIElement element)
        {
            return cmnClass.GenerateControlName(element, DgRoot);
        }

        #region Event Handler
        /// <summary>
        /// Event Handler for Close Gadget Menu Click.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void mnuClose_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            CloseGadget();
            root.Children.Remove((UIElement)((Ewav.ContextMenu.MenuItem)sender).ParentMenu);
            HidePopuMenu();
        }

        /// <summary>
        /// Event Handler for Send To Back Menu Click.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void mnuSendToBack_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            DragCanvas.SetZIndex((UIElement)SelectedGadget, -1);
            root.Children.Remove((UIElement)((Ewav.ContextMenu.MenuItem)sender).ParentMenu);
            HidePopuMenu();
        }

        /// <summary>
        /// Event Handler for Send To HTML Menu Click.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void mnuSendDataToHTML_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            SelectedGadget.ToHTML("", 0);
            root.Children.Remove((UIElement)((Ewav.ContextMenu.MenuItem)sender).ParentMenu);
            HidePopuMenu();
        }

        /// <summary>
        /// Event Handler for Copy to Clipboard.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void mnuCopy_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            CopyToClipboard();
            HidePopuMenu();
        }

        /// <summary>
        /// hides popup menu
        /// </summary>
        public void HidePopuMenu()
        {
            root.Children.Remove(popupMenu);
        }

        #endregion

        #region Helper Methods

        /// <summary>
        /// Copies a grid's output to the clipboard
        /// </summary>
        private void CopyToClipboard()
        {
            if (ControlName.ToString().ToLower().Contains("two"))
            {
                StringBuilder sb = new StringBuilder();

                sb.AppendLine(Gadgetparameters.MainVariableName + " by " + Gadgetparameters.CrosstabVariableName);
                sb.AppendLine();
                sb.AppendLine("\t" + "Yes" + "\t" + "No" + "\tTotal");
                sb.AppendLine("Yes" + "\t" + GridCells.YyVal.ToString() + "\t" + GridCells.YnVal.ToString() + "\t" + GridCells.YtVal.ToString());
                sb.AppendLine("No" + "\t" + GridCells.NyVal.ToString() + "\t" + GridCells.NnVal.ToString() + "\t" + GridCells.NtVal.ToString());
                sb.AppendLine("Total\t" + GridCells.YtVal.ToString() + "\t" + GridCells.TnVal.ToString() + "\t" + GridCells.TtVal.ToString());
                sb.AppendLine();

                sb.AppendLine("(Exposure = Rows; Outcome = Columns)");

                //Clipboard.Clear();
                Clipboard.SetText(sb.ToString());
            }
            else
            {
                StringBuilder sb = new StringBuilder();
                foreach (Grid grid in this.strataGridList)
                {
                    string gridName = grid.Tag.ToString();

                    if (strataGridList.Count > 1)
                    {
                        sb.AppendLine(grid.Tag.ToString());
                    }

                    foreach (UIElement control in grid.Children)
                    {
                        if (control is TextBlock)
                        {
                            int columnNumber = Grid.GetColumn((FrameworkElement)control);
                            string value = ((TextBlock)control).Text;

                            sb.Append(value + "\t");

                            if (columnNumber >= grid.ColumnDefinitions.Count - 2)
                            {
                                sb.AppendLine();
                            }
                        }
                    }

                    sb.AppendLine();
                }

                Clipboard.SetText(sb.ToString());
            }
        }

        /// <summary>
        /// Closes the gadget
        /// </summary>
        private void CloseGadget()
        {
            SelectedGadget.CloseGadget();
            //root.Children.Remove((UIElement)(Root.FindName(ControlName)));
            HidePopuMenu();
        }

        #endregion

        /// <summary>
        /// Updates the Zorder
        /// </summary>
        /// <param name="element"></param>
        /// <param name="bringToFront"></param>
        private void UpdateZOrder(UIElement element, bool bringToFront)
        {
            cmnClass.UpdateZOrder(element, bringToFront, applicationViewModel.LayoutRoot);
        }
    }
}