﻿/*  ----------------------------------------------------------------------------
 *  Emergint Technologies, Inc.
 *  ----------------------------------------------------------------------------
 *  Epi Info™ - Web Analysis & Visualization
 *  ----------------------------------------------------------------------------
 *  File:       CanvasServiceAgent.cs
 *  Namespace:  Ewav.Services    
 *
 *  Author(s):  Daniel Shorter, Mohammad Usman, Ninad Date, Sachin Agnihotri 
 *  Created:    26/08/2013    
 *  Summary:	no summary     
 *  ----------------------------------------------------------------------------
 */
using System;
using System.Collections.Generic;
using Ewav.Web.Services;
using System.ServiceModel.DomainServices.Client;
using EpiDashboard;
using Ewav.BAL;
using Ewav.ViewModels;
using Ewav.Web.Services.CanvasDomainService;


namespace Ewav.Services
{
    /// <summary>
    /// Service agent for Ewav external data sources.  
    /// </summary>
    public class CanvasServiceAgent : ICanvasServiceAgent
    {
        #region Variables
        CanvasDomainContext canvasCtx;
        private Action<int, Exception> _completed;
        private Action<DatatableBag, Exception> _userListLoadCompleted;
        private Action<CanvasDto, Exception> _loadCompleted;
        private Action<bool, Exception> _shareCompleted;
        private Action<DatatableBag, Exception> _readUsersCompleted;
        private Action<bool, Exception> _deleteCompleted;
        #endregion

        #region Constructor
        public CanvasServiceAgent()
        {
        }
        #endregion

        #region Helper Method

        #endregion

        #region Completion Callbacks

        #endregion

        public void SaveCanvas(CanvasDto canvasDto, Action<int, Exception> completed)
        {
            _completed = completed;
            canvasCtx = new CanvasDomainContext();

            InvokeOperation<int> canvasResults =  canvasCtx.SaveCanvas(canvasDto);

            canvasResults.Completed += new EventHandler(canvasResults_Completed);

        }

        public void LoadCanvasForUserList(int UserId, Action<DatatableBag, Exception> completed)
        {
            _userListLoadCompleted = completed;
            canvasCtx = new CanvasDomainContext();
            InvokeOperation<DatatableBag> results = canvasCtx.LoadCanvasListForUser(UserId);

            results.Completed += new EventHandler(results_Completed);
        }

        public void ReadAllUsersInMyOrg(int orgId, Action<DatatableBag, Exception> completed)
        {
            _readUsersCompleted = completed;
            canvasCtx = new CanvasDomainContext();
            InvokeOperation<DatatableBag> readResults = canvasCtx.ReadAllUsersInMyOrg(orgId);

            readResults.Completed += new EventHandler(readResults_Completed);
        }

        void readResults_Completed(object sender, EventArgs e)
        {
            InvokeOperation<DatatableBag> result =
                (InvokeOperation<DatatableBag>)sender;
            Exception ex = null;

            if (result.HasError)
            {
                result.MarkErrorAsHandled();
                ex = result.Error;
            }

            DatatableBag returnedData = ((InvokeOperation<DatatableBag>)sender).Value;
            _readUsersCompleted(returnedData, ex);
        }

        public void LoadCanvas(int canvasId, Action<CanvasDto, Exception> completed)
        {
            _loadCompleted = completed;
            canvasCtx = new CanvasDomainContext();

            InvokeOperation<CanvasDto> loadResults = canvasCtx.LoadCanvas(canvasId);
            loadResults.Completed += new EventHandler(loadResults_Completed);
        }

        public void ShareCanvas(int canvasId, List<int> sharedUserIds, Action<bool, Exception> completed)
        {
            _shareCompleted = completed;
            canvasCtx = new CanvasDomainContext();

            InvokeOperation<bool> shareResults = canvasCtx.ShareCanvas(canvasId, sharedUserIds);
            shareResults.Completed += new EventHandler(shareResults_Completed);
        }


        public void DeleteCanvas(int canvasId, Action<bool, Exception> completed)
        {
            _deleteCompleted = completed;
            canvasCtx = new CanvasDomainContext();

            InvokeOperation<bool> deleteResults = canvasCtx.DeleteCanvas(canvasId);
            deleteResults.Completed += new EventHandler(deleteResults_Completed);
        }

        void deleteResults_Completed(object sender, EventArgs e)
        {
            InvokeOperation<bool> result =
                 (InvokeOperation<bool>)sender;
            Exception ex = null;

            if (result.HasError)
            {
                result.MarkErrorAsHandled();
                ex = result.Error;
            }

            bool returnedData = ((InvokeOperation<bool>)sender).Value;
            _deleteCompleted(returnedData, ex);
        }

        void shareResults_Completed(object sender, EventArgs e)
        {
            InvokeOperation<bool> result =
                (InvokeOperation<bool>)sender;
            Exception ex = null;

            if (result.HasError)
            {
                result.MarkErrorAsHandled();
                ex = result.Error;
            }

            bool returnedData = ((InvokeOperation<bool>)sender).Value;
            _shareCompleted(returnedData, ex);
        }


        void loadResults_Completed(object sender, EventArgs e)
        {
            InvokeOperation<CanvasDto> result =
                (InvokeOperation<CanvasDto>)sender;
            Exception ex = null;

            if (result.HasError)
            {
                result.MarkErrorAsHandled();
                ex = result.Error;
            }

            CanvasDto returnedData = ((InvokeOperation<CanvasDto>)sender).Value;
            _loadCompleted(returnedData, ex);
        }

        void results_Completed(object sender, EventArgs e)
        {
            InvokeOperation<DatatableBag> result =
                (InvokeOperation<DatatableBag>)sender;
            Exception ex = null;

            if (result.HasError)
            {
                result.MarkErrorAsHandled();
                ex = result.Error;
            }

            DatatableBag returnedData = ((InvokeOperation<DatatableBag>)sender).Value;
            _userListLoadCompleted(returnedData, ex);
        }

        void canvasResults_Completed(object sender, EventArgs e)
        {
            InvokeOperation<int> result =
               (InvokeOperation<int>)sender;

            Exception ex = null;
            if (result.HasError)
            {
                result.MarkErrorAsHandled();
                //throw new GadgetException(result.Error.Message);
                ex = result.Error;
            }
            //else
            //{
            int returnedData = ((InvokeOperation<int>)sender).Value;
            _completed(returnedData, ex);
            //}
        }







    }
}