﻿/*  ----------------------------------------------------------------------------
 *  Emergint Technologies, Inc.
 *  ----------------------------------------------------------------------------
 *  Epi Info™ - Web Analysis & Visualization
 *  ----------------------------------------------------------------------------
 *  File:       Scatter.xaml.cs
 *  Namespace:  Ewav    
 *
 *  Author(s):  Daniel Shorter, Mohammad Usman, Ninad Date, Sachin Agnihotri 
 *  Created:    26/08/2013    
 *  Summary:	no summary     
 *  ----------------------------------------------------------------------------
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Linq;
using ComponentArt.Silverlight.DataVisualization.Charting;
using ComponentArt.Silverlight.DataVisualization.Common;
using ComponentArt.Silverlight.Export.PDF;
using EpiDashboard;
using Ewav.BAL;
using Ewav.Client.Application;
using Ewav.ViewModels;
using Ewav.Web.Services;
using CommonLibrary;

namespace Ewav
{
    [PartCreationPolicy(CreationPolicy.NonShared)]
    [Export(typeof(UserControl))]
    [ExportMetadata("type", "chart")]
    [ExportMetadata("tabindex", "12")]
    public partial class Scatter : UserControl, IGadget, IEwavGadget, IChartControl
    {
        private readonly char[] SplitTokens = " \t;".ToCharArray();
        private ApplicationViewModel applicationViewModel = ApplicationViewModel.Instance;
        private TextBlock textBlockX, textBlockY;
        private bool isBooleanWithNoStratas = false;
        private GadgetParameters gadgetOptions;
        private object syncLock = new object();
        private object objXAxisStart;
        private object objXAxisEnd;
        private int Index1 = -1, Index2 = -1;
        EwavColumn Col1, Col2;
        DatatableBag databag;
        DashboardPanel dp;
        SetLabels window;
        ClientCommon.Common CommonClass = new ClientCommon.Common();
        string xAxisVar, yAxisVar;
        //EpiCurveViewModel eCrvViewModel;
        ScatterViewModel sctrViewModel;
        List<string> dateColumnNames = new List<string>();
        List<string> numericColumnNames = new List<string>();
        List<List<StringDataValue>> dataValues = new List<List<StringDataValue>>();
        public event GadgetClosingHandler GadgetClosing;
        public event GadgetProcessingFinishedHandler GadgetProcessingFinished;
        public event GadgetStatusUpdateHandler GadgetStatusUpdate;
        public event GadgetCheckForCancellationHandler GadgetCheckForCancellation;

        private delegate void SetStatusDelegate(string statusMessage);
        private delegate void RequestUpdateStatusDelegate(string statusMessage);
        private delegate bool CheckForCancellationDelegate();
        private delegate void RenderFinishWithErrorDelegate(string errorMessage);
        private delegate void RenderFinishWithWarningDelegate(string errorMessage);
        // private delegate void RenderFinishEpiCurveDelegate(DataTable data, List<List<StringDataValue>> dataValues);
        private delegate void RenderFinishSingleChartDelegate(List<List<StringDataValue>> stratifiedValues);
        //private delegate void RenderFinishScatterChartDelegate(List<NumericDataValue> dataValues, StatisticsRepository.LinearRegression.LinearRegressionResults results, NumericDataValue maxValue, NumericDataValue minValue);
        //private delegate void RenderFinishStackedChartDelegate(List<List<StringDataValue>> dataValues, DataTable data);
        private delegate void SimpleCallback();

        public class StringDataValue
        {
            public double DependentValue { get; set; }
            public string IndependentValue { get; set; }
            public string StratificationValue { get; set; }
            public double CurrentMeanValue { get; set; }
        }

        //public class NumericDataValue
        //{
        //    public decimal DependentValue { get; set; }
        //    public decimal IndependentValue { get; set; }
        //}

        public class TypeStringTuple
        {
            public Type Item1 { get; set; }
            public string Item2 { get; set; }
            public TypeStringTuple(Type item1, string item2)
            {
                Item1 = item1;
                Item2 = item2;
            }
        }

        ClientCommon.Common cmnClass = new ClientCommon.Common();

        public List<ColumnDataType> GetFieldNumericDataType
        {
            get
            {
                List<ColumnDataType> columnDataType = new List<ColumnDataType>();
                //columnDataType.Add(ColumnDataType.DateTime);
                columnDataType.Add(ColumnDataType.Numeric);

                return columnDataType;
            }
        }

        /// <summary>
        /// Default constructor
        /// </summary>
        public Scatter()
        {
            InitializeComponent();

            gadgetOptions = new GadgetParameters();
            gadgetOptions.ShouldIncludeFullSummaryStatistics = false;
            gadgetOptions.ShouldIncludeMissing = false;
            gadgetOptions.ShouldSortHighToLow = false;
            gadgetOptions.ShouldUseAllPossibleValues = false;
            gadgetOptions.StrataVariableNames = new List<string>();

            this.Loaded += new RoutedEventHandler(Scatter_Loaded);

            FillDropDowns();
            //InitializeControl();
        }

        void Scatter_Loaded(object sender, RoutedEventArgs e)
        {
            InitializeControl();
            //this.gadgetExpander.IsExpanded = false;
        }

        private void InitializeControl()
        {
            cbxScatterXAxisField.SelectionChanged += new SelectionChangedEventHandler(ConfigField_SelectionChanged);
            cbxScatterYAxisField.SelectionChanged += new SelectionChangedEventHandler(ConfigField_SelectionChanged);
            try
            {
                DatatableBag eCrvData = new DatatableBag();
                sctrViewModel = (ScatterViewModel)this.DataContext;
                //sctrViewModel.ColumnsLoadedEvent += new EventHandler<SimpleMvvmToolkit.NotificationEventArgs<Exception>>(sctrViewModel_ColumnsLoadedEvent);
                sctrViewModel.RegressTableLoadedEvent += new EventHandler<SimpleMvvmToolkit.NotificationEventArgs<Exception>>(sctrViewModel_RegressResultsLoadedEvent);
                applicationViewModel.ConnectionStringReadyEvent += new ConnectionStringReadyEventHandler(applicationViewModel_ConnectionStringLoadedEvent);
                //sctrViewModel.GetColumns("NEDS", "vwExternalData");
                applicationViewModel.ApplyDataFilterEvent += new ApplyFilterEventHandler(applicationViewModel_ApplyDataFilterEvent);
                applicationViewModel.DefinedVariableAddedEvent += new DefinedVariableAddedEventHandler(applicationViewModel_DefinedVariableAddedEvent);
                applicationViewModel.DefinedVariableInUseDeletedEvent += new DefinedVariableInUseDeletedEventHandler(applicationViewModel_DefinedVariableInUseDeletedEvent);
                applicationViewModel.DefinedVariableNotInUseDeletedEvent += new DefinedVariableNotInUseDeletedEventHandler(applicationViewModel_DefinedVariableNotInUseDeletedEvent);
                applicationViewModel.PreColumnChangedEvent += new PreColumnChangedEventHandler(applicationViewModel_PreColumnChangedEvent);
                applicationViewModel.UnloadedEvent += new UnloadedEventHandler(applicationViewModel_UnloadedEvent);
                sctrViewModel.ErrorNotice += new EventHandler<SimpleMvvmToolkit.NotificationEventArgs<Exception>>(sctrViewModel_ErrorNotice);
                EnableDisableButton();
            }
            catch (Exception)
            {
                throw;
            }

            this.IsProcessing = false;

            this.GadgetStatusUpdate += new GadgetStatusUpdateHandler(RequestUpdateStatusMessage);
            this.GadgetCheckForCancellation += new GadgetCheckForCancellationHandler(IsCancelled);
        }

        void applicationViewModel_UnloadedEvent(object o)
        {
            UnloadGadget();
        }

        private void UnloadGadget()
        {
            applicationViewModel.ConnectionStringReadyEvent -= new ConnectionStringReadyEventHandler(applicationViewModel_ConnectionStringLoadedEvent);
            //sctrViewModel.GetColumns("NEDS", "vwExternalData");
            applicationViewModel.ApplyDataFilterEvent -= new ApplyFilterEventHandler(applicationViewModel_ApplyDataFilterEvent);
            applicationViewModel.DefinedVariableAddedEvent -= new DefinedVariableAddedEventHandler(applicationViewModel_DefinedVariableAddedEvent);
            applicationViewModel.DefinedVariableInUseDeletedEvent -= new DefinedVariableInUseDeletedEventHandler(applicationViewModel_DefinedVariableInUseDeletedEvent);
            applicationViewModel.DefinedVariableNotInUseDeletedEvent -= new DefinedVariableNotInUseDeletedEventHandler(applicationViewModel_DefinedVariableNotInUseDeletedEvent);
            applicationViewModel.PreColumnChangedEvent -= new PreColumnChangedEventHandler(applicationViewModel_PreColumnChangedEvent);
        }

        void applicationViewModel_PreColumnChangedEvent(object o)
        {
            SaveColumnValues();
        }

        void applicationViewModel_DefinedVariableNotInUseDeletedEvent(object o)
        {
            SearchIndex();
            LoadingDropDowns = true;
            FillDropDowns();
            LoadingDropDowns = false;
            DoScatter();
        }

        void applicationViewModel_DefinedVariableInUseDeletedEvent(object o)
        {
            ResetGadget();
        }

        private void ResetGadget()
        {
            SearchIndex();
            if (Col1 != null && Col1.Name == applicationViewModel.ItemToBeRemoved.Name ||
                Col2 != null && Col2.Name == applicationViewModel.ItemToBeRemoved.Name)
            {
                Index1 = Index2 = -1;
                pnlChartContainer.Visibility = System.Windows.Visibility.Collapsed;
                pnlStatus.Visibility = System.Windows.Visibility.Collapsed;
                this.cbxScatterXAxisField.ItemsSource = null;
                this.cbxScatterYAxisField.ItemsSource = null;
            }
            LoadingDropDowns = true;
            FillDropDowns();
            LoadingDropDowns = false;
        }

        void applicationViewModel_DefinedVariableAddedEvent(object o)
        {
            SearchIndex();
            LoadingDropDowns = true;
            FillDropDowns();
            LoadingDropDowns = false;
            DoScatter();
        }

        private void SearchIndex()
        {
            Index1 = CommonClass.SearchCurrentIndex(Col1, CommonClass.GetItemsSource(GetFieldNumericDataType));
            Index2 = CommonClass.SearchCurrentIndex(Col2, CommonClass.GetItemsSource(GetFieldNumericDataType));
        }

        /// <summary>
        /// Saves the Values of Columns.
        /// </summary>
        private void SaveColumnValues()
        {
            Col1 = (EwavColumn)cbxScatterXAxisField.SelectedItem;
            Col2 = (EwavColumn)cbxScatterYAxisField.SelectedItem;
        }

        /// <summary>
        /// Method used to locate the current index for selected column.
        /// </summary>
        /// <param name="Column"></param>
        /// <returns></returns>
        //private int SearchCurrentIndex(EwavColumn Column)
        //{
        //    if (Column != null)
        //    {
        //        List<EwavColumn> SourceColumns = this.applicationViewModel.EwavSelectedDatasource.AllColumns;

        //        IEnumerable<EwavColumn> CBXFieldCols = from cols in SourceColumns
        //                                               orderby cols.Name
        //                                               select cols;

        //        List<EwavColumn> colsList = CBXFieldCols.ToList();

        //        for (int i = 0; i < colsList.Count; i++)
        //        {
        //            if (Column.Name == colsList[i].Name)
        //            {
        //                return i + 1;
        //            }
        //        }
        //    }

        //    return 0;
        //}

        void sctrViewModel_ErrorNotice(object sender, SimpleMvvmToolkit.NotificationEventArgs<Exception> e)
        {
            if (e.Data.Message.Length > 0)
            {
                ChildWindow window = new ErrorWindow(e.Data);
                window.Show();
                //return;
            }
            RenderFinishWithError(e.Data.Message);
            this.SetGadgetToFinishedState();
        }

        private void DoScatter()
        {
            RefreshResults();
            bool byEpiWeek = false;

            if (gadgetOptions.InputVariableList == null)
            {
                return;
            }


            if (gadgetOptions.InputVariableList.ContainsKey("isdatecolumnnumeric"))
            {
                byEpiWeek = bool.Parse(gadgetOptions.InputVariableList["isdatecolumnnumeric"]);
            }
            sctrViewModel = (ScatterViewModel)this.DataContext;
            //eCrvViewModel.GetEpiCurve(gadgetOptions, byEpiWeek, gadgetOptions.MainVariableName, gadgetOptions.CrosstabVariableName);
            //eCrvViewModel.
            //epiCurveWorker_DoWork(dtb, gadgetOptions);
            //if (Index1 > 0 && Index2 > 0)
            //{
            xAxisVar = gadgetOptions.MainVariableName;
            yAxisVar = gadgetOptions.CrosstabVariableName;

            List<MyString> columnNames = new List<MyString>();
            //columnNames.Add(new MyString().VarNamexAxisVar);
            //columnNames.Add(yAxisVar);
            gadgetOptions.UseAdvancedDataFilter = applicationViewModel.UseAdvancedFilter;
            sctrViewModel.GenerateTable(gadgetOptions, columnNames);
            //}

        }

        void applicationViewModel_ApplyDataFilterEvent(object o)
        {
            if (this.DataContext != null)
            {
                DoScatter();
            }
        }

        /// <summary>
        /// method used to enabled disable Generate Button
        /// </summary>
        private void EnableDisableButton()
        {
            if (cbxScatterXAxisField.SelectedIndex > 0 && cbxScatterYAxisField.SelectedIndex > 0)
            {
                btnRun.IsEnabled = true;
            }
            else
            {
                btnRun.IsEnabled = false;
            }
        }

        /// <summary>
        /// application connectionstring loaded event.
        /// </summary>
        /// <param name="o"></param>
        /// <param name="e"></param>
        void applicationViewModel_ConnectionStringLoadedEvent(object o, ConnectionStringEventArgs e)
        {
            // reset tool                   
            gadgetOptions.TableName = applicationViewModel.EwavSelectedDatasource.TableName;
            gadgetOptions.DatasourceName = applicationViewModel.EwavSelectedDatasource.DatasourceName;
        }

        /// <summary>
        /// scatter view model table loaded event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void sctrViewModel_RegressResultsLoadedEvent(object sender, SimpleMvvmToolkit.NotificationEventArgs<Exception> e)
        {
            scatterChartWorker_DoWork();
            this.gadgetExpander.IsExpanded = false;
        }

        /// <summary>
        /// Loads Columns
        /// </summary>
        void FillDropDowns()//object sender, SimpleMvvmToolkit.NotificationEventArgs<Exception> e)
        {
            //List<EwavColumn> SourceColumns =
            //    applicationViewModel.EwavSelectedDatasource.AllColumns;

            //IEnumerable<EwavColumn> CBXFieldCols = from cols in SourceColumns
            //                                       where GetFieldNumericDataType.Contains(cols.SqlDataTypeAsString)
            //                                       orderby cols.Name
            //                                       select cols;

            List<EwavColumn> colsList = CommonClass.GetItemsSource(GetFieldNumericDataType); ;  // CBXFieldCols.ToList();

            //colsList.Insert(0, new EwavColumn() { Name = " ", Index = -1 });
            cbxScatterXAxisField.ItemsSource = colsList;
            cbxScatterXAxisField.SelectedValue = "Index";
            cbxScatterXAxisField.DisplayMemberPath = "Name";
            cbxScatterXAxisField.SelectedIndex = Index1;

            foreach (var item in colsList)
            {
                dateColumnNames.Add(item.Name);
            }

            //columnDataType.Clear();

            //columnDataType.Add(ColumnDataType.Numeric);
            //columnDataType.Add(ColumnDataType.Boolean);
            //columnDataType.Add(ColumnDataType.Text);

            //CBXFieldCols = from cols in SourceColumns
            //               where GetFieldNumericDataType.Contains(cols.SqlDataTypeAsString)
            //               orderby cols.Name
            //               select cols;
            //List<EwavColumn> CaseStatusField = CBXFieldCols.ToList();
            //ewc = new EwavColumn();
            //ewc.Name = " ";
            //dateFields.Insert(0, ewc);
            //CaseStatusField.Insert(0, new EwavColumn() { Name = " ", Index = -1 });
            cbxScatterYAxisField.ItemsSource = colsList; // CaseStatusField;
            cbxScatterYAxisField.SelectedValue = "Index";
            cbxScatterYAxisField.DisplayMemberPath = "Name";
            cbxScatterYAxisField.SelectedIndex = Index2;

            foreach (var item in colsList)
            {
                numericColumnNames.Add(item.Name);
            }
        }

        /// <summary>
        /// Closes the gadget
        /// </summary>
        //private void CloseGadget()
        //{
        //    //if (worker != null && worker.WorkerSupportsCancellation)
        //    //{
        //    //    worker.CancelAsync();
        //    //}
        //    this.GadgetStatusUpdate -= new GadgetStatusUpdateHandler(RequestUpdateStatusMessage);
        //    this.GadgetCheckForCancellation -= new GadgetCheckForCancellationHandler(IsCancelled);

        //    if (GadgetClosing != null)
        //        GadgetClosing(this);

        //    gadgetOptions = null;
        //    chart = null;

        //    GadgetClosing = null;
        //    GadgetCheckForCancellation = null;
        //    GadgetProcessingFinished = null;
        //    GadgetStatusUpdate = null;
        //}

        //void mnuPrint_Click(object sender, RoutedEventArgs e)
        //{
        //    Common.Print(pnlChartContainer);
        //}

        //void imgClose_MouseDown(object sender, MouseButtonEventArgs e)
        //{
        //    CloseGadget();
        //}

        //void imgClose_MouseLeave(object sender, MouseEventArgs e)
        //{
        //    Uri uriSource = new Uri("Images/x.png", UriKind.Relative);
        //    imgClose.Source = new BitmapImage(uriSource);
        //}

        //void imgClose_MouseEnter(object sender, MouseEventArgs e)
        //{
        //    Uri uriSource = new Uri("Images/x_over.png", UriKind.Relative);
        //    imgClose.Source = new BitmapImage(uriSource);
        //}

        //void mnuLabel_Click(object sender, RoutedEventArgs e)
        //{
        //    if (pnlChartContainer.Children.Count > 0 && pnlChartContainer.Children[0] is Chart)
        //    {
        //        Chart currentChart = (Chart)pnlChartContainer.Children[0];

        //        ChartLabelDialog dlg = new ChartLabelDialog();
        //        if (currentChart.Title != null)
        //        {
        //            dlg.Title = currentChart.Title.ToString();
        //        }

        //        if (currentChart.LegendTitle != null)
        //        {
        //            dlg.LegendTitle = currentChart.LegendTitle.ToString();
        //        }

        //        if (currentChart.ActualAxes.Count > 0)
        //        {
        //            if (((DisplayAxis)currentChart.ActualAxes[0]).Orientation == AxisOrientation.Y)
        //            {
        //                if (((DisplayAxis)currentChart.ActualAxes[0]).Title != null)
        //                {
        //                    dlg.YAxisLabel = ((DisplayAxis)currentChart.ActualAxes[0]).Title.ToString();
        //                }
        //                if (((DisplayAxis)currentChart.ActualAxes[1]).Title != null)
        //                {
        //                    dlg.XAxisLabel = ((DisplayAxis)currentChart.ActualAxes[1]).Title.ToString();
        //                }
        //            }
        //            else
        //            {
        //                if (((DisplayAxis)currentChart.ActualAxes[0]).Title != null)
        //                {
        //                    dlg.XAxisLabel = ((DisplayAxis)currentChart.ActualAxes[0]).Title.ToString();
        //                }
        //                if (((DisplayAxis)currentChart.ActualAxes[1]).Title != null)
        //                {
        //                    dlg.YAxisLabel = ((DisplayAxis)currentChart.ActualAxes[1]).Title.ToString();
        //                }
        //            }
        //        }

        //        if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
        //        {
        //            SetChartLabels(dlg.Title, dlg.LegendTitle, dlg.XAxisLabel, dlg.YAxisLabel);
        //            //if (currentChart.ActualAxes.Count > 0)
        //            //{
        //            //    ((DisplayAxis)currentChart.ActualAxes[0]).Title = dlg.YAxisLabel;
        //            //    ((DisplayAxis)currentChart.ActualAxes[1]).Title = dlg.XAxisLabel;
        //            //}
        //            //currentChart.Title = dlg.Title;
        //        }
        //    }
        //}

        /// <summary>
        /// Wires the loaded and unloaded event for the popup window.
        /// </summary>
        public void SetChartLabels()
        {
            window = new SetLabels { DataContext = this.DataContext };
            window.Loaded -= new RoutedEventHandler(window_Loaded);
            window.Loaded += new RoutedEventHandler(window_Loaded);
            window.Unloaded -= new RoutedEventHandler(window_Unloaded);
            window.Unloaded += new RoutedEventHandler(window_Unloaded);
            window.Show();
        }

        /// <summary>
        /// Loaded event for window.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void window_Loaded(object sender, RoutedEventArgs e)
        {
            //window.txtboxXaxis.Text = textBlockX.Text;
            //window.txtboxYaxis.Text = textBlockY.Text;
            //if (dp.Title != null)
            //{
            //    window.txtbxChrtTitle.Text = (string)dp.Title;
            //}
        }

        /// <summary>
        /// Sets Chart Labels
        /// </summary>
        /// <param name="chartTitle"></param>
        /// <param name="legendTitle"></param>
        /// <param name="xAxisLabel"></param>
        /// <param name="yAxisLabel"></param>
        public void SetChartLabels(string chartTitle, string legendTitle, string xAxisLabel, string yAxisLabel)
        {
            //var window = new SetLabels { DataContext = this.DataContext };
            //window.Unloaded += new RoutedEventHandler(window_Unloaded);
            //window.Unloaded += new RoutedEventHandler(window_Unloaded);
            //window.Show();
            //ExportToPdf etp = new ExportToPdf();
            //ExportToPDF etp = new ExportToPDF();
            //etp.SavePNG(current)
            //if (pnlChartContainer.Children.Count > 0 && pnlChartContainer.Children[0] is XYChart)
            //{
            //XYChart currentChart = (XYChart)pnlChartContainer.Children[0];
            //etp.SavePNG(currentChart, 600);
            //if (currentChart.Axes.Count > 0)
            //{
            //if (((DisplayAxis)currentChart.Axes[0]).Orientation == AxisOrientation.Y)
            //{
            //    ((DisplayAxis)currentChart.Axes[0]).Title = yAxisLabel;
            //    ((DisplayAxis)currentChart.Axes[1]).Title = xAxisLabel;
            //}
            //else
            //{
            //    ((DisplayAxis)currentChart.ActualAxes[0]).Title = xAxisLabel;
            //    ((DisplayAxis)currentChart.ActualAxes[1]).Title = yAxisLabel;
            //}
            //YAxisLabel = yAxisLabel;
            // XAxisLabel = xAxisLabel;
            //}
            //currentChart.= chartTitle;
            //ChartTitle = chartTitle
            //if (!string.IsNullOrEmpty(legendTitle))
            //{
            //    currentChart.Legend = new Legend().co
            //    LegendTitle = legendTitle;
            //}
            //else
            //{
            //    currentChart.LegendTitle = null;
            //    LegendTitle = string.Empty;
            //}
            //}
        }

        /// <summary>
        /// unloaded event for window
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void window_Unloaded(object sender, RoutedEventArgs e)
        {
            if (window.DialogResult.Value)
            {
                SetValuesForAxis();
            }
        }

        /// <summary>
        /// Sets the value for labels
        /// </summary>
        private void SetValuesForAxis()
        {
            if (textBlockX != null && textBlockY != null && dp != null)
            {
                textBlockX.Text = window.txtboxXaxis.Text;
                textBlockY.Text = window.txtboxYaxis.Text;
                dp.Title = window.txtbxChrtTitle.Text;
            }
        }

        /// <summary>
        /// Saves chart as an image
        /// </summary>
        public void SaveAsImage()
        {
            ExportToPDF etp = new ExportToPDF();
            // XYChart currentChart = (XYChart)dp.Content;// (XYChart)pnlChartContainer.Children[0];//
            DashboardPanel currentChart = dp;
            etp.SavePNG(currentChart, 200);
        }

        //void mnuSave_Click(object sender, RoutedEventArgs e)
        //{
        //    Common.SaveAsImage(pnlChartContainer);
        //}

        //void mnuCopy_Click(object sender, RoutedEventArgs e)
        //{
        //    if (pnlChartContainer.Children.Count > 0)
        //    {
        //        Chart currentChart = (Chart)pnlChartContainer.Children[0];
        //        BitmapSource img = (BitmapSource)Common.ToImageSource(pnlChartContainer);
        //        PngBitmapEncoder encoder = new PngBitmapEncoder();
        //        encoder.Frames.Add(BitmapFrame.Create(img));
        //        Clipboard.SetImage(encoder.Frames[0]);
        //    }
        //}

        //void mnuSendToBack_Click(object sender, RoutedEventArgs e)
        //{
        //    SendToBack();
        //}

        //void mnuRefresh_Click(object sender, RoutedEventArgs e)
        //{
        //    RefreshResults();
        //}

        //void mnuClose_Click(object sender, RoutedEventArgs e)
        //{
        //    CloseGadget();
        //}

        void cbxChartType_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                string chartType = ((ComboBoxItem)e.AddedItems[0]).Content.ToString();
                switch (chartType)
                {
                    //case "Stacked Column":
                    //    pnlScatterConfig.Visibility = Visibility.Collapsed;
                    //    pnlStackedColumnConfig.Visibility = Visibility.Visible;
                    //    pnlEpiCurveConfig.Visibility = Visibility.Collapsed;
                    //    pnlSingleConfig.Visibility = Visibility.Collapsed;
                    //    break;
                    //case "Epi Curve":
                    //    //pnlScatterConfig.Visibility = Visibility.Collapsed;
                    //    //pnlStackedColumnConfig.Visibility = Visibility.Collapsed;
                    //    pnlEpiCurveConfig.Visibility = Visibility.Visible;
                    //    //pnlSingleConfig.Visibility = Visibility.Collapsed;
                    //    break;
                    case "Scatter":
                        pnlScatterConfig.Visibility = Visibility.Visible;
                        //pnlStackedColumnConfig.Visibility = Visibility.Collapsed;
                        //pnlEpiCurveConfig.Visibility = Visibility.Collapsed;
                        pnlSingleConfig.Visibility = Visibility.Collapsed;
                        break;
                    default:
                        //pnlScatterConfig.Visibility = Visibility.Collapsed;
                        //pnlStackedColumnConfig.Visibility = Visibility.Collapsed;
                        //pnlEpiCurveConfig.Visibility = Visibility.Collapsed;
                        //pnlSingleConfig.Visibility = Visibility.Visible;
                        //if (chartType.ToLower().Equals("pie") || chartType.ToLower().Equals("pareto"))
                        //{
                        //    cbxStrataField.Visibility = System.Windows.Visibility.Collapsed;
                        //    tblockStrataField.Visibility = System.Windows.Visibility.Collapsed;
                        //}
                        //else
                        //{
                        //    cbxStrataField.Visibility = System.Windows.Visibility.Visible;
                        //    tblockStrataField.Visibility = System.Windows.Visibility.Visible;
                        //}
                        break;
                }
                ResetComboboxes();
            }
        }

        /// <summary>
        /// resets selection in Combo Boxes
        /// </summary>
        private void ResetComboboxes()
        {
            LoadingDropDowns = true;

            //if (cbxColumnXAxisField.Items.Count > 0)
            //    cbxColumnXAxisField.SelectedIndex = -1;
            //if (cbxColumnYAxisField.Items.Count > 0)
            //    cbxColumnYAxisField.SelectedIndex = -1;
            //if (cbxCaseStatusField.Items.Count > 0)
            //    cbxCaseStatusField.SelectedIndex = -1;
            //if (cbxDateField.Items.Count > 0)
            //    cbxDateField.SelectedIndex = -1;
            //if (cbxSingleField.Items.Count > 0)
            //    cbxSingleField.SelectedIndex = -1;
            //if (cbxWeightField.Items.Count > 0)
            //    cbxWeightField.SelectedIndex = -1;
            //if (cbxStrataField.Items.Count > 0)
            //    cbxStrataField.SelectedIndex = -1;
            if (cbxScatterXAxisField.Items.Count > 0)
            {
                cbxScatterXAxisField.SelectedIndex = -1;
            }
            if (cbxScatterYAxisField.Items.Count > 0)
            {
                cbxScatterYAxisField.SelectedIndex = -1;
            }

            LoadingDropDowns = false;
        }

        /// <summary>
        /// Config field selection changed event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ConfigField_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!LoadingDropDowns)
            {
                //bool isDropDownList = false;
                //bool isRecoded = false;
                //if (cbxChartType.Text.ToLower().Equals("column") || cbxChartType.Text.ToLower().Equals("bar"))
                //{
                //    if (cbxSingleField.SelectedItem != null && !string.IsNullOrEmpty(cbxSingleField.SelectedItem.ToString()))
                //    {
                //        foreach (DataRow fieldRow in dashboardHelper.FieldTable.Rows)
                //        {
                //            if (fieldRow["columnname"].Equals(cbxSingleField.SelectedItem.ToString()))
                //            {
                //                if (fieldRow["epifieldtype"] is TableBasedDropDownField)
                //                {
                //                    isDropDownList = true;
                //                }
                //                break;
                //            }
                //        }
                //    }
                //    if (cbxSingleField.SelectedIndex >= 0 && dashboardHelper.IsUserDefinedColumn(cbxSingleField.SelectedItem.ToString()))
                //    {
                //        List<IDashboardRule> associatedRules = dashboardHelper.Rules.GetRules(cbxSingleField.SelectedItem.ToString());
                //        foreach (IDashboardRule rule in associatedRules)
                //        {
                //            if (rule is Rule_Recode)
                //            {
                //                isRecoded = true;
                //            }
                //        }
                //    }
                //    if (isDropDownList || isRecoded)
                //    {
                //        checkboxAllValues.IsEnabled = true;
                //    }
                //    else
                //    {
                //        checkboxAllValues.IsEnabled = false;
                //        checkboxAllValues.IsChecked = false;
                //    }
                //}
                //else if (cbxChartType.Text.ToLower().Equals("epi curve"))
                //{
                //string columnName = string.Empty; //= e.AddedItems[0].ToString();
                //if (cbxDateField.SelectedIndex >= 0)
                //{
                //    columnName = cbxDateField.SelectedItem.ToString();
                //}
                //if (e.AddedItems.Count == 1 && e.AddedItems[0] is string && !string.IsNullOrEmpty(columnName))
                //{
                //    if (!dateColumnNames.Contains(columnName))
                //    {
                //        tblockDateInterval.Visibility = System.Windows.Visibility.Collapsed;
                //        cbxDateInterval.Visibility = System.Windows.Visibility.Collapsed;
                //    }
                //    else
                //    {
                //        tblockDateInterval.Visibility = System.Windows.Visibility.Visible;
                //        cbxDateInterval.Visibility = System.Windows.Visibility.Visible;
                //DateTime? minDate = null;
                //DateTime? maxDate = null;
                //dashboardHelper.FindUpperLowerDateValues(columnName, ref minDate, ref maxDate);
                //if (minDate.HasValue && maxDate.HasValue)
                //{
                //    TimeSpan timeSpan = (DateTime)maxDate - (DateTime)minDate;
                //    if (timeSpan.TotalHours <= 24)
                //    {
                //        cbxDateInterval.SelectedIndex = 1; // Hours
                //    }
                //    else if (timeSpan.TotalDays < 32)
                //    {
                //        cbxDateInterval.SelectedIndex = 0; // Days
                //    }
                //    else if (timeSpan.TotalDays <= 366)
                //    {
                //        cbxDateInterval.SelectedIndex = 2; // Months
                //    }
                //    else
                //    {
                //cbxDateInterval.SelectedIndex = 3; // Years
                // }
                // }
                //    }
                //}
                //}
                //else
                //{
                //    checkboxAllValues.IsChecked = false;
                //}
                //RefreshResults();
            }
            EnableDisableButton();
        }

        //private void GenerateStackedColumnChart(object sender, System.ComponentModel.DoWorkEventArgs e)
        //{
        //    if (worker != null && worker.WorkerSupportsCancellation)
        //    {
        //        worker.CancelAsync();
        //    }

        //    lock (syncLock)
        //    {
        //        worker = new System.ComponentModel.BackgroundWorker();
        //        worker.DoWork += new System.ComponentModel.DoWorkEventHandler(stackedChartWorker_DoWork);
        //        worker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(stackedChartWorker_RunWorkerCompleted);
        //        worker.RunWorkerAsync();
        //    }
        //}

        //void stackedChartWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        //{
        //    if (e.Result != null)
        //    {
        //        List<List<StringDataValue>> dataValues = (List<List<StringDataValue>>)((object[])e.Result)[0];
        //        DataTable data = (DataTable)((object[])e.Result)[1];
        //        this.Dispatcher.BeginInvoke(new RenderFinishStackedChartDelegate(RenderFinishStackedChart), dataValues, data);
        //    }
        //}

        //private void stackedChartWorker_DoWork(object sender, System.ComponentModel.DoWorkEventArgs e)
        //{
        //    try
        //    {
        //        this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToProcessingState));
        //        this.Dispatcher.BeginInvoke(new SimpleCallback(ClearResults));

        //        RequestUpdateStatusDelegate requestUpdateStatus = new RequestUpdateStatusDelegate(RequestUpdateStatusMessage);
        //        CheckForCancellationDelegate checkForCancellation = new CheckForCancellationDelegate(IsCancelled);

        //        gadgetOptions.GadgetStatusUpdate += new GadgetStatusUpdateHandler(requestUpdateStatus);
        //        gadgetOptions.GadgetCheckForCancellation += new GadgetCheckForCancellationHandler(checkForCancellation);

        //        DataTable data = GetStackedColumnData();
        //        if (data == null || data.Rows.Count == 0)
        //        {
        //            this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), SharedStrings.NO_RECORDS_SELECTED);
        //            this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
        //            return;
        //        }
        //        else if (worker.CancellationPending)
        //        {
        //            this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), SharedStrings.DASHBOARD_GADGET_STATUS_OPERATION_CANCELLED);
        //            this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
        //            System.Diagnostics.Debug.Print("Stacked chart thread was cancelled");
        //            return;
        //        }
        //        else if (data != null && data.Rows.Count > 0)
        //        {
        //            DataRow[] allRows;

        //            if (dashboardHelper.IsUsingEpiProject && data.Rows[0][0].ToString().Equals(dashboardHelper.Config.Settings.RepresentationOfYes) && (dashboardHelper.IsColumnYesNo(data.Columns[0].ColumnName) || dashboardHelper.IsColumnBoolean(data.Columns[0].ColumnName)))
        //            {
        //                allRows = data.Select(string.Empty, "[" + data.Columns[0].ColumnName + "] DESC");
        //            }
        //            else
        //            {
        //                allRows = data.Select(string.Empty, "[" + data.Columns[0].ColumnName + "]");
        //            }

        //            List<List<StringDataValue>> dataValues = new List<List<StringDataValue>>();
        //            for (int rule = 1; rule < data.Columns.Count; rule++)
        //            {
        //                List<StringDataValue> values = new List<StringDataValue>();
        //                foreach (DataRow row in allRows)
        //                {
        //                    StringDataValue val = new StringDataValue();
        //                    if (row[rule] != DBNull.Value)
        //                    {
        //                        val.DependentValue = (int)row[rule];
        //                    }
        //                    else
        //                    {
        //                        val.DependentValue = 0;
        //                    }
        //                    val.IndependentValue = row[0].ToString();
        //                    values.Add(val);
        //                }
        //                dataValues.Add(values);
        //            }

        //            object[] result = new object[] { dataValues, data };
        //            e.Result = result;

        //            this.Dispatcher.BeginInvoke(new SimpleCallback(RenderFinish));
        //            this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), ex.Message);
        //        this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
        //    }
        //}

        //private void RenderFinishStackedChart(List<List<StringDataValue>> dataValues, DataTable data)
        //{
        //    StackedColumnSeries stackedSeries;
        //    if (((ComboBoxItem)cbxColumnAggregateFunc.SelectedItem).Content.ToString().Equals("Count %"))
        //        stackedSeries = new Stacked100ColumnSeries();
        //    else
        //        stackedSeries = new StackedColumnSeries();

        //    int counter = 1;
        //    foreach (var values in dataValues)
        //    {
        //        SeriesDefinition definition = new SeriesDefinition();
        //        definition.DependentValuePath = "DependentValue";
        //        definition.IndependentValuePath = "IndependentValue";
        //        if (dashboardHelper.IsUsingEpiProject && (View.Fields[cbxColumnYAxisField.SelectedItem.ToString()] is YesNoField || View.Fields[cbxColumnYAxisField.SelectedItem.ToString()] is CheckBoxField))
        //        {
        //            if (data.Columns[counter].ColumnName.Equals(dashboardHelper.Config.Settings.RepresentationOfYes))
        //                definition.Title = cbxColumnYAxisField.SelectedItem.ToString();
        //            else
        //                definition.Title = "Not " + cbxColumnYAxisField.SelectedItem.ToString();
        //        }
        //        else
        //            definition.Title = data.Columns[counter].ColumnName;
        //        definition.ItemsSource = values;
        //        stackedSeries.SeriesDefinitions.Add(definition);
        //        counter++;
        //    }

        //    chart = new Chart();
        //    chart.Loaded += new RoutedEventHandler(chart_Loaded);
        //    chart.BorderThickness = new Thickness(0);
        //    chart.Series.Add(stackedSeries);
        //    chart.Height = 600;
        //    if (data.Rows.Count > 20)
        //        chart.Width = data.Rows.Count * 40;
        //    else
        //        chart.Width = 800;
        //    pnlChartContainer.Children.Clear();
        //    pnlChartContainer.Children.Add(chart);
        //}

        //private DataTable GetStackedColumnData()
        //{
        //    DataTable data = new DataTable();
        //    //string query = "select count(" + cbxColumnYAxisField.SelectedItem.ToString() + ") as totals, " + cbxColumnXAxisField.SelectedItem.ToString() + ", " + cbxColumnYAxisField.SelectedItem.ToString() + " " + view.FromViewSQL + " group by " + cbxColumnYAxisField.SelectedItem.ToString() + ", " + cbxColumnXAxisField.SelectedItem.ToString() + " order by " + cbxColumnYAxisField.SelectedItem.ToString();
        //    //data.Merge(db.Select(db.CreateQuery(query)), true, MissingSchemaAction.Add);
        //    string xAxisColumnName = gadgetOptions.MainVariableName;
        //    string yAxisColumnName = gadgetOptions.CrosstabVariableName;
        //    data = dashboardHelper.GenerateStackedBarTable(xAxisColumnName, yAxisColumnName);
        //    if (data != null && data.Rows.Count > 0)
        //    {
        //        data = Pivot(data, xAxisColumnName, yAxisColumnName, "totals");
        //    }
        //    return data;
        //}

        private void CheckAndSetPosition()
        {
            double top = Canvas.GetTop(this);
            double left = Canvas.GetLeft(this);

            if (top < 0)
            {
                Canvas.SetTop(this, 0);
            }
            if (left < 0)
            {
                Canvas.SetLeft(this, 0);
            }
        }

        /// <summary>
        /// Sends the gadget to the back of the canvas
        /// </summary>
        //private void SendToBack()
        //{
        //    Canvas.SetZIndex(this, -1);
        //}

        /// <summary>
        /// Render Finish
        /// </summary>
        private void RenderFinish()
        {
            waitCursor.Visibility = System.Windows.Visibility.Collapsed;
            spContent.Visibility = System.Windows.Visibility.Visible;
            pnlChartContainer.Visibility = System.Windows.Visibility.Visible;

            pnlStatus.Visibility = System.Windows.Visibility.Collapsed;
            txtStatus.Text = string.Empty;

            SetChartLabels(ChartTitle, LegendTitle, XAxisLabel, YAxisLabel);

            CheckAndSetPosition();
        }

        /// <summary>
        /// Render finish with Warning
        /// </summary>
        /// <param name="errorMessage"></param>
        private void RenderFinishWithWarning(string errorMessage)
        {
            waitCursor.Visibility = System.Windows.Visibility.Collapsed;
            spContent.Visibility = System.Windows.Visibility.Visible;
            pnlChartContainer.Visibility = System.Windows.Visibility.Visible;

            //pnlStatus.Background = Brushes.Gold;
            //pnlStatusTop.Background = Brushes.Goldenrod;

            pnlStatus.Visibility = System.Windows.Visibility.Visible;
            txtStatus.Text = errorMessage;

            SetChartLabels(ChartTitle, LegendTitle, XAxisLabel, YAxisLabel);

            CheckAndSetPosition();
        }

        /// <summary>
        /// Render Finish with Error.
        /// </summary>
        /// <param name="errorMessage"></param>
        private void RenderFinishWithError(string errorMessage)
        {
            waitCursor.Visibility = System.Windows.Visibility.Collapsed;
            spContent.Visibility = System.Windows.Visibility.Visible;
            pnlChartContainer.Visibility = System.Windows.Visibility.Collapsed;

            //pnlStatus.Background = Brushes.Tomato;
            //pnlStatusTop.Background = Brushes.Red;

            pnlStatus.Visibility = System.Windows.Visibility.Visible;

            txtStatus.Text = errorMessage;

            CheckAndSetPosition();
        }

        /// <summary>
        /// updates status
        /// </summary>
        /// <param name="statusMessage"></param>
        private void RequestUpdateStatusMessage(string statusMessage)
        {
            //this.Dispatcher.BeginInvoke(new SetStatusDelegate(SetStatusMessage), statusMessage);
            SetStatusMessage(statusMessage);
        }

        /// <summary>
        /// updates status
        /// </summary>
        /// <param name="statusMessage"></param>
        private void SetStatusMessage(string statusMessage)
        {
            pnlStatus.Visibility = System.Windows.Visibility.Visible;
            txtStatus.Text = statusMessage;
        }

        private bool IsCancelled()
        {
            //if (worker != null && worker.WorkerSupportsCancellation && worker.CancellationPending)
            //{
            //    return true;
            //}
            //else
            //{
            return false;
            //}
        }

        //private void FillComboboxes(bool update = false)
        //{
        //    loadingCombos = true;

        //    string prevDateField = string.Empty;
        //    string prevCaseStatusField = string.Empty;
        //    string prevColumnXAxisField = string.Empty;
        //    string prevColumnYAxisField = string.Empty;
        //    string prevScatterXAxisField = string.Empty;
        //    string prevScatterYAxisField = string.Empty;
        //    string prevSingleField = string.Empty;
        //    string prevWeightField = string.Empty;
        //    string prevStrataField = string.Empty;

        //    if (update)
        //    {
        //        //if (cbxDateField.SelectedIndex >= 0)
        //        //{
        //        //    prevDateField = cbxDateField.SelectedItem.ToString();
        //        //}
        //        //if (cbxCaseStatusField.SelectedIndex >= 0)
        //        //{
        //        //    prevCaseStatusField = cbxCaseStatusField.SelectedItem.ToString();
        //        //}
        //        //if (cbxColumnXAxisField.SelectedIndex >= 0)
        //        //{
        //        //    prevColumnXAxisField = cbxColumnXAxisField.SelectedItem.ToString();
        //        //}
        //        //if (cbxColumnYAxisField.SelectedIndex >= 0)
        //        //{
        //        //    prevColumnYAxisField = cbxColumnYAxisField.SelectedItem.ToString();
        //        //}
        //        if (cbxScatterXAxisField.SelectedIndex >= 0)
        //        {
        //            prevScatterXAxisField = cbxScatterXAxisField.SelectedItem.ToString();
        //        }
        //        if (cbxScatterYAxisField.SelectedIndex >= 0)
        //        {
        //            prevScatterYAxisField = cbxScatterYAxisField.SelectedItem.ToString();
        //        }
        //        //if (cbxSingleField.SelectedIndex >= 0)
        //        //{
        //        //    prevSingleField = cbxSingleField.SelectedItem.ToString();
        //        //}
        //        //if (cbxWeightField.SelectedIndex >= 0)
        //        //{
        //        //    prevWeightField = cbxWeightField.SelectedItem.ToString();
        //        //}
        //        //if (cbxStrataField.SelectedIndex >= 0)
        //        //{
        //        //    prevStrataField = cbxStrataField.SelectedItem.ToString();
        //        //}
        //    }

        //    //cbxCaseStatusField.ItemsSource = null;
        //    //cbxCaseStatusField.Items.Clear();

        //    //cbxDateField.ItemsSource = null;
        //    //cbxDateField.Items.Clear();

        //    List<string> caseStatusFields = new List<string>();
        //    List<string> dateFields = new List<string>();
        //    List<string> numericFields = new List<string>();
        //    List<string> allFields = new List<string>();
        //    List<ColumnDataType> columnDataType = new List<ColumnDataType>();
        //    caseStatusFields.Add(string.Empty);

        //    //if (dashboardHelper.IsUsingEpiProject)
        //    //{
        //    //    ColumnDataType columnDataType = ColumnDataType.Numeric | ColumnDataType.UserDefined;
        //    //    numericFields = dashboardHelper.GetFieldsAsList(columnDataType);

        //    //    columnDataType = ColumnDataType.DateTime | ColumnDataType.Numeric | ColumnDataType.Boolean | ColumnDataType.Text | ColumnDataType.UserDefined;
        //    //    allFields = dashboardHelper.GetFieldsAsList(columnDataType);

        //    //    foreach (Field f in View.Fields)
        //    //    {
        //    //        if ((f is TableBasedDropDownField) || (f is YesNoField))
        //    //        {
        //    //            caseStatusFields.Add(f.Name);
        //    //        }
        //    //        if ((f is DateField) || (f is DateTimeField) || (f is NumberField))
        //    //        {
        //    //            dateFields.Add(f.Name);
        //    //        }
        //    //    }

        //    //    foreach (EpiDashboard.Rules.IDashboardRule rule in dashboardHelper.Rules)
        //    //    {
        //    //        if (rule is EpiDashboard.Rules.DataAssignmentRule)
        //    //        {
        //    //            EpiDashboard.Rules.DataAssignmentRule assignmentRule = rule as EpiDashboard.Rules.DataAssignmentRule;
        //    //            if (!dateFields.Contains(assignmentRule.DestinationColumnName) && assignmentRule.VariableType.Equals(EpiDashboard.Rules.DashboardVariableType.Numeric))
        //    //            {
        //    //                dateFields.Add(assignmentRule.DestinationColumnName);
        //    //            }
        //    //        }
        //    //    }

        //    //    if (allFields.Contains("RecStatus"))
        //    //    {
        //    //        allFields.Remove("RecStatus");
        //    //    }

        //    //    if (allFields.Contains("GlobalRecordId"))
        //    //    {
        //    //        allFields.Remove("GlobalRecordId");
        //    //    }

        //    //    if (allFields.Contains("FKEY"))
        //    //    {
        //    //        allFields.Remove("FKEY");
        //    //    }

        //    //    if (allFields.Contains("UniqueKey"))
        //    //    {
        //    //        allFields.Remove("UniqueKey");
        //    //    }

        //    //    if (dateFields.Contains("SYSTEMDATE"))
        //    //    {
        //    //        allFields.Remove("SYSTEMDATE");
        //    //    }

        //    //    dateFields.Sort();
        //    //    numericFields.Sort();
        //    //    caseStatusFields.Sort();
        //    //    allFields.Sort();
        //    //}
        //    //else
        //    //{

        //    //List<EwavColumn> columnDataTyp = new List<EwavColumn>();
        //    //columnDataType.Add(ColumnDataType.DateTime);
        //    columnDataType.Add(ColumnDataType.Numeric);

        //    IEnumerable<EwavColumn> CBXFieldCols = from cols in sctrViewModel.Columns
        //                                           where columnDataType.Contains(cols.SqlDataTypeAsString)
        //                                           orderby cols.Name
        //                                           select cols;

        //    List<EwavColumn> colsList = CBXFieldCols.ToList();

        //    colsList.Insert(0, new EwavColumn() { Name = " ", Index = -1 });
        //    cbxScatterXAxisField.ItemsSource = colsList;
        //    cbxScatterXAxisField.SelectedValue = "Index";
        //    cbxScatterXAxisField.DisplayMemberPath = "Name";

        //    columnDataType.Clear();
        //    //columnDataType.Add(ColumnDataType.Boolean);
        //    columnDataType.Add(ColumnDataType.Numeric);
        //    //columnDataType.Add(ColumnDataType.Text);

        //    CBXFieldCols = from cols in sctrViewModel.Columns
        //                   where columnDataType.Contains(cols.SqlDataTypeAsString)
        //                   orderby cols.Name
        //                   select cols;

        //    colsList = CBXFieldCols.ToList();

        //    colsList.Insert(0, new EwavColumn() { Name = " ", Index = -1 });
        //    cbxScatterYAxisField.ItemsSource = colsList;
        //    cbxScatterYAxisField.SelectedValue = "Index";
        //    cbxScatterYAxisField.DisplayMemberPath = "Name";

        //    //columnDataType = ColumnDataType.Numeric | ColumnDataType.UserDefined;
        //    //numericFields = dashboardHelper.GetFieldsAsList(columnDataType);

        //    //columnDataType = ColumnDataType.DateTime | ColumnDataType.Numeric | ColumnDataType.Boolean | ColumnDataType.Text | ColumnDataType.UserDefined;
        //    //allFields = dashboardHelper.GetFieldsAsList(columnDataType);
        //    // }

        //    //foreach (EpiDashboard.Rules.IDashboardRule rule in dashboardHelper.Rules)
        //    //{
        //    //    if (rule is EpiDashboard.Rules.Rule_Format)
        //    //    {
        //    //        EpiDashboard.Rules.Rule_Format formatRule = rule as EpiDashboard.Rules.Rule_Format;
        //    //        if (!dateFields.Contains(formatRule.DestinationColumnName) && (formatRule.FormatType == Rules.FormatTypes.Hours || formatRule.FormatType == Rules.FormatTypes.EpiWeek))
        //    //        {
        //    //            dateFields.Add(formatRule.DestinationColumnName);
        //    //        }
        //    //    }
        //    //}

        //    List<string> allFieldsWithSpace = new List<string>();
        //    allFieldsWithSpace.Add(string.Empty);
        //    allFieldsWithSpace.AddRange(allFields);

        //    List<string> numericFieldsWithSpace = new List<string>();
        //    numericFieldsWithSpace.Add(string.Empty);
        //    numericFieldsWithSpace.AddRange(numericFields);

        //    if (dateFields.Contains("SYSTEMDATE"))
        //    {
        //        dateFields.Remove("SYSTEMDATE");
        //    }

        //    if (allFields.Contains("SYSTEMDATE"))
        //    {
        //        allFields.Remove("SYSTEMDATE");
        //    }

        //    //cbxDateField.ItemsSource = dateFields;
        //    //cbxCaseStatusField.ItemsSource = caseStatusFields;
        //    //cbxColumnXAxisField.ItemsSource = allFields;
        //    //cbxColumnYAxisField.ItemsSource = allFields;
        //    cbxScatterXAxisField.ItemsSource = numericFields;
        //    cbxScatterYAxisField.ItemsSource = numericFields;
        //    //cbxSingleField.ItemsSource = allFields;
        //    //cbxWeightField.ItemsSource = numericFieldsWithSpace;
        //    //cbxStrataField.ItemsSource = allFieldsWithSpace;

        //    //if (cbxCaseStatusField.Items.Count > 0)
        //    //{
        //    //    cbxCaseStatusField.SelectedIndex = -1;
        //    //}
        //    //if (cbxDateField.Items.Count > 0)
        //    //{
        //    //    cbxDateField.SelectedIndex = -1;
        //    //}
        //    if (cbxScatterXAxisField.Items.Count > 0)
        //    {
        //        cbxScatterXAxisField.SelectedIndex = -1;
        //        cbxScatterYAxisField.SelectedIndex = -1;
        //        cbxWeightField.SelectedIndex = -1;
        //    }
        //    //if (cbxSingleField.Items.Count > 0)
        //    //{
        //    //    cbxSingleField.SelectedIndex = -1;
        //    //    cbxStrataField.SelectedIndex = -1;
        //    //    cbxColumnXAxisField.SelectedIndex = -1;
        //    //    cbxColumnYAxisField.SelectedIndex = -1;
        //    //}

        //    //if (cbxChartSize.Items.Count == 0)
        //    //{
        //    //    cbxChartSize.Items.Add("Small");
        //    //    cbxChartSize.Items.Add("Medium");
        //    //    cbxChartSize.Items.Add("Large");
        //    //    cbxChartSize.SelectedIndex = 1;
        //    //}

        //    if (update)
        //    {
        //        //cbxDateField.SelectedItem = prevDateField;
        //        //cbxCaseStatusField.SelectedItem = prevCaseStatusField;
        //        //cbxColumnXAxisField.SelectedItem = prevColumnXAxisField;
        //        //cbxColumnYAxisField.SelectedItem = prevColumnYAxisField;
        //        cbxScatterXAxisField.SelectedItem = prevScatterXAxisField;
        //        cbxScatterYAxisField.SelectedItem = prevScatterYAxisField;
        //        //cbxSingleField.SelectedItem = prevSingleField;
        //        //cbxWeightField.SelectedItem = prevWeightField;
        //        //cbxStrataField.SelectedItem = prevStrataField;
        //    }

        //    loadingCombos = false;
        //}

        //private void GenerateScatter()
        //{
        //    //if (worker != null && worker.WorkerSupportsCancellation)
        //    //{
        //    //    worker.CancelAsync();
        //    //}  
        //    //pnlChartContainer.Children.Clear();
        //    //waitCursor.Visibility = System.Windows.Visibility.Visible;
        //    //DatatableBag dtb = sctrViewModel.DataTableBag;

        //    ScatterDataDTO lrr = sctrViewModel.RegressionResults;

        //    Palette p = new Palette();
        //    p.PaletteName = "VibrantA";

        //    XYChart scatterChart = new XYChart()
        //    {
        //        MinWidth = 800,
        //        Height = 500,
        //        HighlightDataPointOnHover = true,
        //        XPath = "IndependentValue",
        //        SelectionVisualHint = ComponentArt.Silverlight.DataVisualization.Charting.SelectionVisualHint.InvertedColor,
        //        Theme = "ArcticWhite",
        //        Palette = Palette.GetPalette("VibrantC"), // "VibrantA")     
        //        EnableAnimation = true,
        //        AnimationDuration = new TimeSpan(0, 0, 0, 4),
        //        EnableDataPointPopup = false,
        //        LegendVisible = true,
        //        LegendDock = ComponentArt.Silverlight.DataVisualization.Charting.Dock.Right,
        //        CoordinatesPaddingPercentage = new Thickness(5, 5, 0, 0),
        //        DefaultAxisAnnotationsVisible = true,
        //        DefaultGridLinesVisible = true,
        //        GlareCoverVisible = false,
        //        AnimationOnLoad = false,
        //        DefaultStripesVisible = true//,
        //        //ThemeVariant = ThemeVariant.Standard// new ThemeVariant()    // Standard  
        //    };

        //    Legend EpiLegend = new Legend()
        //    {
        //        CornerRadius = new CornerRadius(10),
        //        BorderBrush = new SolidColorBrush(Color.FromArgb(255, 248, 215, 226)), //   "#FFc4c4c4", 
        //        Margin = new Thickness(30, 0, 10, 0),
        //        FontFamily = new FontFamily("Segoe UI"),

        //        //    FontWeight = new Thickness(300),
        //        //  AllowDrop = true
        //    };

        //    //            <my:XYChart.XAxisArea>
        //    //    <my:AxisCoordinates AxisId="XAxis" Angle="70" Margin="20"/>
        //    //    <my:ChartLabel Orientation="Horizontal">
        //    //        <TextBlock Text="X-XAXIS LABEL GOES HERE" Width="402" TextAlignment="Center" Padding="0,10,0,10" Margin="0,40,0,0" />
        //    //    </my:ChartLabel>
        //    //</my:XYChart.XAxisArea>

        //    AxisCoordinates XaxisCoordinates = new AxisCoordinates()
        //    {
        //        AxisId = "XAxis",
        //        Angle = 70,
        //        Margin = new Thickness(0, 5, 0, 0),
        //        FontSize = 8.5,
        //        LabelGap = 1.0,
        //        Coordinates = new DateTimeCoordinates()
        //    };

        //    ChartLabel chartLabel = new ChartLabel()
        //    {
        //        Orientation = ChartLabelOrientation.Horizontal
        //    };

        //    textBlockX = new TextBlock()
        //    {
        //        Text = ((EwavColumn)cbxDateField.SelectedItem).Name,
        //        Width = 402,
        //        TextAlignment = System.Windows.TextAlignment.Center,
        //        Padding = new Thickness(0, 10, 0, 10),
        //        Margin = new Thickness(0, 40, 0, 0)
        //    };

        //    chartLabel.Child = textBlockX;

        //    //  AxisCoordinates
        //    barChart.XAxisArea.Add(XaxisCoordinates);  //   axisCoordinates);
        //    barChart.XAxisArea.Add(chartLabel);

        //    textBlockY = new TextBlock()
        //    {
        //        Text = "Count",
        //        Width = 402,
        //        VerticalAlignment = System.Windows.VerticalAlignment.Bottom,
        //        TextAlignment = System.Windows.TextAlignment.Center,
        //        Padding = new Thickness(0, 10, 0, 10),
        //        Margin = new Thickness(0, 40, 0, 0)
        //    };

        //    barChart.YAxisArea.Add(new AxisCoordinates());
        //    barChart.YAxisArea.Add(new ChartLabel()
        //    {
        //        Child = textBlockY,
        //        Orientation = ComponentArt.Silverlight.DataVisualization.Charting.ChartLabelOrientation.Vertical
        //    });

        //    bool byEpiWeek = false;
        //    if (gadgetOptions.InputVariableList.ContainsKey("isdatecolumnnumeric"))
        //    {
        //        byEpiWeek = bool.Parse(gadgetOptions.InputVariableList["isdatecolumnnumeric"]);
        //    }

        //    string seriesTrackerText = "";
        //    int seriesCounter = 0;

        //    List<MyString> chartColsList = dtb.ColumnNameList.GetRange(1, dtb.ColumnNameList.Count - 1);

        //    CompositeSeries compositeSeries = new CompositeSeries()
        //    {
        //        CompositionKind = ComponentArt.Silverlight.DataVisualization.Charting.CompositionKind.Stacked
        //    };

        //    foreach (MyString col in chartColsList)
        //    {
        //        List<ChartDataValue> chartDataValuesList = new List<ChartDataValue>();

        //        foreach (FieldsList row in dtb.RecordList)
        //        {
        //            ChartDataValue data = new ChartDataValue()
        //            {
        //                IndependentValue = row.Fields[0].VarName,
        //                DependentValue = dtb.GetValueAtRow(col.VarName, row)

        //            };

        //            //if (data.IndependentValue.Trim().Length == 0)
        //            //{
        //            //    data.IndependentValue = "1/1/2001";
        //            //}

        //            if (data.DependentValue.Length == 0)
        //            {
        //                data.DependentValue = "0";
        //            }
        //            chartDataValuesList.Add(data);
        //        }

        //        ColorShift topColorShift = new ColorShift();
        //        topColorShift.BrightnessShift = 70;

        //        ColorShift bottomColorShift = new ColorShift();
        //        bottomColorShift.BrightnessShift = 1;
        //        seriesCounter++;
        //        seriesTrackerText += dtb.ColumnNameList.ElementAt<MyString>(seriesCounter).VarName + ",";      //   "S" + seriesCounter + ",";

        //        AreaSeriesAnimator asa = AreaSeriesAnimator.Create("SwipeSmooth");

        //        barChart.Width = chartDataValuesList.Count * 20;

        //        AreaSeries areaSeries = new AreaSeries()
        //        {
        //            Id = dtb.ColumnNameList.ElementAt<MyString>(seriesCounter).VarName, //     "S" + seriesCounter,
        //            LineKind = ComponentArt.Silverlight.DataVisualization.Charting.LineKind.Step,
        //            DataSource = chartDataValuesList,
        //            YPath = "DependentValue",
        //            XPath = "IndependentValue",
        //            SeriesLineVisible = false,
        //            EnableAnimation = true,
        //            Texture = TextureKind.BrushedMetal,
        //            Animator = asa
        //            //LineThickness = 2,
        //            ////    Color = Color.FromArgb(255, 254, 203, 0),
        //            //GradientTopColorShift = topColorShift,
        //            //GradientBottomColorShift = bottomColorShift
        //        };

        //        compositeSeries.SubSeries.Add(areaSeries);
        //    }

        //    seriesTrackerText = seriesTrackerText.Substring(0, seriesTrackerText.Length - 1);
        //    SeriesTracker st = new SeriesTracker();
        //    st.SeriesId = seriesTrackerText;

        //    SeriesAnnotationTracker sat = new SeriesAnnotationTracker();
        //    sat.SeriesIdsCSS = seriesTrackerText;

        //    barChart.XYChartMainArea.Add(compositeSeries);
        //    barChart.XYChartMainArea.Add(st);
        //    barChart.XYChartMainArea.Add(sat);

        //    pnlChartContainer.Visibility = System.Windows.Visibility.Visible;

        //    pnlChartContainer.Children.Clear();
        //    pnlChartContainer.Children.Add(barChart);

        //    RenderFinish();
        //    //System.ComponentModel.BackgroundWorker epiCurveWorker = new System.ComponentModel.BackgroundWorker();
        //    //worker = new BackgroundWorker();
        //    //worker.DoWork += new System.ComponentModel.DoWorkEventHandler(epiCurveWorker_DoWork);
        //    //worker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(epiCurveWorker_RunWorkerCompleted);
        //    //  epiCurveWorker_DoWork(dtb, gadgetOptions);
        //    //   epiCurveWorker_RunWorkerCompleted();
        //    //object[] args = new object[4];
        //    //args[0] = byEpiWeek;
        //    //args[1] = gadgetOptions.MainVariableName;// cbxDateField.SelectedItem.ToString();
        //    //args[2] = gadgetOptions.CrosstabVariableName;
        //    //args[3] = gadgetOptions.InputVariableList["dateinterval"];
        //    //if (cbxCaseStatusField.SelectedIndex > -1)
        //    //{
        //    //    args[2] = cbxCaseStatusField.SelectedItem.ToString();
        //    //}
        //    //else
        //    //{
        //    //    args[2] = string.Empty;
        //    //}
        //    //worker.RunWorkerAsync(args);
        //}

        //void epiCurveWorker_RunWorkerCompleted()
        //{
        //    if (databag != null)
        //    {
        //        //DataTable data = (DataTable)((object[])e.Result)[1];
        //        //List<List<StringDataValue>> dataValues = (List<List<StringDataValue>>)((object[])databag)[0];
        //        //List<List<StringDataValue>> dataValues = new List<List<StringDataValue>>(); ;
        //        //this.Dispatcher.BeginInvoke(new RenderFinishEpiCurveDelegate(RenderFinishEpiCurve), data, dataValues);
        //        RenderFinishEpiCurve(databag, dataValues);
        //    }
        //}

        //private void RenderFinishEpiCurve(DatatableBag data, List<List<StringDataValue>> dataValues)
        //{
        //    waitCursor.Visibility = System.Windows.Visibility.Collapsed;

        //    //StackedHistogramSeries stackedSeries = new StackedHistogramSeries();
        //    //LinearAxis axis = new LinearAxis();
        //    //axis.Orientation = AxisOrientation.Y;
        //    //axis.Minimum = 0;
        //    //axis.ShowGridLines = true;
        //    //stackedSeries.DependentAxis = axis;
        //    //CategoryAxis independentAxis = new CategoryAxis();
        //    //independentAxis.Orientation = AxisOrientation.X;
        //    //if (!dashboardHelper.IsColumnNumeric(cbxDateField.SelectedItem.ToString()))
        //    //    independentAxis.AxisLabelStyle = Resources["RotateAxisStyle"] as Style;
        //    //stackedSeries.IndependentAxis = independentAxis;

        //    //int counter = 1;
        //    //foreach (var values in dataValues)
        //    //{
        //    //    SeriesDefinition definition = new SeriesDefinition();
        //    //    definition.DependentValuePath = "DependentValue";
        //    //    definition.IndependentValuePath = "IndependentValue";
        //    //    if (cbxCaseStatusField.SelectedIndex >= 0 && !string.IsNullOrEmpty(cbxCaseStatusField.SelectedItem.ToString()))
        //    //    {
        //    //        Field field = null;
        //    //        foreach (DataRow fieldRow in dashboardHelper.FieldTable.Rows)
        //    //        {
        //    //            if (fieldRow["columnname"].Equals(cbxCaseStatusField.SelectedItem.ToString()))
        //    //            {
        //    //                if (fieldRow["epifieldtype"] is Field)
        //    //                {
        //    //                    field = fieldRow["epifieldtype"] as Field;
        //    //                }
        //    //                break;
        //    //            }
        //    //        }

        //    //        if (field != null && (field is YesNoField || field is CheckBoxField))
        //    //        {
        //    //            string value = data.Columns[counter].ColumnName;

        //    //            if (value.ToLower().Equals("true") || value.Equals("1"))
        //    //            {
        //    //                definition.Title = "Yes";
        //    //            }
        //    //            else if (value.ToLower().Equals("false") || value.Equals("0"))
        //    //            {
        //    //                definition.Title = "No";
        //    //            }
        //    //            else
        //    //            {
        //    //                definition.Title = cbxCaseStatusField.SelectedItem.ToString();
        //    //            }
        //    //        }
        //    //        else
        //    //        {
        //    //            definition.Title = data.Columns[counter].ColumnName;
        //    //        }
        //    //    }
        //    //    else
        //    //    {
        //    //        definition.Title = SharedStrings.DEFAULT_CHART_LEGEND_ITEM;
        //    //    }
        //    //    definition.ItemsSource = values;

        //    //    stackedSeries.SeriesDefinitions.Add(definition);
        //    //    counter++;
        //    //}

        //    XYChart chart = new XYChart();
        //    chart.BorderThickness = new Thickness(0);
        //    chart.Loaded += new RoutedEventHandler(chart_Loaded);
        //    //chart.Series.Add(stackedSeries);
        //    //chart.XYChartMainArea.Add(stackedSeries);
        //    chart.Height = 500;
        //    //chart.Width = (stackedSeries.IndependentValueCount * 25) + 150;
        //    pnlChartContainer.Children.Clear();
        //    pnlChartContainer.Children.Add(chart);

        //    SetChartLabels(ChartTitle, LegendTitle, XAxisLabel, YAxisLabel);
        //}

        //private List<StringDataValue> ConvertToPct(List<StringDataValue> list)
        //{
        //    List<StringDataValue> values = new List<StringDataValue>();
        //    foreach (StringDataValue value in list)
        //    {
        //        values.Add(new StringDataValue() { IndependentValue = value.IndependentValue, DependentValue = value.DependentValue / 31 * 100 });
        //    }
        //    return values;
        //}

        //private DataTable GetEpiCurveData(bool byEpiWeek, string dateVar, string caseStatusVar)
        //{
        //    DataTable data = new DataTable();

        //    if (byEpiWeek)
        //    {
        //        decimal? minWeek = null;
        //        decimal? maxWeek = null;

        //        dashboardHelper.FindUpperLowerNumericValues(dateVar, ref minWeek, ref maxWeek);
        //        if (minWeek.HasValue && maxWeek.HasValue)
        //        {
        //            // override min and max values if the user set custom start and end points for the x-axis
        //            if (gadgetOptions.InputVariableList.ContainsKey("xaxisstart"))
        //            {
        //                minWeek = decimal.Parse(gadgetOptions.InputVariableList["xaxisstart"]);
        //            }
        //            if (gadgetOptions.InputVariableList.ContainsKey("xaxisend"))
        //            {
        //                maxWeek = decimal.Parse(gadgetOptions.InputVariableList["xaxisend"]);
        //            }

        //            if (string.IsNullOrEmpty(caseStatusVar))
        //            {
        //                gadgetOptions.CrosstabVariableName = "_default_";
        //                data = dashboardHelper.GenerateEpiCurveTable(gadgetOptions, minWeek.Value, maxWeek.Value);
        //            }
        //            else
        //            {
        //                data = dashboardHelper.GenerateEpiCurveTable(gadgetOptions, minWeek.Value, maxWeek.Value);
        //            }
        //            if (data != null && data.Rows.Count > 0)
        //            {
        //                if (string.IsNullOrEmpty(caseStatusVar))
        //                {
        //                    data = Pivot(data, dateVar, "_default_", "totals");
        //                }
        //                else
        //                {
        //                    data = Pivot(data, dateVar, caseStatusVar, "totals");
        //                }
        //            }
        //            decimal currentWeek = minWeek.Value;
        //            while (currentWeek <= maxWeek.Value)
        //            {
        //                if (data.Select("[" + dateVar + "] = '" + currentWeek.ToString() + "'").Count() == 0)
        //                {
        //                    DataRow row = data.NewRow();
        //                    row[0] = currentWeek;
        //                    data.Rows.Add(row);
        //                }
        //                currentWeek = currentWeek + 1;
        //            }
        //        }
        //    }
        //    else
        //    {
        //        string dateInterval = gadgetOptions.InputVariableList["dateinterval"];

        //        DateTime? minDate = null;
        //        DateTime? maxDate = null;

        //        dashboardHelper.FindUpperLowerDateValues(dateVar, ref minDate, ref maxDate);
        //        if (minDate.HasValue && maxDate.HasValue)
        //        {
        //            if (gadgetOptions.InputVariableList.ContainsKey("xaxisstart"))
        //            {
        //                DateTime dt;
        //                bool success = DateTime.TryParse(gadgetOptions.InputVariableList["xaxisstart"], out dt);
        //                if (success)
        //                {
        //                    minDate = dt;
        //                }
        //            }
        //            if (gadgetOptions.InputVariableList.ContainsKey("xaxisend"))
        //            {
        //                DateTime dt;
        //                bool success = DateTime.TryParse(gadgetOptions.InputVariableList["xaxisend"], out dt);
        //                if (success)
        //                {
        //                    maxDate = dt;
        //                }
        //            }

        //            if (string.IsNullOrEmpty(caseStatusVar))
        //            {
        //                gadgetOptions.CrosstabVariableName = "_default_";
        //            }

        //            data = dashboardHelper.GenerateEpiCurveTable(gadgetOptions, (DateTime)minDate, (DateTime)maxDate);

        //            if (data != null && data.Rows.Count > 0)
        //            {
        //                if (string.IsNullOrEmpty(caseStatusVar))
        //                {
        //                    data = Pivot(data, dateVar, "_default_", "totals");
        //                }
        //                else
        //                {
        //                    data = Pivot(data, dateVar, caseStatusVar, "totals");
        //                }
        //            }
        //            DateTime currentDate = minDate.Value;
        //            while (currentDate <= maxDate.Value)
        //            {
        //                if (IsCancelled())
        //                {
        //                    data = null;
        //                    return data;
        //                }

        //                if (data.Select("[" + dateVar + "] = '" + currentDate.ToString() + "'").Count() == 0)
        //                {
        //                    DataRow row = data.NewRow();
        //                    row[0] = currentDate;
        //                    data.Rows.Add(row);
        //                }

        //                //if (dashboardHelper.IsUsingEpiProject && View.Fields.Contains(dateVar) && View.Fields[dateVar] is DateTimeField && !(View.Fields[dateVar] is DateField) && !(View.Fields[dateVar] is TimeField))
        //                if (dateInterval.ToLower().Equals("hours"))
        //                {
        //                    currentDate = currentDate.AddHours(1);
        //                }
        //                else if (dateInterval.ToLower().Equals("days"))
        //                {
        //                    currentDate = currentDate.AddDays(1);
        //                }
        //                else if (dateInterval.ToLower().Equals("months"))
        //                {
        //                    currentDate = currentDate.AddMonths(1);
        //                }
        //                else if (dateInterval.ToLower().Equals("years"))
        //                {
        //                    currentDate = currentDate.AddMonths(1);
        //                }
        //            }
        //        }
        //    }
        //    return data;
        //}

        //public static DataTable Pivot(DataTable dataValues, string keyColumn, string pivotNameColumn, string pivotValueColumn)
        //{
        //    DataTable tmp = new DataTable();
        //    DataRow r;
        //    string LastKey = "//dummy//";
        //    int rule, pValIndex, pNameIndex;
        //    string s;
        //    bool FirstRow = true;

        //    pValIndex = dataValues.Columns[pivotValueColumn].Ordinal;
        //    pNameIndex = dataValues.Columns[pivotNameColumn].Ordinal;

        //    for (rule = 0; rule <= dataValues.Columns.Count - 1; rule++)
        //    {
        //        if (rule != pValIndex && rule != pNameIndex)
        //            tmp.Columns.Add(dataValues.Columns[rule].ColumnName, dataValues.Columns[rule].DataType);
        //    }

        //    r = tmp.NewRow();

        //    foreach (DataRow row1 in dataValues.Rows)
        //    {
        //        if (row1[keyColumn] != DBNull.Value)
        //        {
        //            if (row1[keyColumn].ToString() != LastKey)
        //            {
        //                if (!FirstRow)
        //                    tmp.Rows.Add(r);

        //                r = tmp.NewRow();
        //                FirstRow = false;

        //                //loop thru fields of row1 and populate tmp table
        //                for (rule = 0; rule <= row1.ItemArray.Length - 3; rule++)
        //                    r[rule] = row1[tmp.Columns[rule].ToString()];

        //                LastKey = row1[keyColumn].ToString();
        //            }

        //            s = row1[pNameIndex].ToString();

        //            if (!string.IsNullOrEmpty(s))
        //            {
        //                if (!tmp.Columns.Contains(s))
        //                    tmp.Columns.Add(s, typeof(int));// dataValues.Columns[pNameIndex].DataType);
        //                r[s] = row1[pValIndex];
        //            }
        //        }
        //    }

        //    //add that final row to the datatable:
        //    tmp.Rows.Add(r);
        //    return tmp;
        //}

        //void Triangle_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        //{
        //    FrameworkElement source = sender as FrameworkElement;
        //    CollapseExpandConfigPanel();
        //}

        //private void CollapseExpandConfigPanel()
        //{
        //    if (triangleCollapsed)
        //    {
        //        ConfigExpandedTriangle.Visibility = Visibility.Visible;
        //        ConfigCollapsedTriangle.Visibility = Visibility.Collapsed;
        //        ConfigCollapsedTitle.Visibility = Visibility.Collapsed;
        //        ConfigGrid.Height = Double.NaN;
        //        ConfigGrid.UpdateLayout();
        //    }
        //    else
        //    {
        //        ConfigCollapsedTriangle.Visibility = Visibility.Visible;
        //        ConfigExpandedTriangle.Visibility = Visibility.Collapsed;
        //        ConfigCollapsedTitle.Visibility = Visibility.Visible;
        //        ConfigGrid.Height = 50;
        //    }
        //    triangleCollapsed = !triangleCollapsed;
        //}

        private bool IsBooleanWithNoStratas
        {
            get
            {
                return this.isBooleanWithNoStratas;
            }
            set
            {
                this.isBooleanWithNoStratas = value;
            }
        }

        public void ClearResults()
        {
            txtStatus.Text = string.Empty;
            pnlStatus.Visibility = Visibility.Collapsed;
            waitCursor.Visibility = Visibility.Visible;

            IsBooleanWithNoStratas = false;

            pnlChartContainer.Children.Clear();
            pnlChartContainer.Visibility = System.Windows.Visibility.Collapsed;
        }

        #region Private Properties
        //private View View
        //{
        //    get
        //    {
        //        return this.dashboardHelper.View;
        //    }
        //}

        //private IDbDriver Database
        //{
        //    get
        //    {
        //        return this.dashboardHelper.Database;
        //    }
        //}
        #endregion // Private Properties

        #region IGadget Members

        public bool IsProcessing { get; set; }

        /// <summary>
        /// Updates the variable names available in the gadget's properties
        /// </summary>
        public void UpdateVariableNames()
        {
            //FillComboboxes(true);
        }

        public void RefreshResults()
        {
            if (!LoadingCanvas && !LoadingDropDowns &&
                gadgetOptions != null &&
                (cbxScatterXAxisField.SelectedIndex > -1 && cbxScatterYAxisField.SelectedIndex > -1))//((cbxDateField.SelectedIndex > -1)))|| (cbxColumnXAxisField.SelectedIndex > -1 && cbxColumnYAxisField.SelectedIndex > -1) || (cbxScatterXAxisField.SelectedIndex > -1 && cbxScatterYAxisField.SelectedIndex > -1) || (cbxSingleField.SelectedIndex > -1)))
            {
                waitCursor.Visibility = Visibility.Visible;
                //pnlStatus.Visibility = Visibility.Visible;

                //pnlStatus.Background = Brushes.PaleGreen;
                //pnlStatusTop.Background = Brushes.Green;

                //this.Dispatcher.BeginInvoke(GadgetStatusUpdate, SharedStrings.DASHBOARD_GADGET_STATUS_INITIALIZING);

                gadgetOptions.MainVariableName = string.Empty;
                gadgetOptions.WeightVariableName = string.Empty;
                gadgetOptions.StrataVariableNames = new List<string>();
                gadgetOptions.CrosstabVariableName = string.Empty;
                gadgetOptions.InputVariableList = new Dictionary<string, string>();
                gadgetOptions.ShouldSortHighToLow = false;
                gadgetOptions.TableName = applicationViewModel.EwavSelectedDatasource.TableName;

                //baseWorker = new BackgroundWorker();

                //if (checkboxAllValues.IsChecked == true)
                //{
                //    gadgetOptions.ShouldUseAllPossibleValues = true;
                //}
                //else
                //{
                gadgetOptions.ShouldUseAllPossibleValues = false;
                //}

                //switch (((ComboBoxItem)cbxChartType.SelectedItem).Content.ToString())
                //{
                //case "Stacked Column":
                //    //GenerateStackedColumnChart();
                //    if (cbxColumnXAxisField.SelectedIndex >= 0 && cbxColumnYAxisField.SelectedIndex >= 0)
                //    {
                //        gadgetOptions.MainVariableName = cbxColumnXAxisField.SelectedItem.ToString();
                //        gadgetOptions.CrosstabVariableName = cbxColumnYAxisField.SelectedItem.ToString();
                //        baseWorker.DoWork += new System.ComponentModel.DoWorkEventHandler(GenerateStackedColumnChart);
                //        baseWorker.RunWorkerAsync();
                //    }
                //    break;
                // case "Epi Curve":
                //if (!string.IsNullOrEmpty(txtXAxisStartValue.Text))
                //{
                //    gadgetOptions.InputVariableList.Add("xaxisstart", txtXAxisStartValue.Text);
                //}
                //if (!string.IsNullOrEmpty(txtXAxisEndValue.Text))
                //{
                //    gadgetOptions.InputVariableList.Add("xaxisend", txtXAxisEndValue.Text);
                //}
                //gadgetOptions.InputVariableList.Add("isdatecolumnnumeric", numericColumnNames.Contains(cbxDateField.SelectedItem.ToString()).ToString().ToLower());
                //gadgetOptions.InputVariableList.Add("dateinterval", ((ComboBoxItem)cbxDateInterval.SelectedItem).Content.ToString());
                //gadgetOptions.MainVariableName = ((EwavColumn)cbxDateField.SelectedItem).Name.ToString();
                //if (cbxCaseStatusField.SelectedIndex >= 0 && ((EwavColumn)cbxCaseStatusField.SelectedItem).Name.ToString().Trim() != "")
                //{
                //    gadgetOptions.CrosstabVariableName = ((EwavColumn)cbxCaseStatusField.SelectedItem).Name.ToString();
                //}
                //GenerateEpiCurve(dashboardHelper.IsColumnNumeric(cbxDateField.SelectedItem.ToString()));
                //baseWorker.DoWork += new System.ComponentModel.DoWorkEventHandler(GenerateEpiCurve);

                //GenerateEpiCurve();
                //baseWorker.RunWorkerAsync();
                // break;
                //case "Scatter":
                //    //GenerateScatterChart();                        
                if (cbxScatterXAxisField.SelectedIndex >= 0 && cbxScatterYAxisField.SelectedIndex >= 0)
                {
                    gadgetOptions.MainVariableName = ((EwavColumn)cbxScatterXAxisField.SelectedItem).Name.ToString();
                    gadgetOptions.CrosstabVariableName = ((EwavColumn)cbxScatterYAxisField.SelectedItem).Name.ToString();
                    //gadgetOptions.DatasourceName = "NNDSS-HIV";
                    //gadgetOptions.TableName = "vwExternalData";
                    //GenerateScatterChart();
                    gadgetOptions.TableName =
                        applicationViewModel.EwavSelectedDatasource.TableName;
                    gadgetOptions.DatasourceName =
                        applicationViewModel.EwavSelectedDatasource.DatasourceName;
                }
                //break;
                //case "Pie":
                //    if (cbxChartSize.SelectedIndex > -1)
                //    {
                //        gadgetOptions.InputVariableList.Add("chartsize", cbxChartSize.SelectedItem.ToString());
                //    }
                //    gadgetOptions.InputVariableList.Add("charttype", "pie");
                //    gadgetOptions.ShouldSortHighToLow = true;
                //    gadgetOptions.MainVariableName = cbxSingleField.SelectedItem.ToString();
                //    if (cbxWeightField.SelectedIndex > -1 && !string.IsNullOrEmpty(cbxWeightField.SelectedItem.ToString()))
                //    {
                //        gadgetOptions.WeightVariableName = cbxWeightField.SelectedItem.ToString();
                //    }
                //    if (cbxStrataField.SelectedIndex > -1 && !string.IsNullOrEmpty(cbxStrataField.SelectedItem.ToString()))
                //    {
                //        List<string> strataVars = new List<string>();
                //        strataVars.Add(cbxStrataField.SelectedItem.ToString());
                //        gadgetOptions.StrataVariableNames = strataVars;
                //    }
                //    baseWorker.DoWork += new System.ComponentModel.DoWorkEventHandler(GenerateSingleChart);
                //    baseWorker.RunWorkerAsync();
                //    //GenerateSingleChart();
                //    break;
                //    default:
                //        if (cbxChartSize.SelectedIndex > -1)
                //        {
                //            gadgetOptions.InputVariableList.Add("chartsize", cbxChartSize.SelectedItem.ToString());
                //        }
                //        string chartType = (((ComboBoxItem)cbxChartType.SelectedItem).Content.ToString());
                //        if (chartType.ToLower().Equals("pareto"))
                //        {
                //            gadgetOptions.ShouldSortHighToLow = true;
                //            gadgetOptions.InputVariableList.Add("charttype", "pareto");
                //        }
                //        gadgetOptions.MainVariableName = cbxSingleField.SelectedItem.ToString();
                //        if (cbxWeightField.SelectedIndex > -1 && !string.IsNullOrEmpty(cbxWeightField.SelectedItem.ToString()))
                //        {
                //            gadgetOptions.WeightVariableName = cbxWeightField.SelectedItem.ToString();
                //        }
                //        if (!chartType.ToLower().Equals("pareto") && cbxStrataField.SelectedIndex > -1 && !string.IsNullOrEmpty(cbxStrataField.SelectedItem.ToString()))
                //        {
                //            List<string> strataVars = new List<string>();
                //            strataVars.Add(cbxStrataField.SelectedItem.ToString());
                //            gadgetOptions.StrataVariableNames = strataVars;
                //        }
                //        baseWorker.DoWork += new System.ComponentModel.DoWorkEventHandler(GenerateSingleChart);
                //        baseWorker.RunWorkerAsync();
                //        //GenerateSingleChart();
                //        break;
                //}
            }
        }

        void chart_Loaded(object sender, RoutedEventArgs e)
        {
            SetChartLabels(ChartTitle, LegendTitle, XAxisLabel, YAxisLabel);
        }

        void scatterChartWorker_DoWork()//(object sender, System.ComponentModel.DoWorkEventArgs e)
        {
            try
            {
                //this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToProcessingState));
                SetGadgetToProcessingState();
                //this.Dispatcher.BeginInvoke(new SimpleCallback(ClearResults));
                ClearResults();

                //RequestUpdateStatusDelegate requestUpdateStatus = new RequestUpdateStatusDelegate(RequestUpdateStatusMessage);
                //CheckForCancellationDelegate checkForCancellation = new CheckForCancellationDelegate(IsCancelled);

                //gadgetOptions.GadgetStatusUpdate += new GadgetStatusUpdateHandler(requestUpdateStatus);
                //gadgetOptions.GadgetCheckForCancellation += new GadgetCheckForCancellationHandler(checkForCancellation);

                //string xAxisVar = gadgetOptions.MainVariableName;
                //string yAxisVar = gadgetOptions.CrosstabVariableName;

                //List<string> columnNames = new List<string>();
                //columnNames.Add(xAxisVar);
                //columnNames.Add(yAxisVar);

                //DataTable regressTable = dashboardHelper.GenerateTable(columnNames);

                ScatterDataDTO regressTable = sctrViewModel.RegressionResults;
                RenderFinishScatterChart(regressTable.DataValues, regressTable.RegresResults, regressTable.MaxValue, regressTable.MinValue);
                //RenderFinishScatterChart()
                //if (regressTable == null) //|| regressTable.Rows.Count == 0)
                //{
                //    //this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), SharedStrings.NO_RECORDS_SELECTED);
                //    RenderFinishWithError(SharedStrings.NO_RECORDS_SELECTED);
                //    //this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
                //    SetGadgetToFinishedState();
                //    return;
                //}
                //else if (worker.CancellationPending)
                //{
                //    this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), SharedStrings.DASHBOARD_GADGET_STATUS_OPERATION_CANCELLED);
                //    this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
                //    System.Diagnostics.Debug.Print("Single chart thread was cancelled");
                //    return;
                //}
                //else
                //{
                //    List<NumericDataValue> dataValues = new List<NumericDataValue>();
                //    NumericDataValue minValue = null;
                //    NumericDataValue maxValue = null;

                //    //regressTable.FieldsList.Fields

                //    //foreach (DataRow row in regressTable.Rows)
                //    foreach(FieldsList row in regressTable.RecordList)
                //    {
                //        if(regressTable.GetValueAtRow(yAxisVar,row).Equals(DBNull.Value) || regressTable.GetValueAtRow(xAxisVar, row).Equals(DBNull.Value))
                //        //if (row[yAxisVar].Equals(DBNull.Value) || row[xAxisVar].Equals(DBNull.Value))
                //        {
                //            continue;
                //        }
                //       // NumericDataValue currentValue = new NumericDataValue() { DependentValue = Convert.ToDecimal(row[yAxisVar]), IndependentValue = Convert.ToDecimal(row[xAxisVar]) };
                //        NumericDataValue currentValue = new NumericDataValue() { DependentValue = Convert.ToDecimal(regressTable.GetValueAtRow(yAxisVar, row)), IndependentValue = Convert.ToDecimal(regressTable.GetValueAtRow(xAxisVar, row)) };
                //        dataValues.Add(currentValue);
                //        if (minValue == null)
                //        {
                //            minValue = currentValue;
                //        }
                //        else
                //        {
                //            if (currentValue.IndependentValue < minValue.IndependentValue)
                //            {
                //                minValue = currentValue;
                //            }
                //        }
                //        if (maxValue == null)
                //        {
                //            maxValue = currentValue;
                //        }
                //        else
                //        {
                //            if (currentValue.IndependentValue > maxValue.IndependentValue)
                //            {
                //                maxValue = currentValue;
                //            }
                //        }
                //    }

                //    StatisticsRepository.LinearRegression linearRegression = new StatisticsRepository.LinearRegression();
                //    Dictionary<string, string> inputVariableList = new Dictionary<string, string>();
                //    inputVariableList.Add(yAxisVar, "dependvar");
                //    inputVariableList.Add("intercept", "true");
                //    inputVariableList.Add("includemissing", "false");
                //    inputVariableList.Add("p", "0.95");
                //    inputVariableList.Add(xAxisVar, "unsorted");

                //    StatisticsRepository.LinearRegression.LinearRegressionResults regresResults = linearRegression.LinearRegression(inputVariableList, regressTable);

                //    //object[] result = new object[] { dataValues, regresResults, maxValue, minValue };
                //    //e.Result = result;

                //    //this.Dispatcher.BeginInvoke(new SimpleCallback(RenderFinish));
                //    //this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
                //    RenderFinishScatterChart(dataValues, regresResults, maxValue, minValue);

                //}.
                SetGadgetToFinishedState();
            }
            catch (Exception ex)
            {
                //this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), ex.Message);
                RenderFinishWithError(ex.Message);
                //this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
                SetGadgetToFinishedState();
            }
        }

        //private void GenerateSingleChart(object sender, System.ComponentModel.DoWorkEventArgs e)
        //{
        //    if (worker != null && worker.WorkerSupportsCancellation)
        //    {
        //        worker.CancelAsync();
        //    }

        //    //waitCursor.Visibility = System.Windows.Visibility.Visible;

        //    //if (cbxChartSize.SelectedIndex > -1)
        //    //{
        //    //    gadgetOptions.InputVariableList.Add("chartsize", cbxChartSize.SelectedItem.ToString());                
        //    //}

        //    //gadgetOptions.MainVariableName = cbxSingleField.SelectedItem.ToString();
        //    //if (cbxWeightField.SelectedIndex > -1)
        //    //{
        //    //    gadgetOptions.WeightVariableName = cbxWeightField.SelectedItem.ToString();
        //    //}
        //    //if (cbxStrataField.SelectedIndex > -1)
        //    //{
        //    //    List<string> strataVars = new List<string>();
        //    //    strataVars.Add(cbxStrataField.SelectedItem.ToString());
        //    //    gadgetOptions.StrataVariableNames = strataVars;
        //    //}

        //    lock (syncLock)
        //    {
        //        //System.ComponentModel.BackgroundWorker singleChartWorker = new System.ComponentModel.BackgroundWorker();
        //        worker = new System.ComponentModel.BackgroundWorker();
        //        worker.DoWork += new System.ComponentModel.DoWorkEventHandler(singleChartWorker_DoWork);
        //        worker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(singleChartWorker_RunWorkerCompleted);
        //        object[] args = new object[2];
        //        args[0] = gadgetOptions;
        //        //args[1] = cbxSingleField.SelectedItem.ToString();
        //        worker.RunWorkerAsync(args);
        //    }
        //}

        private void RenderFinishScatterChart(List<NumericDataValue> dataValues, LinRegressionResults results, NumericDataValue maxValue, NumericDataValue minValue)
        {
            dp = new DashboardPanel();
            //dp.Title = "My Chart Title";
            dp.Theme = "ArcticWhite";
            dp.Style = Resources["DashboardPanelStyle1"] as Style;

            XYChart chart = new XYChart();
            chart.XPath = "IndependentValue";
            chart.DefaultStripesVisible = true;
            chart.HighlightDataPointOnHover = true;
            chart.Theme = "ArcticWhite";
            chart.GlareCoverVisible = true;
            chart.SelectionVisualHint = SelectionVisualHint.InvertedColor;
            chart.Width = dataValues.Count * 3;
            chart.MinWidth = 800.0;
            chart.Height = 500.0;
            chart.EnableDataPointPopup = true;
            chart.EnableAnimation = true;
            chart.AnimationDuration = new TimeSpan(0, 0, 0, 5, 0);
            chart.AnimationOnLoad = false;
            chart.HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
            chart.CoordinatesPaddingPercentage = new Thickness(0, 5, 0, 0);

            //chart.Loaded += new RoutedEventHandler(chart_Loaded);
            //chart.BorderThickness = new Thickness(0);
            //ScatterSeries series = new ScatterSeries();

            //MarkerSeries mseries = new MarkerSeries();
            //mseries.YPath = "IndependentValue";
            //mseries.Marker = MarkerSeries.tri

            var xAxisCoords = new AxisCoordinates();
            //xAxisCoords.Angle = 70.0;
            xAxisCoords.LabelGap = 1.0;
            xAxisCoords.Margin = new Thickness(0, 5, 0, 0);
            //xAxisCoords.MaximumAnnotationLevel = 1;
            xAxisCoords.LabelMargin = 5.0;

            ChartLabel chrtLabel = new ChartLabel();
            chrtLabel.Orientation = ChartLabelOrientation.Horizontal;

            textBlockX = new TextBlock();
            textBlockX.Width = 402.0;
            textBlockX.Text = xAxisVar;
            textBlockX.TextAlignment = TextAlignment.Center;
            textBlockX.Padding = new Thickness(0.0, 10.0, 0.0, 10.0);
            textBlockX.Margin = new Thickness(0.0, 20.0, 0.0, 0.0);
            chrtLabel.Child = textBlockX;

            chart.XAxisArea.Add(xAxisCoords);
            chart.XAxisArea.Add(chrtLabel);

            var yAxisCoords = new AxisCoordinates();

            chrtLabel = new ChartLabel();
            chrtLabel.Orientation = ChartLabelOrientation.Vertical;

            textBlockY = new TextBlock();
            //tb.Padding = new Thickness(20, 10, 0, 0);
            textBlockY.Text = yAxisVar; //"Y-Axis";
            textBlockY.TextAlignment = TextAlignment.Center;
            textBlockY.Height = 36.0;
            textBlockY.Margin = new Thickness(0, 20, 0, 0);
            textBlockY.VerticalAlignment = System.Windows.VerticalAlignment.Bottom;

            chrtLabel.Child = textBlockY;

            chart.YAxisArea.Add(yAxisCoords);
            chart.YAxisArea.Add(chrtLabel);

            MarkerSeriesAnimator msa = MarkerSeriesAnimator.Create("Dropping");

            MarkerSeries mseries = new MarkerSeries();
            mseries.YPath = "DependentValue";
            mseries.XPath = "IndependentValue";
            mseries.Id = "S0";
            mseries.Animator = msa;
            mseries.DataSource = dataValues;

            //LinearAxis xaxis = new LinearAxis();
            //xaxis.Orientation = AxisOrientation.X;
            //xaxis.Title = cbxScatterXAxisField.SelectedItem.ToString();
            //series.IndependentAxis = xaxis;

            //LinearAxis yaxis = new LinearAxis();
            //yaxis.Orientation = AxisOrientation.Y;
            //yaxis.Title = cbxScatterYAxisField.SelectedItem.ToString();
            //series.DependentRangeAxis = yaxis;
            //yaxis.ShowGridLines = true;

            //series.IndependentValuePath = "IndependentValue";
            //series.DependentValuePath = "DependentValue";
            //series.ItemsSource = dataValues;
            //CompositeSeries cseries = new CompositeSeries();
            //cseries.SubSeries.Add(mseries);
            //cseries.SubSeries.Add(lseries);
            //chart.DataSeries.Add(cseries);
            chart.XYChartMainArea.Add(mseries);

            if (results.Variables.Count == 0)
            {
                RenderFinishWithError(SharedStrings.NO_RECORDS_SELECTED);
                return;
            }

            if (results.Variables != null)
            {
                decimal coefficient = Convert.ToDecimal(results.Variables[0].Coefficient);
                decimal constant = Convert.ToDecimal(results.Variables[1].Coefficient);

                NumericDataValue newMaxValue = new NumericDataValue();
                newMaxValue.IndependentValue = maxValue.IndependentValue + 1;
                newMaxValue.DependentValue = (coefficient * maxValue.IndependentValue) + constant;
                NumericDataValue newMinValue = new NumericDataValue();
                newMinValue.IndependentValue = minValue.IndependentValue - 1;
                newMinValue.DependentValue = (coefficient * minValue.IndependentValue) + constant;

                List<NumericDataValue> regresValues = new List<NumericDataValue>();
                regresValues.Add(newMinValue);
                regresValues.Add(newMaxValue);

                LineSeriesAnimator lsa = LineSeriesAnimator.Create("Morph");

                LineSeries regression = new LineSeries();
                //regression.DependentRangeAxis = yaxis;
                //regression.IndependentAxis = xaxis;
                regression.YPath = "DependentValue";
                regression.XPath = "IndependentValue";
                regression.Id = "S1";
                regression.Thickness = 2;
                regression.Animator = lsa;
                regression.DataSource = regresValues;
                chart.XYChartMainArea.Add(regression);

                dp.Content = chart;

                pnlChartContainer.Children.Add(dp);
                //pnlChartContainer.Width = chart.Width;
                //series.LegendItems.Clear();
                //regression.LegendItems.Clear();
                //LineSeries lseries = new LineSeries();
                //lseries.YPath = "DependentValue";
                //lseries.XPath = "IndependentValue";
                //lseries.Id = "S0";
                //lseries.DataSource = dataValues;
                //chart.Height = 600;
                //if (dataValues.Count > 20)
                //    chart.Width = dataValues.Count * 25;
                //else
                //    chart.Width = 800;
                //pnlChartContainer.Children.Clear();
                //pnlChartContainer.Children.Add(chart);
                //panelMain.Visibility = System.Windows.Visibility.Visible;
                //grdChart.Visibility = System.Windows.Visibility.Visible;
                //pnlChartContainer.Visibility = System.Windows.Visibility.Visible;
            }
            else
            {
                pnlChartContainer.Children.Clear();
                RenderFinishWithWarning("Insufficient data to produce this chart.");
            }
            RenderFinish();
        }

        //private void RenderFinishSingleChart(List<List<StringDataValue>> stratifiedValues)
        //{
        //    waitCursor.Visibility = System.Windows.Visibility.Collapsed;
        //    pnlChartContainer.Children.Clear();

        //    Chart chart = new Chart();
        //    chart.Loaded += new RoutedEventHandler(chart_Loaded);
        //    chart.BorderThickness = new Thickness(0);
        //    LinearAxis dependentAxis = new LinearAxis();
        //    dependentAxis.Minimum = 0;
        //    dependentAxis.ShowGridLines = true;

        //    CategoryAxis independentAxis = new CategoryAxis();
        //    independentAxis.Orientation = AxisOrientation.X;

        //    foreach (List<StringDataValue> dataValues in stratifiedValues)
        //    {
        //        DataPointSeries series;
        //        LineSeries avgSeries = null;

        //        switch (((ComboBoxItem)cbxChartType.SelectedItem).Content.ToString())
        //        {
        //            case "Bar":
        //                series = new BarSeries();
        //                dependentAxis.Orientation = AxisOrientation.X;
        //                ((BarSeries)series).DependentRangeAxis = dependentAxis;
        //                break;
        //            case "Column":
        //                series = new ColumnSeries();
        //                if (dashboardHelper.IsColumnText(cbxSingleField.SelectedItem.ToString()) && axisLabelMaxLength > 5)
        //                    independentAxis.AxisLabelStyle = Resources["RotateAxisStyle"] as Style;
        //                ((ColumnSeries)series).IndependentAxis = independentAxis;
        //                dependentAxis.Orientation = AxisOrientation.Y;
        //                ((ColumnSeries)series).DependentRangeAxis = dependentAxis;
        //                break;
        //            case "Pareto":
        //                series = new ColumnSeries();
        //                if (dashboardHelper.IsColumnText(cbxSingleField.SelectedItem.ToString()) && axisLabelMaxLength > 5)
        //                    independentAxis.AxisLabelStyle = Resources["RotateAxisStyle"] as Style;
        //                ((ColumnSeries)series).IndependentAxis = independentAxis;
        //                dependentAxis.Orientation = AxisOrientation.Y;
        //                ((ColumnSeries)series).DependentRangeAxis = dependentAxis;

        //                LinearAxis percentAxis = new LinearAxis();
        //                percentAxis.Minimum = 0;
        //                percentAxis.Maximum = 100;
        //                percentAxis.ShowGridLines = false;
        //                percentAxis.Orientation = AxisOrientation.Y;
        //                percentAxis.Location = AxisLocation.Right;

        //                avgSeries = new LineSeries();
        //                avgSeries.IndependentValuePath = "IndependentValue";
        //                avgSeries.DependentValuePath = "CurrentMeanValue";
        //                avgSeries.Title = "Accumulated %";
        //                avgSeries.DependentRangeAxis = percentAxis;
        //                avgSeries.IndependentAxis = independentAxis;
        //                avgSeries.PolylineStyle = Resources["GooglePolylineStyle"] as Style;
        //                avgSeries.DataPointStyle = Resources["GoogleDataPointStyle"] as Style;

        //                break;
        //            case "Line":
        //                series = new LineSeries();
        //                if (dashboardHelper.IsColumnText(cbxSingleField.SelectedItem.ToString()) && axisLabelMaxLength > 5)
        //                    independentAxis.AxisLabelStyle = Resources["RotateAxisStyle"] as Style;
        //                ((LineSeries)series).IndependentAxis = independentAxis;
        //                dependentAxis.Orientation = AxisOrientation.Y;
        //                ((LineSeries)series).DependentRangeAxis = dependentAxis;
        //                break;
        //            case "Pie":
        //                if (stratifiedValues.Count > 1)
        //                {
        //                    if (dataValues.Count > 0)
        //                    {
        //                        Label lblStrata = new Label();
        //                        lblStrata.Margin = new Thickness(0, 10, 0, 0);
        //                        lblStrata.FontWeight = FontWeights.Bold;
        //                        lblStrata.Content = dataValues[0].StratificationValue;
        //                        pnlChartContainer.Children.Add(lblStrata);
        //                    }
        //                }
        //                chart = new LabeledPieChart();
        //                chart.Loaded += new RoutedEventHandler(chart_Loaded);
        //                chart.BorderThickness = new Thickness(0);
        //                series = new LabeledPieSeries();
        //                series.LegendItemStyle = Resources["PieLegendStyle"] as Style;
        //                ((LabeledPieSeries)series).PieChartLabelStyle = Resources["pieChartLabelStyle"] as Style;
        //                ((LabeledPieSeries)series).PieChartLabelItemTemplate = Resources["pieChartLabelDataTemplate"] as DataTemplate;
        //                ((LabeledPieSeries)series).LabelDisplayMode = DisplayMode.Connected;
        //                chart.LegendTitle = cbxSingleField.SelectedItem.ToString();
        //                chart.Series.Add(series);
        //                chart.Height = 600;
        //                chart.Width = 1024;
        //                chart.Margin = new Thickness(100, 0, 0, 0);
        //                pnlChartContainer.Children.Add(chart);
        //                break;
        //            case "Scatter":
        //                series = new ScatterSeries();
        //                dependentAxis.Orientation = AxisOrientation.Y;
        //                ((ScatterSeries)series).DependentRangeAxis = dependentAxis;
        //                break;
        //            default:
        //                series = new LineSeries();
        //                dependentAxis.Orientation = AxisOrientation.Y;
        //                ((LineSeries)series).DependentRangeAxis = dependentAxis;
        //                break;
        //        }

        //        series.IndependentValuePath = "IndependentValue";
        //        series.DependentValuePath = "DependentValue";

        //        series.ItemsSource = dataValues;
        //        if (avgSeries != null)
        //        {
        //            List<StringDataValue> paretoDataValues = new List<StringDataValue>(dataValues);

        //            double max = 0;

        //            foreach (StringDataValue sdv in paretoDataValues)
        //            {
        //                max = max + sdv.DependentValue;
        //            }

        //            double runningPercent = 0;
        //            foreach (StringDataValue sdv in paretoDataValues)
        //            {
        //                sdv.CurrentMeanValue = ((sdv.DependentValue / max) * 100) + runningPercent;
        //                runningPercent = sdv.CurrentMeanValue;
        //            }

        //            avgSeries.ItemsSource = paretoDataValues;
        //        }

        //        if (stratifiedValues.Count > 1)
        //        {
        //            if (!(series is LabeledPieSeries) && cbxStrataField.SelectedIndex > -1)
        //            {
        //                chart.LegendTitle = cbxStrataField.SelectedItem.ToString();
        //            }

        //            if (dataValues.Count > 0 && !IsBooleanWithNoStratas)
        //            {
        //                series.Title = dataValues[0].StratificationValue.Split('=')[1].Trim();
        //            }
        //            else if (dataValues.Count > 0 && IsBooleanWithNoStratas)
        //            {
        //                series.Title = dataValues[0].StratificationValue.Trim();
        //                chart.LegendTitle = dataValues[0].IndependentValue;
        //            }
        //        }
        //        else
        //        {
        //            series.Title = "Count";
        //        }

        //        if (series is LabeledPieSeries)
        //        {
        //            //chart.Height = 600;
        //            //chart.Width = 800;
        //        }
        //        else if (series is BarSeries)
        //        {
        //            if (dataValues.Count > 30)
        //                chart.Height = dataValues.Count * 30;
        //            else
        //                chart.Height = 600;
        //            chart.Width = 800;
        //        }
        //        else
        //        {
        //            chart.Height = 600;
        //            if (dataValues.Count > 20)
        //                chart.Width = dataValues.Count * 40;
        //            else
        //                chart.Width = 800;
        //        }
        //        if (!(series is LabeledPieSeries))
        //        {
        //            chart.Series.Add(series);
        //            if (avgSeries != null)
        //            {
        //                chart.Series.Add(avgSeries);
        //            }
        //        }

        //        ///////////////////////////////////////////////////////
        //        string chartSize = "Medium";

        //        if (gadgetOptions == null)
        //        {
        //            return;
        //        }

        //        if (gadgetOptions.InputVariableList != null && gadgetOptions.InputVariableList.ContainsKey("chartsize"))
        //        {
        //            chartSize = gadgetOptions.InputVariableList["chartsize"];
        //        }

        //        switch (chartSize)
        //        {
        //            case "Small":
        //                chart.Height = chart.Height / 2;
        //                chart.Width = chart.Width / 2;
        //                break;
        //            case "Medium":
        //                chart.Height = chart.Height / 1.5;
        //                chart.Width = chart.Width / 1.5;
        //                break;
        //        }
        //        ///////////////////////////////////////////////////////
        //    }
        //    if (!((ComboBoxItem)cbxChartType.SelectedItem).Content.ToString().Equals("Pie"))
        //    {
        //        pnlChartContainer.Children.Add(chart);
        //    }
        //}

        //void singleChartWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        //{
        //    if (e.Result != null)
        //    {
        //        List<List<StringDataValue>> stratifiedValues = (List<List<StringDataValue>>)e.Result;
        //        this.Dispatcher.BeginInvoke(new RenderFinishSingleChartDelegate(RenderFinishSingleChart), stratifiedValues);
        //    }
        //}

        //void singleChartWorker_DoWork(object sender, System.ComponentModel.DoWorkEventArgs e)
        //{
        //    try
        //    {
        //        GadgetParameters gadgetOptions = (GadgetParameters)((object[])e.Argument)[0];
        //        string varName = gadgetOptions.MainVariableName;//((object[])e.Argument)[1].ToString();
        //        string singleChartType = string.Empty;
        //        if (gadgetOptions.InputVariableList.ContainsKey("charttype"))
        //        {
        //            singleChartType = gadgetOptions.InputVariableList["charttype"];
        //        }

        //        this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToProcessingState));
        //        this.Dispatcher.BeginInvoke(new SimpleCallback(ClearResults));

        //        RequestUpdateStatusDelegate requestUpdateStatus = new RequestUpdateStatusDelegate(RequestUpdateStatusMessage);
        //        CheckForCancellationDelegate checkForCancellation = new CheckForCancellationDelegate(IsCancelled);

        //        gadgetOptions.GadgetStatusUpdate += new GadgetStatusUpdateHandler(requestUpdateStatus);
        //        gadgetOptions.GadgetCheckForCancellation += new GadgetCheckForCancellationHandler(checkForCancellation);

        //        Dictionary<DataTable, List<DescriptiveStatistics>> res = dashboardHelper.GenerateFrequencyTable(gadgetOptions);

        //        if (res == null || res.Count == 0)
        //        {
        //            this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), SharedStrings.NO_RECORDS_SELECTED);
        //            this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
        //            return;
        //        }
        //        else if (worker.CancellationPending)
        //        {
        //            this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), SharedStrings.DASHBOARD_GADGET_STATUS_OPERATION_CANCELLED);
        //            this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
        //            System.Diagnostics.Debug.Print("Single chart thread was cancelled");
        //            return;
        //        }
        //        else
        //        {
        //            DataTable frequencies = new DataTable();
        //            List<List<StringDataValue>> stratifiedValues = new List<List<StringDataValue>>();
        //            axisLabelMaxLength = 0;

        //            if (GadgetStatusUpdate != null)
        //            {
        //                this.Dispatcher.BeginInvoke(GadgetStatusUpdate, SharedStrings.DASHBOARD_GADGET_STATUS_DISPLAYING_OUTPUT);
        //                System.Threading.Thread.Sleep(150); // necessary to prevent this message from overriding the 'no records selected'
        //            }

        //            if (res.Count == 1 && !singleChartType.Equals("pie") && !singleChartType.Equals("pareto") && dashboardHelper.IsUsingEpiProject && dashboardHelper.View.Fields.Contains(gadgetOptions.MainVariableName) && (dashboardHelper.View.Fields[gadgetOptions.MainVariableName] is YesNoField || dashboardHelper.View.Fields[gadgetOptions.MainVariableName] is CheckBoxField))
        //            {
        //                IsBooleanWithNoStratas = true;
        //            }
        //            else
        //            {
        //                IsBooleanWithNoStratas = false;
        //            }

        //            foreach (KeyValuePair<DataTable, List<DescriptiveStatistics>> tableKvp in res)
        //            {
        //                string strataValue = tableKvp.Key.TableName;

        //                double count = 0;
        //                foreach (DescriptiveStatistics ds in tableKvp.Value)
        //                {
        //                    count = count + ds.observations;
        //                }

        //                if (count == 0)
        //                {
        //                    continue;
        //                }

        //                frequencies = tableKvp.Key;

        //                List<StringDataValue> dataValues = new List<StringDataValue>();
        //                int maxLegendItems = 15;
        //                object[] itemArray = null;

        //                if (singleChartType.Equals("pie") && !frequencies.Columns[0].DataType.ToString().Equals("System.DateTime") && !frequencies.Columns[0].DataType.ToString().Equals("System.Double") && !frequencies.Columns[0].DataType.ToString().Equals("System.Single") && !frequencies.Columns[0].DataType.ToString().Equals("System.Int32"))
        //                {
        //                    int rowStoppedAt = 0;
        //                    DataTable condensedTable = new DataTable(frequencies.TableName);
        //                    condensedTable = frequencies.Clone();

        //                    for (int rule = 0; rule < frequencies.Rows.Count; rule++)
        //                    {
        //                        if (rule < maxLegendItems)
        //                        {
        //                            condensedTable.Rows.Add(frequencies.Rows[rule].ItemArray);
        //                        }
        //                        else if (rule == maxLegendItems)
        //                        {
        //                            condensedTable.Rows.Add("All others", frequencies.Rows[rule][1]);
        //                            rowStoppedAt = rule;
        //                        }
        //                        else
        //                        {
        //                            condensedTable.Rows[maxLegendItems][1] = (double)condensedTable.Rows[rowStoppedAt][1] + (double)frequencies.Rows[rule][1];
        //                        }
        //                    }

        //                    if (condensedTable.Rows.Count > maxLegendItems)
        //                    {
        //                        itemArray = condensedTable.Rows[maxLegendItems].ItemArray;
        //                        DataRow rowToRemove = condensedTable.Rows[maxLegendItems];
        //                        condensedTable.Rows.Remove(rowToRemove);
        //                    }

        //                    frequencies = condensedTable.Copy();

        //                    if (!dashboardHelper.IsColumnYesNo(gadgetOptions.MainVariableName) && !dashboardHelper.IsColumnBoolean(gadgetOptions.MainVariableName))
        //                    {
        //                        frequencies = frequencies.Select(string.Empty, frequencies.Columns[0].ColumnName).CopyToDataTable().DefaultView.ToTable(frequencies.TableName);
        //                    }

        //                    if (itemArray != null)
        //                    {
        //                        frequencies.Rows.Add(itemArray);
        //                    }
        //                }

        //                double sum = 0;
        //                double runningTotal = 0;
        //                double maxValue = 0;

        //                foreach (DataRow row in frequencies.Rows)
        //                {
        //                    double value = (double)row["freq"];
        //                    sum = sum + value;
        //                    if (value > maxValue)
        //                    {
        //                        maxValue = value;
        //                    }
        //                }

        //                if (IsBooleanWithNoStratas)
        //                {
        //                    foreach (DataRow row in frequencies.Rows)
        //                    {
        //                        runningTotal = runningTotal + (double)row["freq"];
        //                        double currentTotal = (runningTotal / sum) * maxValue;
        //                        StringDataValue sdv = new StringDataValue() { DependentValue = (double)row["freq"], IndependentValue = strataValue, StratificationValue = row[varName].ToString(), CurrentMeanValue = currentTotal };
        //                        List<StringDataValue> listSdv = new List<StringDataValue>();
        //                        listSdv.Add(sdv);
        //                        //    dataValues.Add(new StringDataValue() { DependentValue = (double)row["freq"], IndependentValue = row[varName].ToString(), StratificationValue = string.Empty });
        //                        if (row[varName].ToString().Length > axisLabelMaxLength)
        //                        {
        //                            axisLabelMaxLength = row[varName].ToString().Length;
        //                        }
        //                        stratifiedValues.Add(listSdv);
        //                    }
        //                }
        //                else
        //                {
        //                    foreach (DataRow row in frequencies.Rows)
        //                    {
        //                        runningTotal = runningTotal + (double)row["freq"];
        //                        double currentTotal = (runningTotal / sum) * maxValue;
        //                        dataValues.Add(new StringDataValue() { DependentValue = (double)row["freq"], IndependentValue = row[varName].ToString(), StratificationValue = strataValue, CurrentMeanValue = currentTotal });
        //                        if (row[varName].ToString().Length > axisLabelMaxLength)
        //                        {
        //                            axisLabelMaxLength = row[varName].ToString().Length;
        //                        }
        //                    }
        //                    stratifiedValues.Add(dataValues);
        //                }
        //            }
        //            e.Result = stratifiedValues;

        //            if (stratifiedValues.Count == 0)
        //            {
        //                this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), SharedStrings.NO_RECORDS_SELECTED);
        //            }
        //            else
        //            {
        //                this.Dispatcher.BeginInvoke(new SimpleCallback(RenderFinish));
        //            }
        //            this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        this.Dispatcher.BeginInvoke(new RenderFinishWithErrorDelegate(RenderFinishWithError), ex.Message);
        //        this.Dispatcher.BeginInvoke(new SimpleCallback(SetGadgetToFinishedState));
        //    }
        //}

        /// <summary>
        /// Sets the gadget to its 'processing' state
        /// </summary>
        public void SetGadgetToProcessingState()
        {
            //cbxCaseStatusField.IsEnabled = false;
            //cbxDateInterval.IsEnabled = false;
            //cbxChartSize.IsEnabled = false;
            //cbxChartType.IsEnabled = false;
            //cbxColumnAggregateFunc.IsEnabled = false;
            //cbxColumnXAxisField.IsEnabled = false;
            //cbxColumnYAxisField.IsEnabled = false;
            //cbxDateField.IsEnabled = false;
            cbxScatterXAxisField.IsEnabled = false;
            cbxScatterYAxisField.IsEnabled = false;
            //cbxSingleField.IsEnabled = false;
            //cbxStrataField.IsEnabled = false;
            //cbxWeightField.IsEnabled = false;
            //txtXAxisEndValue.IsEnabled = false;
            //txtXAxisStartValue.IsEnabled = false;

            this.IsProcessing = true;
        }

        /// <summary>
        /// Sets the gadget to its 'finished' state
        /// </summary>
        public void SetGadgetToFinishedState()
        {
            //cbxCaseStatusField.IsEnabled = true;
            //cbxDateInterval.IsEnabled = true;
            //cbxChartSize.IsEnabled = true;
            //cbxChartType.IsEnabled = true;
            //cbxColumnAggregateFunc.IsEnabled = true;
            //cbxColumnXAxisField.IsEnabled = true;
            //cbxColumnYAxisField.IsEnabled = true;
            //cbxDateField.IsEnabled = true;
            cbxScatterXAxisField.IsEnabled = true;
            cbxScatterYAxisField.IsEnabled = true;
            //cbxSingleField.IsEnabled = true;
            //cbxStrataField.IsEnabled = true;
            //cbxWeightField.IsEnabled = true;
            //txtXAxisEndValue.IsEnabled = true;
            //txtXAxisStartValue.IsEnabled = true;

            this.IsProcessing = false;

            if (GadgetProcessingFinished != null)
            {
                GadgetProcessingFinished(this);
            }
        }

        /// <summary>
        /// Serializes the gadget into Xml
        /// </summary>
        /// <param name="doc">The Xml docment</param>
        XNode IGadget.Serialize(XDocument doc)
        {
            string caseStatusField = string.Empty;
            string dateField = string.Empty;
            string singleField = string.Empty;
            string weightField = string.Empty;
            string strataField = string.Empty;
            string columnAggregateFunction = string.Empty;
            string xAxisField = string.Empty;
            string yAxisField = string.Empty;
            string xAxisFieldScatter = string.Empty;
            string yAxisFieldScatter = string.Empty;
            string chartType = string.Empty;
            string chartSize = "Medium";
            string xAxisRotation = "90";

            //if (cbxXAxisLabelRotation.SelectedIndex == 0)
            //{
            //    xAxisRotation = "0";
            //}
            //else if (cbxXAxisLabelRotation.SelectedIndex == 1)
            //{
            //    xAxisRotation = "45";
            //}

            //if (cbxChartSize.SelectedIndex >= 0)
            //{
            //    chartSize = ((ComboBoxItem)cbxChartSize.SelectedItem).Content.ToString();
            //}

            // chart type should come first
            //if (cbxChartType.SelectedIndex > -1)
            //{
            chartType = "Scatter";
            //}
            //if (cbxCaseStatusField.SelectedIndex > -1)
            //{
            //    caseStatusField = cbxCaseStatusField.SelectedItem.ToString().Replace("<", "&lt;");
            //}
            //if (cbxDateField.SelectedIndex > -1)
            //{
            //    dateField = cbxDateField.SelectedItem.ToString().Replace("<", "&lt;");
            //}
            //if (cbxColumnXAxisField.SelectedIndex > -1)
            //{
            //    xAxisField = ((EwavColumn)cbxColumnXAxisField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            //}
            //if (cbxColumnYAxisField.SelectedIndex > -1)
            //{
            //    yAxisField = ((EwavColumn)cbxColumnYAxisField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            //}
            //if (cbxSingleField.SelectedIndex > -1)
            //{
            //    singleField = ((EwavColumn)cbxSingleField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            //}
            if (cbxScatterXAxisField.SelectedIndex > -1)
            {
                xAxisFieldScatter = ((EwavColumn)cbxScatterXAxisField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            }
            if (cbxScatterYAxisField.SelectedIndex > -1)
            {
                yAxisFieldScatter = ((EwavColumn)cbxScatterYAxisField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            }
            //if (cbxWeightField.SelectedIndex > -1)
            //{
            //    weightField = ((EwavColumn)cbxWeightField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            //}
            //if (cbxStrataField.SelectedIndex > -1)
            //{
            //    strataField = ((EwavColumn)cbxStrataField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            //}
            //if (cbxColumnAggregateFunc.SelectedIndex > -1)
            //{
            //    columnAggregateFunction = ((ComboBoxItem)cbxColumnAggregateFunc.SelectedItem).Content.ToString();
            //}

            //string xmlString =
            //"<chartType>" + chartType + "</chartType>" +
            //"<chartSize>" + chartSize + "</chartSize>" +
            //"<chartTitle>" + ChartTitle + "</chartTitle>" +
            //"<chartLegendTitle>" + LegendTitle + "</chartLegendTitle>" +
            //"<columnAggregateFunction>" + columnAggregateFunction + "</columnAggregateFunction>" +
            //"<caseStatusVariable>" + caseStatusField + "</caseStatusVariable>" +
            //"<dateVariable>" + dateField + "</dateVariable>" +
            //    // "<dateInterval>" + cbxDateInterval.Text + "</dateInterval>" +
            //"<singleVariable>" + singleField + "</singleVariable>" +
            //"<weightVariable>" + weightField + "</weightVariable>" +
            //"<strataVariable>" + strataField + "</strataVariable>" +
            //"<yAxisVariable>" + yAxisField + "</yAxisVariable>" +
            //"<xAxisVariable>" + xAxisField + "</xAxisVariable>" +
            //    //"<yAxisScatterVariable>" + yAxisFieldScatter + "</yAxisScatterVariable>" +
            //    //"<xAxisScatterVariable>" + xAxisFieldScatter + "</xAxisScatterVariable>" +
            //"<yAxisLabel>" + YAxisLabel + "</yAxisLabel>" +
            //"<xAxisLabel>" + XAxisLabel + "</xAxisLabel>";
            //"<xAxisRotation>" + xAxisRotation + "</xAxisRotation>" +
            //"<xAxisStartValue>" + txtXAxisStartValue.Text + "</xAxisStartValue>" +
            //"<xAxisEndValue>" + txtXAxisEndValue.Text + "</xAxisEndValue>" +
            //"<allValues>" + checkboxAllValues.IsChecked.ToString() + "</allValues>" +
            //"<customHeading>" + CustomOutputHeading + "</customHeading>" +
            //"<customDescription>" + CustomOutputDescription + "</customDescription>" +
            //"<customCaption>" + CustomOutputCaption + "</customCaption>";

            //System.Xml.XmlElement element = doc.CreateElement("chartGadget");
            //element.InnerXml = xmlString;

            //System.Xml.XmlAttribute locationY = doc.CreateAttribute("top");
            //System.Xml.XmlAttribute locationX = doc.CreateAttribute("left");
            //System.Xml.XmlAttribute collapsed = doc.CreateAttribute("collapsed");
            //System.Xml.XmlAttribute type = doc.CreateAttribute("gadgetType");

            //locationY.Value = Canvas.GetTop(this).ToString("F0");
            //locationX.Value = Canvas.GetLeft(this).ToString("F0");
            //collapsed.Value = "false"; // currently no way to collapse the gadget, so leave this 'false' for now
            //type.Value = "EpiDashboard.ChartControl";

            XElement element = new XElement("chart",
                new XAttribute("top", Canvas.GetTop(this).ToString("F2")),
                new XAttribute("left", Canvas.GetLeft(this).ToString("F2")),
                new XAttribute("collapsed", "false"),
                new XAttribute("gadgetType", "Ewav.Scatter"),
                new XElement("chartType", chartType),
                new XElement("chartSize", chartSize),
                new XElement("chartLegendTitle", LegendTitle),
                new XElement("columnAggregateFunction", columnAggregateFunction),
                new XElement("caseStatusVariable", caseStatusField),
                new XElement("dateVariable", dateField),
                new XElement("singleVariable", singleField),
                new XElement("weightVariable", weightField),
                new XElement("strataVariable", strataField),
                new XElement("yAxisVariable", yAxisField),
                new XElement("xAxisVariable", xAxisField),
                new XElement("yAxisScatterVariable", yAxisFieldScatter),
                new XElement("xAxisScatterVariable", xAxisFieldScatter),
                new XElement("yAxisLabel", YAxisLabel),
                new XElement("xAxisLabel", XAxisLabel));

            //element.Attributes.Append(locationY);
            //element.Attributes.Append(locationX);
            //element.Attributes.Append(collapsed);
            //element.Attributes.Append(type);

            //return element;
            return element;
        }

        /// <summary>
        /// Creates the charting gadget from an Xml element
        /// </summary>
        ///// <param name="element">The element from which to create the gadget</param>
        //public void CreateFromXml(XmlElement element)
        //{
        //    this.loadingCombos = true;

        //    foreach (XmlElement child in element.ChildNodes)
        //    {
        //        switch (child.Name.ToLower())
        //        {
        //            case "casestatusvariable":
        //                cbxCaseStatusField.Text = child.InnerText.Replace("&lt;", "<");
        //                break;
        //            case "datevariable":
        //                cbxDateField.Text = child.InnerText.Replace("&lt;", "<");
        //                break;
        //            case "dateinterval":
        //                cbxDateInterval.Text = child.InnerText;
        //                break;
        //            case "yaxisvariable":
        //                cbxColumnYAxisField.Text = child.InnerText.Replace("&lt;", "<");
        //                break;
        //            case "xaxisvariable":
        //                cbxColumnXAxisField.Text = child.InnerText.Replace("&lt;", "<");
        //                break;
        //            case "yaxisscattervariable":
        //                cbxScatterYAxisField.Text = child.InnerText.Replace("&lt;", "<");
        //                break;
        //            case "xaxisscattervariable":
        //                cbxScatterXAxisField.Text = child.InnerText.Replace("&lt;", "<");
        //                break;
        //            case "allvalues":
        //                if (child.InnerText.ToLower().Equals("true"))
        //                {
        //                    checkboxAllValues.IsChecked = true;
        //                }
        //                else
        //                {
        //                    checkboxAllValues.IsChecked = false;
        //                }
        //                break;
        //            case "singlevariable":
        //                cbxSingleField.Text = child.InnerText.Replace("&lt;", "<");
        //                cbxSingleField.Text = child.InnerText.Replace("&lt;", "<");
        //                break;
        //            case "weightvariable":
        //                cbxWeightField.Text = child.InnerText.Replace("&lt;", "<");
        //                cbxWeightField.Text = child.InnerText.Replace("&lt;", "<");
        //                break;
        //            case "stratavariable":
        //                cbxStrataField.Text = child.InnerText.Replace("&lt;", "<");
        //                cbxStrataField.Text = child.InnerText.Replace("&lt;", "<");
        //                break;
        //            case "columnaggregatefunction":
        //                cbxColumnAggregateFunc.Text = child.InnerText;
        //                break;
        //            case "charttype":
        //                cbxChartType.Text = child.InnerText;
        //                break;
        //            case "customheading":
        //                this.CustomOutputHeading = child.InnerText;
        //                break;
        //            case "customdescription":
        //                this.CustomOutputDescription = child.InnerText;
        //                break;
        //            case "customcaption":
        //                this.CustomOutputCaption = child.InnerText;
        //                break;
        //            case "chartsize":
        //                cbxChartSize.SelectedItem = child.InnerText;
        //                break;
        //            case "charttitle":
        //                ChartTitle = child.InnerText;
        //                break;
        //            case "chartlegendtitle":
        //                LegendTitle = child.InnerText;
        //                break;
        //            case "xaxislabel":
        //                XAxisLabel = child.InnerText;
        //                break;
        //            case "yaxislabel":
        //                YAxisLabel = child.InnerText;
        //                break;
        //            case "xaxisstartvalue":
        //                txtXAxisStartValue.Text = child.InnerText;
        //                break;
        //            case "xaxisendvalue":
        //                txtXAxisEndValue.Text = child.InnerText;
        //                break;
        //        }
        //    }

        //    foreach (XmlAttribute attribute in element.Attributes)
        //    {
        //        switch (attribute.Name.ToLower())
        //        {
        //            case "top":
        //                Canvas.SetTop(this, double.Parse(attribute.Value));
        //                break;
        //            case "left":
        //                Canvas.SetLeft(this, double.Parse(attribute.Value));
        //                break;
        //        }
        //    }

        //    this.loadingCombos = false;

        //    RefreshResults();
        //    CollapseExpandConfigPanel();
        //}

        /// <summary>
        /// Converts the gadget's output to Html
        /// </summary>
        /// <returns></returns>
        public string ToHTML(string htmlFileName = "", int count = 0)
        {
            // Check to see if a chart has been created.
            if (pnlChartContainer.ActualHeight == 0 || pnlChartContainer.ActualWidth == 0)
            {
                return string.Empty;
            }

            StringBuilder htmlBuilder = new StringBuilder();

            if (string.IsNullOrEmpty(CustomOutputHeading))
            {
                htmlBuilder.AppendLine("<h2 class=\"gadgetHeading\">Chart</h2>");
            }
            else if (CustomOutputHeading != "(none)")
            {
                htmlBuilder.AppendLine(string.Format("<h2 class=\"gadgetHeading\">{0}</h2>", CustomOutputHeading));
            }

            if (!string.IsNullOrEmpty(CustomOutputDescription))
            {
                htmlBuilder.AppendLine(string.Format("<p class=\"gadgetsummary\">{0}</p>", CustomOutputDescription));
            }

            string imageFileName = string.Empty;

            if (htmlFileName.EndsWith(".html"))
            {
                imageFileName = htmlFileName.Remove(htmlFileName.Length - 5, 5);
            }
            else if (htmlFileName.EndsWith(".htm"))
            {
                imageFileName = htmlFileName.Remove(htmlFileName.Length - 4, 4);
            }

            imageFileName = string.Format("{0}_{1}.png", imageFileName, count.ToString());

            //BitmapSource img = (BitmapSource)Common.ToImageSource(pnlChartContainer);
            //FileStream stream = new FileStream(imageFileName, FileMode.Create);
            ////JpegBitmapEncoder encoder = new JpegBitmapEncoder();
            //PngBitmapEncoder encoder = new PngBitmapEncoder();
            //encoder.Frames.Add(BitmapFrame.Create(img));
            //encoder.Save(stream);
            //stream.Close();

            htmlBuilder.AppendLine(string.Format("<img src=\"{0}\" />", imageFileName));

            return htmlBuilder.ToString();
        }

        public string CustomOutputHeading { get; set; }

        public string CustomOutputDescription { get; set; }

        public string CustomOutputCaption { get; set; }

        public string ChartTitle { get; set; }

        public string LegendTitle { get; set; }

        public string XAxisLabel { get; set; }

        public string YAxisLabel { get; set; }

        #endregion

        /// <summary>
        /// Generate Button Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRun_Click(object sender, RoutedEventArgs e)
        {
            //Serialize(new XDocument());
            DoScatter();
            //sctrViewModel.GenerateTable(
        }

        /// <summary>
        /// Closes the gadget
        /// </summary>
        void CloseGadget()
        {
            DragCanvas dc = this.Parent as DragCanvas;

            Canvas parentCanvas = (Canvas)this.Parent;
            parentCanvas.Children.Remove((UIElement)this);

            UnloadGadget();

            dc.Cleanup(this as UserControl);
        }

        /// <summary>
        /// Close button click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CloseButton_Click(object sender, RoutedEventArgs e)
        {
            CloseGadget();
        }

        private void ResizeButton_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            // TODO: Add event handler implementation here.
            if (spContent.Visibility == System.Windows.Visibility.Visible)
            {
                spContent.Visibility = System.Windows.Visibility.Collapsed;
            }
            else
            {
                spContent.Visibility = System.Windows.Visibility.Visible;
            }
        }
        /// <summary>
        /// Mouse left button down
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LayoutRoot_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
        }

        /// <summary>
        /// Mouse right button down
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LayoutRoot_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            Point p = e.GetSafePosition(cmnClass.GetParentObject<Grid>(this, "LayoutRoot"));
            ((Ewav.DragCanvas)VisualTreeHelper.GetParent(this)).GadgetNameOnRightClick = MyControlName;
            ((Ewav.DragCanvas)VisualTreeHelper.GetParent(this)).StrataList = null;
            ((Ewav.DragCanvas)VisualTreeHelper.GetParent(this)).SelectedGadget = this;
            cmnClass.UpdateZOrder(this, true, cmnClass.GetParentObject<Grid>(this, "LayoutRoot"));
        }

        public string MyControlName
        {
            get
            {
                return "Scatter";
            }
        }

        public string MyUIName
        {
            get
            {
                return "Scatter";
            }
        }

        public ApplicationViewModel ApplicationViewModel
        {
            get
            {
                return this.applicationViewModel;
            }
        }

        void IGadget.CloseGadget()
        {
            CloseGadget();
        }

        public void CreateFromXml(XElement element)
        {
            LoadingCanvas = true;

            //InitializeControl();

            List<EwavColumn> weightColList = cmnClass.GetItemsSource(GetFieldNumericDataType);

            foreach (XElement child in element.Descendants())
            {
                switch (child.Name.ToString().ToLower())
                {
                    //case "casestatusvariable":
                    //    cbxCaseStatusField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "datevariable":
                    //    cbxDateField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "dateinterval":
                    //    cbxDateInterval.Text = child.InnerText;
                    //    break;
                    //case "yaxisvariable":
                    //    cbxColumnYAxisField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "xaxisvariable":
                    //    cbxColumnXAxisField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    case "yaxisscattervariable":
                        cbxScatterYAxisField.SelectedIndex = cmnClass.FindIndexToSelect(weightColList, child.Value.ToString().Replace("&lt;", "<"));
                        Index2 = cbxScatterYAxisField.SelectedIndex;
                        break;
                    case "xaxisscattervariable":
                        cbxScatterXAxisField.SelectedIndex = cmnClass.FindIndexToSelect(weightColList, child.Value.ToString().Replace("&lt;", "<"));
                        Index1 = cbxScatterXAxisField.SelectedIndex;
                        break;
                    //case "allvalues":
                    //    if (child.InnerText.ToLower().Equals("true"))
                    //    {
                    //        checkboxAllValues.IsChecked = true;
                    //    }
                    //    else
                    //    {
                    //        checkboxAllValues.IsChecked = false;
                    //    }
                    //    break;
                    //case "singlevariable":
                    //    cbxSingleField.Text = child.InnerText.Replace("&lt;", "<");
                    //    cbxSingleField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "weightvariable":
                    //    cbxWeightField.Text = child.InnerText.Replace("&lt;", "<");
                    //    cbxWeightField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "stratavariable":
                    //    cbxStrataField.Text = child.InnerText.Replace("&lt;", "<");
                    //    cbxStrataField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "columnaggregatefunction":
                    //    cbxColumnAggregateFunc.Text = child.InnerText;
                    //    break;
                    //case "charttype":
                    //    cbxChartType.Text = child.InnerText;
                    //    break;
                    case "customheading":
                        this.CustomOutputHeading = child.Value;
                        break;
                    case "customdescription":
                        this.CustomOutputDescription = child.Value;
                        break;
                    case "customcaption":
                        this.CustomOutputCaption = child.Value;
                        break;
                    case "chartsize":
                        cbxChartSize.SelectedItem = child.Value;
                        break;
                    case "charttitle":
                        ChartTitle = child.Value;
                        break;
                    case "chartlegendtitle":
                        LegendTitle = child.Value;
                        break;
                    case "xaxislabel":
                        XAxisLabel = child.Value;
                        break;
                    case "yaxislabel":
                        YAxisLabel = child.Value;
                        break;
                    //case "xaxisstartvalue":
                    //    txtXAxisStartValue.Text = child.InnerText;
                    //    break;
                    //case "xaxisendvalue":
                    //    txtXAxisEndValue.Text = child.InnerText;
                    //    break;
                }
            }

            double mouseVerticalPosition = 0.0, mouseHorizontalPosition = 0.0;

            foreach (XAttribute attribute in element.Attributes())
            {
                switch (attribute.Name.ToString().ToLower())
                {
                    case "top":
                        mouseVerticalPosition = double.Parse(element.Attribute("top").Value.ToString());
                        break;
                    case "left":
                        mouseHorizontalPosition = double.Parse(element.Attribute("left").Value.ToString());
                        break;
                }
            }
            LoadingCanvas = false;

            DoScatter();

            cmnClass.AddControlToCanvas(this, mouseVerticalPosition, mouseHorizontalPosition, applicationViewModel.LayoutRoot);
        }

        public bool LoadingCanvas { get; set; }

        public bool LoadingDropDowns { get; set; }
    }
}

namespace Ewav.Web.Services
{
    public partial class ScatterDomainContext
    {
        //This is the place to set RIA Services query timeout. 
        //TimeSpan(0, 5, 0) means five minutes vs the 
        //default 60 sec
        partial void OnCreated()
        {
            if (!DesignerProperties.GetIsInDesignMode(Application.Current.RootVisual))
            {
                ((WebDomainClient<IScatterDomainServiceContract>)DomainClient).ChannelFactory.Endpoint.Binding.SendTimeout =
                    new TimeSpan(0, 120, 0);
            }
        }
    }
}