﻿/*  ----------------------------------------------------------------------------
 *  Emergint Technologies, Inc.
 *  ----------------------------------------------------------------------------
 *  Epi Info™ - Web Analysis & Visualization
 *  ----------------------------------------------------------------------------
 *  File:       XYChartControl.xaml.cs
 *  Namespace:  Ewav    
 *
 *  Author(s):  Daniel Shorter, Mohammad Usman, Ninad Date, Sachin Agnihotri 
 *  Created:    26/08/2013    
 *  Summary:	no summary     
 *  ----------------------------------------------------------------------------
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Linq;
using CommonLibrary;
using ComponentArt.Silverlight.DataVisualization;
using ComponentArt.Silverlight.DataVisualization.Charting;
using ComponentArt.Silverlight.DataVisualization.Common;
using ComponentArt.Silverlight.Export.PDF;
using EpiDashboard;
using Ewav.BAL;
using Ewav.ClientCommon;
using Ewav.Common;
using Ewav.DTO;
using Ewav.ExtensionMethods;
using Ewav.ViewModels;
using Ewav.Web.Services;

namespace Ewav
{
    [PartCreationPolicy(CreationPolicy.NonShared)]
    [Export(typeof(UserControl))]
    [ExportMetadata("type", "chart")]
    [ExportMetadata("tabindex", "13")]
    public partial class XYChartControl : UserControl, IGadget, IEwavGadget, IChartControl
    {
        private readonly char[] SplitTokens = " \t;".ToCharArray();
        private ApplicationViewModel applicationViewModel = ApplicationViewModel.Instance;
        private bool loadingCombos;
        private TextBlock textBlockX, textBlockY;
        private bool isBooleanWithNoStratas = false;
        private GadgetParameters gadgetOptions;
        private object syncLock = new object();
        private object objXAxisStart;
        private object objXAxisEnd;
        DatatableBag databag;
        DashboardPanel dbp;
        XYChartViewModel xyChartViewModel;
        List<string> dateColumnNames = new List<string>();
        List<string> numericColumnNames = new List<string>();
        List<List<StringDataValue>> dataValues = new List<List<StringDataValue>>();
        ClientCommon.Common cmnClass;
        List<EwavColumn> primaryListColumns = null, stratListColumns = null;
        List<ColumnDataType> columnDataType_primary, columnDataType_numeric;
        XYChart XYChartControlChart;
        PieChart PieChart;
        string ErrMessage = "";
        int Index1 = -1, Index2 = -1, Index3 = -1, Index4 = -1, Index5 = -1;
        EwavColumn Col1, Col2, Col3, Col4, Col5;
        private bool chartDataLoaded = false;

        private bool loadingDropDowns = false;

        public bool LoadingDropDowns
        {
            get
            {
                return loadingDropDowns;
            }
            set
            {
                loadingDropDowns = value;
            }
        }

        private bool loadingCanvas = false;

        public bool LoadingCanvas
        {
            get
            {
                return loadingCanvas;
            }
            set
            {
                loadingCanvas = value;
            }
        }

        private List<string> comboBoxItems = new List<string>();

        public List<string> ComboBoxItems
        {
            get
            {
                return comboBoxItems;
            }
            set
            {
                comboBoxItems = value;
            }
        }

        /// <summary>
        /// Toes the HTML.
        /// </summary>
        /// <param name="htmlFileName">Name of the HTML file.</param>
        /// <param name="count">The count.</param>
        /// <returns></returns>
        public string ToHTML(string htmlFileName = "", int count = 0)
        {
            // TODO: Implement this method
            throw new NotImplementedException();
        }

        public event GadgetClosingHandler GadgetClosing;
        public event GadgetProcessingFinishedHandler GadgetProcessingFinished;
        public event GadgetStatusUpdateHandler GadgetStatusUpdate;
        public event GadgetCheckForCancellationHandler GadgetCheckForCancellation;

        private delegate void SetStatusDelegate(string statusMessage);
        private delegate void RequestUpdateStatusDelegate(string statusMessage);
        private delegate bool CheckForCancellationDelegate();
        private delegate void RenderFinishWithErrorDelegate(string errorMessage);
        private delegate void RenderFinishWithWarningDelegate(string errorMessage);
        // private delegate void RenderFinishEpiCurveDelegate(DataTable data, List<List<StringDataValue>> dataValues);
        private delegate void RenderFinishSingleChartDelegate(List<List<StringDataValue>> stratifiedValues);
        //private delegate void RenderFinishScatterChartDelegate(List<NumericDataValue> dataValues, StatisticsRepository.LinearRegression.LinearRegressionResults results, NumericDataValue maxValue, NumericDataValue minValue);
        //private delegate void RenderFinishStackedChartDelegate(List<List<StringDataValue>> dataValues, DataTable data);
        private delegate void SimpleCallback();

        public List<ColumnDataType> GetPrimaryFieldColumnDataType
        {
            get
            {
                columnDataType_primary = new List<ColumnDataType>();

                columnDataType_primary.Add(ColumnDataType.Boolean);
                columnDataType_primary.Add(ColumnDataType.Numeric);
                columnDataType_primary.Add(ColumnDataType.Text);
                columnDataType_primary.Add(ColumnDataType.DateTime);
                columnDataType_primary.Add(ColumnDataType.UserDefined);

                return columnDataType_primary;
            }
        }

        public List<ColumnDataType> GetNumericFieldColumnDataType
        {
            get
            {
                columnDataType_numeric = new List<ColumnDataType>();
                columnDataType_numeric.Add(ColumnDataType.Numeric);

                return columnDataType_numeric;
            }
        }

        XNode IGadget.Serialize(XDocument doc)
        {
            string caseStatusField = string.Empty;
            string dateField = string.Empty;
            string singleField = string.Empty;
            string weightField = string.Empty;
            string strataField = string.Empty;
            string columnAggregateFunction = string.Empty;
            string xAxisField = string.Empty;
            string yAxisField = string.Empty;
            string xAxisFieldScatter = string.Empty;
            string yAxisFieldScatter = string.Empty;
            string chartType = string.Empty;
            string chartSize = "Medium";
            string xAxisRotation = "90";

            //if (cbxXAxisLabelRotation.SelectedIndex == 0)
            //{
            //    xAxisRotation = "0";
            //}
            //else if (cbxXAxisLabelRotation.SelectedIndex == 1)
            //{
            //    xAxisRotation = "45";
            //}

            if (cbxChartSize.SelectedIndex >= 0)
            {
                //chartSize = ((ComboBoxItem)cbxChartSize.SelectedItem).Content.ToString();
                chartSize = cbxChartSize.SelectedValue.ToString();
            }

            // chart type should come first
            //if (cbxChartType.SelectedIndex > -1)
            //{
            chartType = this.chartTypeEnum.ToString();
            //}
            //if (cbxCaseStatusField.SelectedIndex > -1)
            //{
            //    caseStatusField = cbxCaseStatusField.SelectedItem.ToString().Replace("<", "&lt;");
            //}
            //if (cbxDateField.SelectedIndex > -1)
            //{
            //    dateField = cbxDateField.SelectedItem.ToString().Replace("<", "&lt;");
            //}
            if (cbxColumnXAxisField.SelectedIndex > -1)
            {
                xAxisField = ((EwavColumn)cbxColumnXAxisField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            }
            if (cbxColumnYAxisField.SelectedIndex > -1)
            {
                yAxisField = ((EwavColumn)cbxColumnYAxisField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            }
            if (cbxSingleField.SelectedIndex > -1)
            {
                singleField = ((EwavColumn)cbxSingleField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            }
            //if (cbxScatterXAxisField.SelectedIndex > -1)
            //{
            //    xAxisFieldScatter = cbxScatterXAxisField.SelectedItem.ToString().Replace("<", "&lt;");
            //}
            //if (cbxScatterYAxisField.SelectedIndex > -1)
            //{
            //    yAxisFieldScatter = cbxScatterYAxisField.SelectedItem.ToString().Replace("<", "&lt;");
            //}
            if (cbxWeightField.SelectedIndex > -1)
            {
                weightField = ((EwavColumn)cbxWeightField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            }
            if (cbxStrataField.SelectedIndex > -1)
            {
                strataField = ((EwavColumn)cbxStrataField.SelectedItem).Name.ToString().Replace("<", "&lt;");
            }
            if (cbxColumnAggregateFunc.SelectedIndex > -1)
            {
                columnAggregateFunction = ((ComboBoxItem)cbxColumnAggregateFunc.SelectedItem).Content.ToString();
            }

            //string xmlString =
            //"<chartType>" + chartType + "</chartType>" +
            //"<chartSize>" + chartSize + "</chartSize>" +
            //"<chartTitle>" + ChartTitle + "</chartTitle>" +
            //"<chartLegendTitle>" + LegendTitle + "</chartLegendTitle>" +
            //"<columnAggregateFunction>" + columnAggregateFunction + "</columnAggregateFunction>" +
            //"<caseStatusVariable>" + caseStatusField + "</caseStatusVariable>" +
            //"<dateVariable>" + dateField + "</dateVariable>" +
            //    // "<dateInterval>" + cbxDateInterval.Text + "</dateInterval>" +
            //"<singleVariable>" + singleField + "</singleVariable>" +
            //"<weightVariable>" + weightField + "</weightVariable>" +
            //"<strataVariable>" + strataField + "</strataVariable>" +
            //"<yAxisVariable>" + yAxisField + "</yAxisVariable>" +
            //"<xAxisVariable>" + xAxisField + "</xAxisVariable>" +
            //    //"<yAxisScatterVariable>" + yAxisFieldScatter + "</yAxisScatterVariable>" +
            //    //"<xAxisScatterVariable>" + xAxisFieldScatter + "</xAxisScatterVariable>" +
            //"<yAxisLabel>" + YAxisLabel + "</yAxisLabel>" +
            //"<xAxisLabel>" + XAxisLabel + "</xAxisLabel>";
            //"<xAxisRotation>" + xAxisRotation + "</xAxisRotation>" +
            //"<xAxisStartValue>" + txtXAxisStartValue.Text + "</xAxisStartValue>" +
            //"<xAxisEndValue>" + txtXAxisEndValue.Text + "</xAxisEndValue>" +
            //"<allValues>" + checkboxAllValues.IsChecked.ToString() + "</allValues>" +
            //"<customHeading>" + CustomOutputHeading + "</customHeading>" +
            //"<customDescription>" + CustomOutputDescription + "</customDescription>" +
            //"<customCaption>" + CustomOutputCaption + "</customCaption>";

            //System.Xml.XmlElement element = doc.CreateElement("chartGadget");
            //element.InnerXml = xmlString;

            //System.Xml.XmlAttribute locationY = doc.CreateAttribute("top");
            //System.Xml.XmlAttribute locationX = doc.CreateAttribute("left");
            //System.Xml.XmlAttribute collapsed = doc.CreateAttribute("collapsed");
            //System.Xml.XmlAttribute type = doc.CreateAttribute("gadgetType");

            //locationY.Value = Canvas.GetTop(this).ToString("F0");
            //locationX.Value = Canvas.GetLeft(this).ToString("F0");
            //collapsed.Value = "false"; // currently no way to collapse the gadget, so leave this 'false' for now
            //type.Value = "EpiDashboard.ChartControl";

            XElement element = new XElement("chart",
                new XAttribute("top", Canvas.GetTop(this).ToString("F2")),
                new XAttribute("left", Canvas.GetLeft(this).ToString("F2")),
                new XAttribute("collapsed", "false"),
                new XAttribute("gadgetType", "Ewav.XYChartControl"),
                new XElement("chartType", chartType),
                new XElement("chartSize", chartSize),
                new XElement("chartLegendTitle", LegendTitle),
                new XElement("columnAggregateFunction", columnAggregateFunction),
                new XElement("caseStatusVariable", caseStatusField),
                new XElement("dateVariable", dateField),
                new XElement("singleVariable", singleField),
                new XElement("weightVariable", weightField),
                new XElement("strataVariable", strataField),
                new XElement("yAxisVariable", yAxisField),
                new XElement("xAxisVariable", xAxisField),
                new XElement("yAxisLabel", YAxisLabel),
                new XElement("xAxisLabel", XAxisLabel));

            //element.Attributes.Append(locationY);
            //element.Attributes.Append(locationX);
            //element.Attributes.Append(collapsed);
            //element.Attributes.Append(type);

            //return element;
            return element;
        }

        public class StringDataValue
        {
            public double DependentValue { get; set; }
            public string IndependentValue { get; set; }
            public string StratificationValue { get; set; }
            public double CurrentMeanValue { get; set; }
        }

        public class NumericDataValue
        {
            public decimal DependentValue { get; set; }
            public decimal IndependentValue { get; set; }
        }

        public class TypeStringTuple
        {
            public Type Item1 { get; set; }
            public string Item2 { get; set; }
            public TypeStringTuple(Type item1, string item2)
            {
                this.Item1 = item1;
                this.Item2 = item2;
            }
        }

        private string chartName;

        public string ChartName
        {
            get
            {
                if (this.chartName == null)
                {
                    return "XYChart";
                }
                else
                {
                    return this.chartName;
                }
            }
            set
            {
                this.chartName = value;
                this.chartTypeEnum = (XYControlChartTypes)Utils.ParseEnum<XYControlChartTypes>(this.chartName.Replace(" ", ""));
            }
        }

        public XYChartControl()
        {
            cmnClass = new ClientCommon.Common();
            this.InitializeComponent();

            this.gadgetOptions = new GadgetParameters();
            this.gadgetOptions.ShouldIncludeFullSummaryStatistics = false;
            this.gadgetOptions.ShouldIncludeMissing = false;
            this.gadgetOptions.ShouldSortHighToLow = false;
            this.gadgetOptions.ShouldUseAllPossibleValues = false;
            this.gadgetOptions.StrataVariableList = new List<MyString>();

            this.FillDropDowns();

            this.Loaded += new RoutedEventHandler(XYChartControl_Loaded);
            ComboBoxItems.Clear();
            ComboBoxItems.Add("Small");
            ComboBoxItems.Add("Medium");
            ComboBoxItems.Add("Large");
            cbxChartSize.ItemsSource = ComboBoxItems;
            cbxChartSize.SelectedIndex = 0;
            //this.GName.Text = this.ChartName.FromCamelCase();
            //this.InitializeXYControl();
        }

        ScaleTransform scaleTransform;

        void XYChartControl_Loaded(object sender, RoutedEventArgs e)
        {
            InitializeXYControl();
            //DoChart();
            this.EnableDisableButton();
            ShowAppropriateChart();
        }

        public void InitializeXYControl()
        {
            scaleTransform = new ScaleTransform();
            applicationViewModel.ApplyDataFilterEvent += new Client.Application.ApplyFilterEventHandler(applicationViewModel_ApplyDataFilterEvent);
            applicationViewModel.DefinedVariableAddedEvent += new Client.Application.DefinedVariableAddedEventHandler(applicationViewModel_DefinedVariableAddedEvent);
            applicationViewModel.DefinedVariableInUseDeletedEvent += new Client.Application.DefinedVariableInUseDeletedEventHandler(applicationViewModel_DefinedVariableInUseDeletedEvent);
            applicationViewModel.DefinedVariableNotInUseDeletedEvent += new Client.Application.DefinedVariableNotInUseDeletedEventHandler(applicationViewModel_DefinedVariableNotInUseDeletedEvent);
            applicationViewModel.PreColumnChangedEvent += new Client.Application.PreColumnChangedEventHandler(applicationViewModel_PreColumnChangedEvent);

            applicationViewModel.UnloadedEvent += new Client.Application.UnloadedEventHandler(applicationViewModel_UnloadedEvent);

            try
            {
                DatatableBag eCrvData = new DatatableBag();
                this.xyChartViewModel = (XYChartViewModel)this.DataContext;
                this.xyChartViewModel.FrequencyTableLoadedEvent += new EventHandler<SimpleMvvmToolkit.NotificationEventArgs<Exception>>(XYChartControlViewModel_FrequencyTableLoadedEvent);
                this.xyChartViewModel.ErrorNotice += new EventHandler<SimpleMvvmToolkit.NotificationEventArgs<Exception>>(xyChartViewModel_ErrorNotice);
            }
            catch (Exception)
            {
                throw;
            }
        }

        void applicationViewModel_UnloadedEvent(object o)
        {
            UnloadGadget();
        }

        private void UnloadGadget()
        {
            applicationViewModel.ApplyDataFilterEvent -= new Client.Application.ApplyFilterEventHandler(applicationViewModel_ApplyDataFilterEvent);
            applicationViewModel.DefinedVariableAddedEvent -= new Client.Application.DefinedVariableAddedEventHandler(applicationViewModel_DefinedVariableAddedEvent);
            applicationViewModel.DefinedVariableInUseDeletedEvent -= new Client.Application.DefinedVariableInUseDeletedEventHandler(applicationViewModel_DefinedVariableInUseDeletedEvent);
            applicationViewModel.DefinedVariableNotInUseDeletedEvent -= new Client.Application.DefinedVariableNotInUseDeletedEventHandler(applicationViewModel_DefinedVariableNotInUseDeletedEvent);
            applicationViewModel.PreColumnChangedEvent -= new Client.Application.PreColumnChangedEventHandler(applicationViewModel_PreColumnChangedEvent);
        }

        private void ShowAppropriateChart()
        {
            SetChartTitle(this.ChartName);

            if (!LoadingCanvas)
            {
                if (this.chartTypeEnum == XYControlChartTypes.StackedColumn)
                {
                    this.pnlStackedColumnConfig.Visibility = System.Windows.Visibility.Visible;
                    this.pnlSingleConfig.Visibility = System.Windows.Visibility.Collapsed;
                }
                else
                {
                    this.pnlStackedColumnConfig.Visibility = System.Windows.Visibility.Collapsed;
                    this.pnlSingleConfig.Visibility = System.Windows.Visibility.Visible;
                }

                if (this.chartTypeEnum == XYControlChartTypes.Pareto)
                {
                    this.cbxStrataField.Visibility = System.Windows.Visibility.Collapsed;
                    this.tblockStrataField.Visibility = System.Windows.Visibility.Collapsed;
                }
            }
        }

        void applicationViewModel_PreColumnChangedEvent(object o)
        {
            //throw new NotImplementedException();
            SaveColumnValues();
        }

        void applicationViewModel_DefinedVariableNotInUseDeletedEvent(object o)
        {
            SearchIndex();
            LoadingDropDowns = true;
            FillDropDowns();
            LoadingDropDowns = false;
            DoChart();
        }

        void applicationViewModel_DefinedVariableInUseDeletedEvent(object o)
        {
            //throw new NotImplementedException();
            //if ((FieldSelectedCol1 != null || Col2 != null ||  CrossTabSelectedCol3 != null) 
            //    || (StrataSelectedCol4 != null || Col5 != null))
            //{
            if (IsDFUsedInThisGadget())
            {
                ResetGadget();
            }
            else
            {
                SearchIndex();
            }
            LoadingDropDowns = true;
            FillDropDowns();
            LoadingDropDowns = false;
        }

        private bool IsDFUsedInThisGadget()
        {
            return (Col1 != null && Col1.Name == applicationViewModel.ItemToBeRemoved.Name) ||
                            (Col2 != null && Col2.Name == applicationViewModel.ItemToBeRemoved.Name) ||
                            (Col3 != null && Col3.Name == applicationViewModel.ItemToBeRemoved.Name) ||
                            (Col4 != null && Col4.Name == applicationViewModel.ItemToBeRemoved.Name) ||
                            (Col5 != null && Col5.Name == applicationViewModel.ItemToBeRemoved.Name);
        }

        private void ResetGadget()
        {
            SearchIndex();
            if (IsDFUsedInThisGadget())
            {
                Index1 = Index2 = Index3 = Index4 = Index5 = -1;
                pnlChartContainer.Visibility = System.Windows.Visibility.Collapsed;
            }
            LoadingDropDowns = true;
            FillDropDowns();
            LoadingDropDowns = false;

        }


        void applicationViewModel_DefinedVariableAddedEvent(object o)
        {
            //throw new NotImplementedException();
            SearchIndex();
            LoadingDropDowns = true;
            FillDropDowns();
            LoadingDropDowns = false;
            DoChart();
        }

        /// <summary>
        /// Searches for Selected index of all the columns
        /// </summary>
        private void SearchIndex()
        {
            if (Col1 != null)
            {
                Index1 = cmnClass.SearchCurrentIndex(Col1, cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType)); // SearchCurrentIndex(Col1);
            }
            if (Col2 != null)
            {
                Index2 = cmnClass.SearchCurrentIndex(Col2, cmnClass.GetItemsSource(GetNumericFieldColumnDataType));  //SearchCurrentIndex(Col2);
            }
            if (Col3 != null)
            {
                Index3 = cmnClass.SearchCurrentIndex(Col3, cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType, false));  //SearchCurrentIndex(Col3);
            }
            if (Col4 != null)
            {
                Index4 = cmnClass.SearchCurrentIndex(Col4, cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType));  //SearchCurrentIndex(Col4);
            }
            if (Col5 != null)
            {
                Index5 = cmnClass.SearchCurrentIndex(Col5, cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType, false));  //SearchCurrentIndex(Col5);
            }
        }

        /// <summary>
        /// Saves columns values
        /// </summary>
        private void SaveColumnValues()
        {
            Col1 = (EwavColumn)cbxSingleField.SelectedItem;
            Col2 = (EwavColumn)cbxWeightField.SelectedItem;
            Col3 = (EwavColumn)cbxStrataField.SelectedItem;
            Col4 = (EwavColumn)cbxColumnXAxisField.SelectedItem;
            Col5 = (EwavColumn)cbxColumnYAxisField.SelectedItem;
        }

        /// <summary>
        /// Searchs for current index of selected column
        /// </summary>
        /// <param name="Column"></param>
        /// <returns></returns>
        //private int SearchCurrentIndex(EwavColumn Column)
        //{
        //    List<EwavColumn> SourceColumns = this.applicationViewModel.EwavSelectedDatasource.AllColumns;

        //    IEnumerable<EwavColumn> CBXFieldCols = from cols in SourceColumns
        //                                           orderby cols.Name
        //                                           select cols;

        //    List<EwavColumn> colsList = CBXFieldCols.ToList();

        //    for (int i = 0; i < colsList.Count; i++)
        //    {
        //        if (Column.Name == colsList[i].Name)
        //        {
        //            return i;//+ 1;
        //        }
        //    }
        //    return 0;
        //}

        void xyChartViewModel_ErrorNotice(object sender, SimpleMvvmToolkit.NotificationEventArgs<Exception> e)
        {
            //if (e.Data != null && e.Data.Message.Length > 0)
            //{
            //    ChildWindow window = new ErrorWindow(e.Data);
            //    window.Show();
            //    //return;
            //}
            RenderFinishWithError(e.Data.Message);
            this.SetGadgetToFinishedState();
        }

        void applicationViewModel_ApplyDataFilterEvent(object o)
        {
            if (applicationViewModel.RemoveIndicator &&
                IsDFUsedInThisGadget())
            {
                ResetGadget();
            }
            else
            {
                DoChart();
            }
        }



        void XYChartControlViewModel_FrequencyTableLoadedEvent(object sender, SimpleMvvmToolkit.NotificationEventArgs<Exception> e)
        {
            chartDataLoaded = true;

            this.GenerateXYChartControl();
        }

        void FillDropDowns()
        {
            //IEnumerable<EwavColumn> CBXFieldCols = from cols in this.applicationViewModel.EwavSelectedDatasource.AllColumns    //  eCrvViewModel.Columns
            //                                       where columnDataType.Contains(cols.SqlDataTypeAsString)
            //                                       orderby cols.Name
            //                                       select cols;
            //List<EwavColumn> colsList = CBXFieldCols.ToList();
            //colsList.Insert(0, new EwavColumn() { Name = " ", Index = -1, NoCamelName = " " });
            primaryListColumns = cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType);
            this.cbxSingleField.ItemsSource = primaryListColumns;
            this.cbxSingleField.SelectedValue = "Index";
            //this.cbxSingleField.DisplayMemberPath = "NoCamelName";
            this.cbxSingleField.DisplayMemberPath = "Name";
            this.cbxSingleField.SelectedIndex = Index1;

            this.cbxColumnXAxisField.ItemsSource = primaryListColumns;
            this.cbxColumnXAxisField.SelectedValue = "Index";
            //this.cbxColumnXAxisField.DisplayMemberPath = "NoCamelName";
            this.cbxColumnXAxisField.DisplayMemberPath = "Name";
            this.cbxColumnXAxisField.SelectedIndex = Index4;

            //foreach (var item in primaryListColumns)
            //{
            //    this.dateColumnNames.Add(item.Name);
            //}

            stratListColumns = cmnClass.GetItemsSource(GetNumericFieldColumnDataType);

            //colsList.Insert(0, new EwavColumn() { Name = " ", Index = -1 });
            //stratListColumns.Add(ColumnDataType.Numeric);
            //columnDataType.Add(ColumnDataType.Boolean);
            //columnDataType.Add(ColumnDataType.Text);

            //CBXFieldCols = from cols in this.applicationViewModel.EwavSelectedDatasource.AllColumns
            //               where columnDataType.Contains(cols.SqlDataTypeAsString)
            //               orderby cols.Name
            //               select cols;
            //List<EwavColumn> CaseStatusField = CBXFieldCols.ToList();

            //CaseStatusField.Insert(0, new EwavColumn() { Name = " ", Index = -1, NoCamelName = " " });

            this.cbxWeightField.ItemsSource = stratListColumns;
            this.cbxWeightField.SelectedValue = "Index";
            //this.cbxWeightField.DisplayMemberPath = "NoCamelName";
            this.cbxWeightField.DisplayMemberPath = "Name";
            this.cbxWeightField.SelectedIndex = Index2;

            primaryListColumns = cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType, false);

            this.cbxStrataField.ItemsSource = primaryListColumns;
            this.cbxStrataField.SelectedValue = "Index";
            //this.cbxStrataField.DisplayMemberPath = "NoCamelName";
            this.cbxStrataField.DisplayMemberPath = "Name";
            this.cbxStrataField.SelectedIndex = Index3;

            this.cbxColumnYAxisField.ItemsSource = primaryListColumns;
            this.cbxColumnYAxisField.SelectedValue = "Index";
            //this.cbxColumnYAxisField.DisplayMemberPath = "NoCamelName";
            this.cbxColumnYAxisField.DisplayMemberPath = "Name";
            this.cbxColumnYAxisField.SelectedIndex = Index5;
            //ewc = new EwavColumn();
            //ewc.Name = " ";
            //dateFields.Insert(0, ewc);
            //CaseStatusField.Insert(0, new EwavColumn() { Name = " ", Index = -1 });
            //cbxCaseStatusField.ItemsSource = CaseStatusField;
            //cbxCaseStatusField.SelectedValue = "Index";
            //cbxCaseStatusField.DisplayMemberPath = "Name";
            //foreach (var item in stratListColumns)
            //{
            //    this.numericColumnNames.Add(item.Name);
            //}
        }

        bool ValidateComboBoxes()
        {
            StackPanel stackpnl = null;
            bool indicator = true;
            if (pnlSingleConfig.Visibility == System.Windows.Visibility.Visible)
            {
                stackpnl = pnlSingleConfig;
            }
            else
            {
                stackpnl = pnlStackedColumnConfig;
            }
            Dictionary<string, string> testDict = new Dictionary<string, string>();
            foreach (var item in stackpnl.Children)
            {
                if (item is ComboBox && ((ComboBox)item).SelectedIndex > 0 &&
                    !((ComboBox)item).Name.ToString().Contains("ChartSize") &&
                    !((ComboBox)item).Name.ToString().Contains("ColumnAggregateFunc"))
                {
                    try
                    {
                        testDict.Add(((EwavColumn)((ComboBox)item).SelectedItem).Name, ((EwavColumn)((ComboBox)item).SelectedItem).Name);
                    }
                    catch (Exception ex)
                    {
                        RenderFinishWithError(ex.Message);
                        return false;
                    }
                }
                if (item is ComboBox &&
                    (((ComboBox)item).Name == "cbxSingleField" || ((ComboBox)item).Name == "cbxColumnXAxisField") &&
                    ((ComboBox)item).SelectedIndex < 0)
                {
                    indicator = false;
                }
            }
            return indicator;
        }

        void CloseGadget()
        {
            this.gadgetOptions = null;
            //           chart = null;

            DragCanvas dc = this.Parent as DragCanvas;

            Canvas parentCanvas = (Canvas)this.Parent;
            parentCanvas.Children.Remove((UIElement)this);
            UnloadGadget();
            dc.Cleanup(this as UserControl);
        }

        SetLabels window;

        public void SetChartLabels()
        {
            this.window = new SetLabels { DataContext = this.DataContext };

            this.window.Unloaded -= new RoutedEventHandler(window_Unloaded);
            this.window.Unloaded += new RoutedEventHandler(window_Unloaded);
            this.window.Show();
        }

        public void SetChartTitle(string chartTitle)
        {
            this.ChartName = chartTitle;
            this.GName.Text = this.ChartName;
        }

        public void SetChartLabels(string chartTitle, string legendTitle, string xAxisLabel, string yAxisLabel)
        {
            //var window = new SetLabels { DataContext = this.DataContext };
            //window.Unloaded += new RoutedEventHandler(window_Unloaded);
            //window.Unloaded += new RoutedEventHandler(window_Unloaded);
            //window.Show();
            //ExportToPdf etp = new ExportToPdf();
            //ExportToPDF etp = new ExportToPDF();
            //etp.SavePNG(current)
            if (this.pnlChartContainer.Children.Count > 0 && this.pnlChartContainer.Children[0] is XYChart)
            {
                XYChart currentChart = (XYChart)this.pnlChartContainer.Children[0];
                //etp.SavePNG(currentChart, 600);
                if (currentChart.Axes.Count > 0)
                {
                    //if (((DisplayAxis)currentChart.Axes[0]).Orientation == AxisOrientation.Y)
                    //{
                    //    ((DisplayAxis)currentChart.Axes[0]).Title = yAxisLabel;
                    //    ((DisplayAxis)currentChart.Axes[1]).Title = xAxisLabel;
                    //}
                    //else
                    //{
                    //    ((DisplayAxis)currentChart.ActualAxes[0]).Title = xAxisLabel;
                    //    ((DisplayAxis)currentChart.ActualAxes[1]).Title = yAxisLabel;
                    //}
                    //YAxisLabel = yAxisLabel;
                    // XAxisLabel = xAxisLabel;
                }
                //currentChart.= chartTitle;
                //ChartTitle = chartTitle
                //if (!string.IsNullOrEmpty(legendTitle))
                //{
                //    currentChart.Legend = new Legend().co
                //    LegendTitle = legendTitle;
                //}
                //else
                //{
                //    currentChart.LegendTitle = null;
                //    LegendTitle = string.Empty;
                //}
            }
        }

        void window_Unloaded(object sender, RoutedEventArgs e)
        {
            if (this.window.DialogResult.Value)
            {
                this.SetValuesForAxis();
            }
        }

        private void SetValuesForAxis()
        {
            if (textBlockX != null && textBlockY != null && dbp != null)
            {
                this.textBlockX.Text = this.window.txtboxXaxis.Text;
                this.textBlockY.Text = this.window.txtboxYaxis.Text;
                //txtChartTitle.Text = window.txtbxChrtTitle.Text;
                this.dbp.Title = this.window.txtbxChrtTitle.Text;
            }
        }

        public void SaveAsImage()
        {
            ExportToPDF etp = new ExportToPDF();
            // XYChart currentChart = (XYChart)dp.Content;// (XYChart)pnlChartContainer.Children[0];//
            DashboardPanel currentChart = this.dbp;
            etp.SavePNG(currentChart, 100);
        }

        private void ResetComboboxes()
        {
            this.loadingCombos = true;

            //if (cbxCaseStatusField.Items.Count > 0)
            //    cbxCaseStatusField.SelectedIndex = -1;
            //if (cbxDateField.Items.Count > 0)
            //    cbxDateField.SelectedIndex = -1;
            //if (cbxSingleField.Items.Count > 0)
            //    cbxSingleField.SelectedIndex = -1;

            this.loadingCombos = false;
        }

        private void CheckAndSetPosition()
        {
            double top = Canvas.GetTop(this);
            double left = Canvas.GetLeft(this);

            if (top < 0)
            {
                Canvas.SetTop(this, 0);
            }
            if (left < 0)
            {
                Canvas.SetLeft(this, 0);
            }
        }

        /// <summary>
        /// Sends the gadget to the back of the canvas
        /// </summary>
        //private void SendToBack()
        //{
        //    Canvas.SetZIndex(this, -1);
        //}

        private void RenderFinish()
        {
            this.waitCursor.Visibility = System.Windows.Visibility.Collapsed;
            spContent.Visibility = System.Windows.Visibility.Visible;
            this.pnlChartContainer.Visibility = System.Windows.Visibility.Visible;

            this.pnlStatus.Visibility = System.Windows.Visibility.Collapsed;
            this.txtStatus.Text = string.Empty;

            this.SetChartLabels(this.ChartTitle, this.LegendTitle, this.XAxisLabel, this.YAxisLabel);

            this.CheckAndSetPosition();

            string chartSize = "Large";

            if (this.gadgetOptions.InputVariableList != null && this.gadgetOptions.InputVariableList.ContainsKey("chartsize"))
            {
                chartSize = this.gadgetOptions.InputVariableList["chartsize"];
            }

            ScaleTransform st = this.RenderTransform as ScaleTransform;

            switch (this.chartTypeEnum)
            {
                case XYControlChartTypes.StackedColumn:
                case XYControlChartTypes.Line:
                case XYControlChartTypes.Column:
                case XYControlChartTypes.Bar:
                case XYControlChartTypes.Pareto:
                    switch (chartSize)
                    {
                        case "Medium":
                            this.XYChartControlChart.Height = this.XYChartControlChart.Height * 1.15 + 100;
                            this.XYChartControlChart.Width = this.XYChartControlChart.Width * 1.15 + 100;
                            break;
                        case "Large":
                            this.XYChartControlChart.Height = this.XYChartControlChart.Height * 1.3 + 100;
                            this.XYChartControlChart.Width = this.XYChartControlChart.Width * 1.3 + 100;

                            break;
                    }
                    break;
                case XYControlChartTypes.Pie:
                    switch (chartSize)
                    {
                        case "Medium":

                            this.PieChart.Height = this.PieChart.Height * 1.15 + 100;
                            this.PieChart.Width = this.PieChart.Width * 1.15 + 100;
                            break;
                        case "Large":
                            this.PieChart.Height = this.PieChart.Height * 1.3 + 100;
                            this.PieChart.Width = this.PieChart.Width * 1.3 + 100;
                            break;
                    }
                    break;
                default:
                    throw new Exception(string.Format("Unhandled chart type {0}", chartTypeEnum.ToString()));
            }
        }

        private void RenderFinishWithWarning(string errorMessage)
        {
            this.waitCursor.Visibility = System.Windows.Visibility.Collapsed;
            spContent.Visibility = System.Windows.Visibility.Visible;
            this.pnlChartContainer.Visibility = System.Windows.Visibility.Visible;

            //pnlStatus.Background = Brushes.Gold;
            //pnlStatusTop.Background = Brushes.Goldenrod;

            this.pnlStatus.Visibility = System.Windows.Visibility.Visible;
            this.txtStatus.Text = errorMessage;

            this.SetChartLabels(this.ChartTitle, this.LegendTitle, this.XAxisLabel, this.YAxisLabel);

            this.CheckAndSetPosition();
        }

        private void RenderFinishWithError(string errorMessage)
        {
            this.waitCursor.Visibility = System.Windows.Visibility.Collapsed;
            spContent.Visibility = System.Windows.Visibility.Visible;
            this.pnlChartContainer.Visibility = System.Windows.Visibility.Collapsed;

            //pnlStatus.Background = Brushes.Tomato;
            //pnlStatusTop.Background = Brushes.Red;

            this.pnlStatus.Visibility = System.Windows.Visibility.Visible;

            this.txtStatus.Text = errorMessage;

            this.CheckAndSetPosition();
        }

        private void RequestUpdateStatusMessage(string statusMessage)
        {
            //this.Dispatcher.BeginInvoke(new SetStatusDelegate(SetStatusMessage), statusMessage);
            this.SetStatusMessage(statusMessage);
        }

        private void SetStatusMessage(string statusMessage)
        {
            this.pnlStatus.Visibility = System.Windows.Visibility.Visible;
            this.txtStatus.Text = statusMessage;
        }

        private bool IsCancelled()
        {
            //if (worker != null && worker.WorkerSupportsCancellation && worker.CancellationPending)
            //{
            //    return true;
            //}
            //else
            //{
            return false;
            //}
        }

        //private void FillComboboxes(bool update = false)
        //{
        //  List<string> caseStatusFields = new List<string>();
        //  List<string> dateFields = new List<string>();
        //  List<string> numericFields = new List<string>();
        //  List<string> allFields = new List<string>();
        //  List<ColumnDataType> columnDataType = new List<ColumnDataType>();
        //  caseStatusFields.Add(string.Empty);

        //  if (update)
        //  {
        //    //cbxDateField.SelectedItem = prevDateField;
        //    //cbxCaseStatusField.SelectedItem = prevCaseStatusField;
        //    //cbxColumnXAxisField.SelectedItem = prevColumnXAxisField;
        //    //cbxColumnYAxisField.SelectedItem = prevColumnYAxisField;
        //    //cbxScatterXAxisField.SelectedItem = prevScatterXAxisField;
        //    //cbxScatterYAxisField.SelectedItem = prevScatterYAxisField;
        //    //cbxSingleField.SelectedItem = prevSingleField;
        //    //cbxWeightField.SelectedItem = prevWeightField;
        //    //cbxStrataField.SelectedItem = prevStrataField;
        //  }

        //  this.loadingCombos = false;
        //}

        private XYControlChartTypes chartTypeEnum;

        private void GenerateXYChartControl()
        {
            this.dbp = new DashboardPanel();
            //dp.Title = "My Chart Title";
            this.dbp.Theme = "ArcticWhite";
            this.dbp.Style = this.Resources["DashboardPanelStyle1"] as Style;

            List<FrequencyResultData> frd = ((XYChartViewModel)this.DataContext).FrequencyTable;
            this.PieChart = new PieChart();
            if (this.chartTypeEnum == XYControlChartTypes.Pie)
            {
                this.PieChart.Name = "Chart1";
                this.PieChart.XPath = "IndependantValue";
                this.PieChart.ChartKind = PieChartKind.Pie2D;
                this.PieChart.Palette = Palette.GetPalette("VibrantC");
                this.PieChart.MinWidth = 700;
                this.PieChart.MaxWidth = 2000;
                this.PieChart.Height = 500;
                this.PieChart.Width = 1000;
            }
            else
            {
                this.XYChartControlChart = new XYChart
                {
                    RenderTransform = scaleTransform,
                    MinWidth = 800,
                    MaxWidth = 2000,
                    Height = 400,
                    Width = 1000,
                    HighlightDataPointOnHover = true,
                    XPath = "IndependentValue",
                    SelectionVisualHint = ComponentArt.Silverlight.DataVisualization.Charting.SelectionVisualHint.InvertedColor,
                    Theme = "ArcticWhite",
                    Palette = Palette.GetPalette("VibrantA"), // "VibrantC")     
                    EnableAnimation = true,
                    AnimationDuration = new TimeSpan(0, 0, 0, 4),
                    EnableDataPointPopup = true,
                    LegendVisible = true,
                    LegendDock = ComponentArt.Silverlight.DataVisualization.Charting.Dock.Right,
                    CoordinatesPaddingPercentage = new Thickness(5, 5, 0, 0),
                    DefaultAxisAnnotationsVisible = true,
                    DefaultGridLinesVisible = true,
                    GlareCoverVisible = false,
                    AnimationOnLoad = false,
                    DefaultStripesVisible = true
                    //Orientation = Orientation.Horizontal
                    //ThemeVariant = ThemeVariant.Standard// new ThemeVariant()    // Standard  
                };
            }

            Legend EpiLegend = new Legend()
            {
                // CornerRadius = new CornerRadius(10),
                BorderBrush = new SolidColorBrush(Color.FromArgb(255, 192, 207, 226)), //   "#FFc0cfe2", 
                //Margin = new Thickness(30, 0, 10, 0),
                FontFamily = new FontFamily("Segoe UI"),
            };

            AxisCoordinates XaxisCoordinates = new AxisCoordinates()
            {

                //  AxisId = "XAxis",
                //  Angle = 70,
                Margin = new Thickness(0, 5, 0, 0),
                //FontSize = 8.5,
                LabelGap = 1.0//,
                //  Coordinates = new NumericCoordinates()     //      DateTimeCoordinates()

            };

            ChartLabel chartLabel = new ChartLabel()
            {
                //Orientation = ChartLabelOrientation.Vertical
            };

            CompositeSeries compositeSeries = new CompositeSeries()
            {
                // CompositionKind = ComponentArt.Silverlight.DataVisualization.Charting.CompositionKind.SideBySide
            };

            PieSeries pieSeries = new PieSeries();

            ChartLabel ychartlabel = new ChartLabel();

            switch (this.chartTypeEnum)
            {
                case XYControlChartTypes.StackedColumn:
                    this.XYChartControlChart.Orientation = Orientation.Vertical;
                    chartLabel.Orientation = ChartLabelOrientation.Horizontal;
                    this.ChartName = "StackedColumn";
                    this.textBlockX = new TextBlock()
                    {
                        Text = ((EwavColumn)this.cbxColumnXAxisField.SelectedItem).Name,
                        //Width = 402,
                        TextAlignment = System.Windows.TextAlignment.Center,
                        Padding = new Thickness(0, 10, 0, 10),
                        Margin = new Thickness(20, 0, 0, 0)
                    };

                    XaxisCoordinates.Angle = 70.0;
                    ychartlabel.Orientation = ComponentArt.Silverlight.DataVisualization.Charting.ChartLabelOrientation.Vertical;
                    if (((ComboBoxItem)this.cbxColumnAggregateFunc.SelectedItem).Content.ToString() == "Count")
                    {
                        compositeSeries.CompositionKind = ComponentArt.Silverlight.DataVisualization.Charting.CompositionKind.Stacked;
                    }
                    else
                    {
                        compositeSeries.CompositionKind = ComponentArt.Silverlight.DataVisualization.Charting.CompositionKind.Stacked100;
                    }

                    break;
                case XYControlChartTypes.Column:
                    this.ChartName = "Column";
                    this.XYChartControlChart.Orientation = Orientation.Vertical;
                    chartLabel.Orientation = ChartLabelOrientation.Horizontal;
                    this.textBlockX = new TextBlock()
                    {
                        Text = ((EwavColumn)this.cbxSingleField.SelectedItem).Name,
                        //Width = 402,
                        TextAlignment = System.Windows.TextAlignment.Center,
                        Padding = new Thickness(0, 10, 0, 10),
                        Margin = new Thickness(20, 0, 0, 0)
                    };

                    ychartlabel.Orientation = ComponentArt.Silverlight.DataVisualization.Charting.ChartLabelOrientation.Vertical;
                    XaxisCoordinates.Angle = 70.0;

                    compositeSeries.CompositionKind = ComponentArt.Silverlight.DataVisualization.Charting.CompositionKind.SideBySide;

                    break;
                case XYControlChartTypes.Line:
                    this.XYChartControlChart.Orientation = Orientation.Vertical;
                    this.ChartName = "Line";
                    chartLabel.Orientation = ChartLabelOrientation.Horizontal;
                    this.textBlockX = new TextBlock()
                    {
                        Text = ((EwavColumn)this.cbxSingleField.SelectedItem).Name,
                        //Width = 402,
                        TextAlignment = System.Windows.TextAlignment.Center,
                        Padding = new Thickness(0, 10, 0, 10),
                        Margin = new Thickness(20, 0, 0, 0)
                    };
                    ychartlabel.Orientation = ComponentArt.Silverlight.DataVisualization.Charting.ChartLabelOrientation.Vertical;
                    XaxisCoordinates.Angle = 70.0;
                    break;
                case XYControlChartTypes.Bar:
                    this.ChartName = "Bar";
                    this.XYChartControlChart.Orientation = Orientation.Horizontal;
                    chartLabel.Orientation = ChartLabelOrientation.Vertical;
                    this.textBlockX = new TextBlock()
                    {
                        Text = ((EwavColumn)this.cbxSingleField.SelectedItem).Name,
                        //Width = 402,
                        TextAlignment = System.Windows.TextAlignment.Center,
                        //  Padding = new Thickness(0, 10, 0, 10),
                        // Margin = new Thickness(20, 0, 0, 0)
                    };
                    ychartlabel.Orientation = ComponentArt.Silverlight.DataVisualization.Charting.ChartLabelOrientation.Horizontal;
                    compositeSeries.CompositionKind = ComponentArt.Silverlight.DataVisualization.Charting.CompositionKind.SideBySide;
                    break;
                case XYControlChartTypes.Pareto:
                    this.ChartName = "Pareto";
                    this.XYChartControlChart.Orientation = Orientation.Vertical;
                    chartLabel.Orientation = ChartLabelOrientation.Horizontal;
                    this.textBlockX = new TextBlock()
                    {
                        Text = ((EwavColumn)this.cbxSingleField.SelectedItem).Name,
                        //Width = 402,
                        TextAlignment = System.Windows.TextAlignment.Center,
                        Padding = new Thickness(0, 10, 0, 10),
                        Margin = new Thickness(20, 0, 0, 0)
                    };

                    ychartlabel.Orientation = ComponentArt.Silverlight.DataVisualization.Charting.ChartLabelOrientation.Vertical;

                    XaxisCoordinates.Angle = 70.0;

                    compositeSeries.CompositionKind = ComponentArt.Silverlight.DataVisualization.Charting.CompositionKind.SideBySide;

                    break;
                default:
                    break;
            }

            chartLabel.Child = this.textBlockX;

            this.textBlockY = new TextBlock()
            {
                Text = "Count",
                //Width = 402,
                VerticalAlignment = System.Windows.VerticalAlignment.Bottom,
                TextAlignment = System.Windows.TextAlignment.Center,
                //  Padding = new Thickness(0, 10, 0, 10),
                // Margin = new Thickness(0, 0, 0, 20)
            };

            if (this.chartTypeEnum == XYControlChartTypes.Pie)
            {
            }
            else
            {
                this.XYChartControlChart.XAxisArea.Add(XaxisCoordinates);  //   axisCoordinates);
                this.XYChartControlChart.XAxisArea.Add(chartLabel);

                this.XYChartControlChart.YAxisArea.Add(new AxisCoordinates());

                ychartlabel.Child = this.textBlockY;

                this.XYChartControlChart.YAxisArea.Add(ychartlabel);
            }
            //  AxisCoordinates

            //string seriesTrackerText = "";
            //int seriesCounter = 0;

            //List<MyString> chartColsList = dtb.ColumnNameList.GetRange(1, dtb.ColumnNameList.Count - 1);

            string XYChartControlSeriesId, pieSeriesID;
            int sId = 0;
            double max = 0;

            foreach (FrequencyResultData frData in frd)
            {
                List<ChartDataValue> chartDataValuesList = new List<ChartDataValue>();

                List<EwavFrequencyControlDto> fcDtoList = frData.FrequencyControlDtoList;

                if (fcDtoList.Count < 1)
                {
                    this.RenderFinishWithError(SharedStrings.NO_RECORDS_SELECTED);

                    return;
                }

                if (fcDtoList[0].NameOfDtoList.Contains("Missing"))
                {
                    continue;
                }

                XYChartControlSeriesId = fcDtoList[0].NameOfDtoList;
                pieSeriesID = string.Format("S{0}", sId);
                sId++;
                double total = 0;

                if (this.chartTypeEnum == XYControlChartTypes.Pareto)
                {
                    foreach (EwavFrequencyControlDto fcd in fcDtoList)
                    {
                        max = max + fcd.FrequencyColumn.SafeParsetoDou();
                        total += Convert.ToDouble(fcd.FrequencyColumn);
                    }
                }

                double runningPercent = 0;

                foreach (EwavFrequencyControlDto fcd in fcDtoList)
                {
                    total += Convert.ToDouble(fcd.FrequencyColumn);
                }

                foreach (EwavFrequencyControlDto fcd in fcDtoList)
                {
                    ChartDataValue data;
                    if (this.chartTypeEnum == XYControlChartTypes.Pie)
                    {
                        data = new ChartDataValue()
                        {
                            IndependentValue = string.Format("{0} - {1}", XYChartControlSeriesId, fcd.FreqVariable),
                            DependentValue = fcd.FrequencyColumn
                        };
                        //pieSeries.Label = data.IndependentValue;
                    }
                    else
                    {
                        data = new ChartDataValue()
                        {
                            IndependentValue = fcd.FreqVariable.ToString(),
                            DependentValue = fcd.FrequencyColumn
                        };
                    }

                    if (this.chartTypeEnum == XYControlChartTypes.Pareto)
                    {
                        // double 
                        data.currentMeanValue = (((data.DependentValue.SafeParsetoDou() / max) * 100) + runningPercent).ToString();
                        runningPercent = data.currentMeanValue.SafeParsetoDou();
                    }

                    if (data.DependentValue.Length == 0)
                    {
                        data.DependentValue = "0";
                    }
                    chartDataValuesList.Add(data);
                }

                switch (this.chartTypeEnum)
                {
                    case XYControlChartTypes.Pie:
                        pieSeries = new PieSeries();
                        //pieSeries.Id = pieSeriesID; //XYChartControlSeriesId;
                        pieSeries.Id = "S0";
                        pieSeries.YPath = "DependentValue";
                        pieSeries.XPath = "IndependentValue";
                        pieSeries.Texture = TextureKind.BrushedMetal;
                        pieSeries.DataSource = chartDataValuesList;
                        pieSeries.EnableDataPointPopup = true;
                        pieSeries.DataPointPopup = this.Resources["CustomDataPointPopup"] as DataPointPopup;
                        pieSeries.ShowPointAnnotations = true;
                        DataPointAnnotation dpa = new DataPointAnnotation();
                        dpa.RelativeX = 0.8;
                        dpa.RelativeY = 0.5;
                        dpa.RelativeRadialOffset = 0.3;
                        dpa.HorizontalOffset = 5;
                        dpa.Template = this.Resources["cdpa"] as DataTemplate;

                        dpa.LineStroke = new SolidColorBrush(Color.FromArgb(255, 0, 0, 0));
                        //dpa.Template = (DataTemplate)XamlReader.Load("<DataTemplate xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\">"
                        //  + "<StackPanel Orientation=\"Horizontal\"><TextBlock Text=\"{Binding DataPoint.X}\" Foreground=\"Black\" HorizontalAlignment=\"Right\" FontWeight=\"Bold\"/><TextBlock Text=\" : \" Foreground=\"Black\" HorizontalAlignment=\"Right\" FontWeight=\"Bold\"/><TextBlock Text=\"{Binding Text, ElementName=percentText}\" Foreground=\"Black\" HorizontalAlignment=\"Right\" FontWeight=\"Bold\"/></StackPanel></DataTemplate>");
                        pieSeries.DataPointAnnotations.Add(dpa);

                        // pieSeries.DataPointAnnotation = Resources["CustomDataPointAnnotation"] as DataPointAnnotation;
                        this.PieChart.PieChartMainArea.Add(pieSeries);

                        pieSeries.ShowPointAnnotations = true;
                        this.PieChart.LegendVisible = true;

                        break;
                    case XYControlChartTypes.Line:
                        LineSeries XYChartControlLineSeries = new LineSeries();
                        //  LineSeries XYChartControlSeries = new LineSeries();

                        XYChartControlLineSeries.Id = XYChartControlSeriesId;
                        XYChartControlLineSeries.YPath = "DependentValue";
                        XYChartControlLineSeries.XPath = "IndependentValue";
                        XYChartControlLineSeries.Texture = TextureKind.BrushedMetal;
                        XYChartControlLineSeries.DataSource = chartDataValuesList;
                        //  XYChartControlChart.XYChartMainArea.Add(XYChartControlSeries);

                        this.XYChartControlChart.Width = (chartDataValuesList.Count * 30) + 250;
                        compositeSeries.SubSeries.Add(XYChartControlLineSeries);

                        break;
                    case XYControlChartTypes.Pareto:
                        BarSeries XYChartControlSeries = new BarSeries();
                        //  LineSeries XYChartControlSeries = new LineSeries();

                        XYChartControlSeries.Id = XYChartControlSeriesId;
                        XYChartControlSeries.YPath = "DependentValue";
                        XYChartControlSeries.XPath = "IndependentValue";
                        XYChartControlSeries.Texture = TextureKind.BrushedMetal;

                        XYChartControlSeries.DataSource = chartDataValuesList;

                        AxisCoordinates Y2AxisCoordinates = new AxisCoordinates()
                        {
                            AxisId = "PercentAxis",
                            Coordinates = new NumericCoordinates()
                            {
                                From = 0,
                                To = 100,
                                Step = 10
                            },
                        };

                        AxisCoordinates YAxisCoordinates = new AxisCoordinates();

                        LineSeries lineSeries = new LineSeries();

                        if (this.chartTypeEnum == XYControlChartTypes.Pareto)
                        {
                            lineSeries.Id = "aa";
                            lineSeries.YAxisId = "PercentAxis";
                            lineSeries.Label = "Accumulated %";
                            lineSeries.YPath = "CurrentMeanValue";
                            lineSeries.XPath = "IndependentValue";
                            lineSeries.Texture = TextureKind.BrushedMetal;
                            lineSeries.MarkerVisible = false;
                            lineSeries.DashStyle = LineDashStyle.Dash;

                            lineSeries.DataSource = chartDataValuesList;
                            // lineSeries.DataPointPopup = Resources["CustomDataPointPopupPareto"] as DataPointPopup;
                            this.XYChartControlChart.Y2AxisArea.Add(Y2AxisCoordinates);
                        }

                        this.XYChartControlChart.Width = (chartDataValuesList.Count * 30) + 250;  // 25;
                        this.XYChartControlChart.MaxHeight = 800;
                        this.XYChartControlChart.MinWidth = 600;
                        compositeSeries.SubSeries.Add(XYChartControlSeries);
                        compositeSeries.SubSeries.Add(lineSeries);

                        break;
                    default:
                        XYChartControlSeries = new BarSeries();
                        //  LineSeries XYChartControlSeries = new LineSeries();

                        XYChartControlSeries.Id = XYChartControlSeriesId;
                        XYChartControlSeries.YPath = "DependentValue";
                        XYChartControlSeries.XPath = "IndependentValue";
                        XYChartControlSeries.Texture = TextureKind.BrushedMetal;
                        //  XYChartControlSeries.YAxisId = 
                        XYChartControlSeries.DataSource = chartDataValuesList;
                        //  XYChartControlChart.XYChartMainArea.Add(XYChartControlSeries);    

                        if (this.chartTypeEnum == XYControlChartTypes.Bar)
                        {
                            //this.XYChartControlChart.Width = 1000;
                            this.XYChartControlChart.MaxWidth = 800;
                            this.XYChartControlChart.Height = (chartDataValuesList.Count * 30) + 150;
                            this.XYChartControlChart.MinHeight = 400;
                        }
                        else
                        {
                            this.XYChartControlChart.Width = (chartDataValuesList.Count * 30) + 150;
                        }

                        compositeSeries.SubSeries.Add(XYChartControlSeries);

                        break;
                }
                //string datatemplate = "<DataTemplate> <TextBlock TextAlignment=\"Center\" Text=\"YES\" /> </DataTemplate>";
                //DataTemplate template = (DataTemplate)XamlReader.Load(datatemplate);
                //DataPointPopup dpp = new DataPointPopup();
                //dpp.IndexTemplate = template;
                //PieChart.DataPointPopup = dpp;
            }

            //seriesTrackerText = seriesTrackerText.Substring(0, seriesTrackerText.Length - 1);
            // SeriesTracker st = new SeriesTracker();
            // st.SeriesId = seriesTrackerText;

            //SeriesAnnotationTracker sat = new SeriesAnnotationTracker();
            //sat.SeriesIdsCSS = seriesTrackerText;

            //XYChartControlChart.XYChartMainArea.Add(compositeSeries);
            //XYChartControlChart.XYChartMainArea.Add(st);
            //XYChartControlChart.XYChartMainArea.Add(sat);

            //pnlChartContainer.Visibility = System.Windows.Visibility.Visible;

            if (this.chartTypeEnum == XYControlChartTypes.Pie)
            {
                this.PieChart.EnableDataPointPopup = true;
                this.PieChart.DataPointPopup = this.Resources["CustomDataPointPopup"] as DataPointPopup;
                this.pnlChartContainer.Children.Clear();
                this.dbp.Content = this.PieChart;
            }
            else
            {
                this.XYChartControlChart.XYChartMainArea.Add(compositeSeries);

                this.pnlChartContainer.Children.Clear();
                this.dbp.Content = this.XYChartControlChart;
                //pnlChartContainer.Children.Add(dbp);
            }
            this.pnlChartContainer.Children.Add(this.dbp);
            this.GName.Text = this.ChartName.FromCamelCase();
            this.RenderFinish();
        }

        void epiCurveWorker_RunWorkerCompleted()
        {
            if (this.databag != null)
            {
                //DataTable data = (DataTable)((object[])e.Result)[1];
                //List<List<StringDataValue>> dataValues = (List<List<StringDataValue>>)((object[])databag)[0];
                //List<List<StringDataValue>> dataValues = new List<List<StringDataValue>>(); ;
                //this.Dispatcher.BeginInvoke(new RenderFinishEpiCurveDelegate(RenderFinishEpiCurve), data, dataValues);
                this.RenderFinishEpiCurve(this.databag, this.dataValues);
            }
        }

        private void RenderFinishEpiCurve(DatatableBag data, List<List<StringDataValue>> dataValues)
        {
            this.waitCursor.Visibility = System.Windows.Visibility.Collapsed;

            this.SetChartLabels(this.ChartTitle, this.LegendTitle, this.XAxisLabel, this.YAxisLabel);
        }

        private List<StringDataValue> ConvertToPct(List<StringDataValue> list)
        {
            List<StringDataValue> values = new List<StringDataValue>();
            foreach (StringDataValue value in list)
            {
                values.Add(new StringDataValue() { IndependentValue = value.IndependentValue, DependentValue = value.DependentValue / 31 * 100 });
            }
            return values;
        }

        private bool IsBooleanWithNoStratas
        {
            get
            {
                return this.isBooleanWithNoStratas;
            }
            set
            {
                this.isBooleanWithNoStratas = value;
            }
        }

        public void ClearResults()
        {
            this.txtStatus.Text = string.Empty;
            this.pnlStatus.Visibility = Visibility.Collapsed;
            this.waitCursor.Visibility = Visibility.Visible;

            this.IsBooleanWithNoStratas = false;

            this.pnlChartContainer.Children.Clear();
            this.pnlChartContainer.Visibility = System.Windows.Visibility.Collapsed;
        }

        #region Private Properties
        //private View View
        //{
        //    get
        //    {
        //        return this.dashboardHelper.View;
        //    }
        //}

        //private IDbDriver Database
        //{
        //    get
        //    {
        //        return this.dashboardHelper.Database;
        //    }
        //}
        #endregion // Private Properties

        #region IGadget Members

        public bool IsProcessing { get; set; }

        /// <summary>
        /// Updates the variable names available in the gadget's properties
        /// </summary>
        public void UpdateVariableNames()
        {
            //FillComboboxes(true);
        }

        public void RefreshResults()
        {
            try
            {
                this.waitCursor.Visibility = Visibility.Visible;
                this.pnlStatus.Visibility = Visibility.Collapsed;

                //pnlStatus.Background = Brushes.PaleGreen;
                //pnlStatusTop.Background = Brushes.Green;

                this.gadgetOptions = new GadgetParameters();

                this.gadgetOptions.MainVariableName = string.Empty;
                this.gadgetOptions.WeightVariableName = string.Empty;
                this.gadgetOptions.StrataVariableList = new List<MyString>();

                this.gadgetOptions.TableName = this.applicationViewModel.EwavSelectedDatasource.TableName;
                this.gadgetOptions.DatasourceName = this.applicationViewModel.EwavSelectedDatasource.DatasourceName;

                this.gadgetOptions.InputVariableList = new Dictionary<string, string>();
                this.gadgetOptions.ShouldUseAllPossibleValues = false;

                switch (this.chartTypeEnum)
                {
                    case XYControlChartTypes.StackedColumn:

                        if (this.cbxColumnXAxisField.SelectedIndex > -1)
                        {
                            this.gadgetOptions.MainVariableName = ((EwavColumn)this.cbxColumnXAxisField.SelectedItem).Name;
                        }
                        else
                        {
                            this.gadgetOptions.MainVariableName = null;
                        }

                        //      gadgetOptions.CrosstabVariableName = cbxStrataField;    
                        if (this.cbxColumnYAxisField.SelectedIndex > 0)
                        {
                            MyString mys = new MyString();
                            mys.VarName = ((EwavColumn)this.cbxColumnYAxisField.SelectedItem).Name;
                            this.gadgetOptions.StrataVariableList.Add(mys);
                        }
                        else
                        {
                            this.gadgetOptions.StrataVariableList.Clear();
                        }
                        break;
                    case XYControlChartTypes.Bar:
                    case XYControlChartTypes.Column:
                    case XYControlChartTypes.Line:
                    case XYControlChartTypes.Pie:
                        if (this.cbxChartSize.SelectedIndex > -1)
                        {
                            this.gadgetOptions.InputVariableList.Add("chartsize", this.cbxChartSize.SelectedValue.ToString());
                        }
                        // gadgetOptions.MainVariableName = ((EwavColumn)cbxSingleField.SelectedItem).Name;
                        //      gadgetOptions.CrosstabVariableName = cbxStrataField;    

                        if (this.cbxWeightField.SelectedItem != null)
                        {
                            this.gadgetOptions.WeightVariableName = ((EwavColumn)this.cbxWeightField.SelectedItem).Name;
                        }
                        else
                        {
                            this.gadgetOptions.WeightVariableName = null;
                        }

                        if (this.cbxSingleField.SelectedItem != null)
                        {
                            this.gadgetOptions.MainVariableName = ((EwavColumn)this.cbxSingleField.SelectedItem).Name.ToString();
                        }
                        else
                        {
                            this.gadgetOptions.MainVariableName = null;
                        }
                        if (this.cbxStrataField.SelectedIndex > 0)
                        {
                            MyString mys = new MyString();
                            mys.VarName = ((EwavColumn)this.cbxStrataField.SelectedItem).Name;
                            this.gadgetOptions.StrataVariableList.Add(mys);
                        }
                        else
                        {
                            this.gadgetOptions.StrataVariableList.Clear();
                        }
                        break;
                    case XYControlChartTypes.Pareto:

                        if (this.cbxChartSize.SelectedIndex > -1)
                        {
                            this.gadgetOptions.InputVariableList.Add("chartsize", this.cbxChartSize.SelectedValue.ToString());
                        }

                        if (this.cbxWeightField.SelectedItem != null)
                        {
                            this.gadgetOptions.WeightVariableName = ((EwavColumn)this.cbxWeightField.SelectedItem).Name;
                        }
                        else
                        {
                            this.gadgetOptions.WeightVariableName = null;
                        }

                        this.gadgetOptions.ShouldSortHighToLow = true;

                        if (this.cbxSingleField.SelectedItem != null)
                        {
                            this.gadgetOptions.MainVariableName = ((EwavColumn)this.cbxSingleField.SelectedItem).Name.ToString();
                        }
                        else
                        {
                            this.gadgetOptions.MainVariableName = null;
                        }
                        if (this.cbxStrataField.SelectedIndex > 0)
                        {
                            MyString mys = new MyString();
                            mys.VarName = ((EwavColumn)this.cbxStrataField.SelectedItem).Name;
                            this.gadgetOptions.StrataVariableList.Add(mys);
                        }
                        else
                        {
                            this.gadgetOptions.StrataVariableList.Clear();
                        }
                        break;
                    default:
                        break;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.StackTrace);
            }
        }

        /// <summary>
        /// Sets the gadget to its 'processing' state
        /// </summary>
        public void SetGadgetToProcessingState()
        {
            //cbxCaseStatusField.IsEnabled = false;
            //cbxDateInterval.IsEnabled = false;
            //cbxChartSize.IsEnabled = false;
            //cbxChartType.IsEnabled = false;
            //cbxColumnAggregateFunc.IsEnabled = false;
            //cbxColumnXAxisField.IsEnabled = false;
            //cbxColumnYAxisField.IsEnabled = false;
            //  cbxDateField.IsEnabled = false;
            //cbxScatterXAxisField.IsEnabled = false;
            //cbxScatterYAxisField.IsEnabled = false;
            //cbxSingleField.IsEnabled = false;
            //cbxStrataField.IsEnabled = false;
            //cbxWeightField.IsEnabled = false;
            //txtXAxisEndValue.IsEnabled = false;
            //txtXAxisStartValue.IsEnabled = false;
            this.IsProcessing = true;
        }

        /// <summary>
        /// Sets the gadget to its 'finished' state
        /// </summary>
        public void SetGadgetToFinishedState()
        {
        }

        public string CustomOutputHeading { get; set; }

        public string CustomOutputDescription { get; set; }

        public string CustomOutputCaption { get; set; }

        public string ChartTitle { get; set; }

        public string LegendTitle { get; set; }

        public string XAxisLabel { get; set; }

        public string YAxisLabel { get; set; }

        #endregion

        private void DoChart()
        {
            if (!LoadingDropDowns && !LoadingCanvas)
            {
                // MEF patch 
                if (this.DataContext == null)
                {
                    // Only reply if chart data is loaded  
                    if (this.chartDataLoaded)
                    {
                        return;
                    }
                }

                if (ValidateComboBoxes())
                {
                    IsUserDefindVariableInUse();
                    this.RefreshResults();
                    xyChartViewModel = (XYChartViewModel)this.DataContext;
                    this.gadgetOptions.TableName = this.applicationViewModel.EwavSelectedDatasource.TableName;
                    this.gadgetOptions.DatasourceName = this.applicationViewModel.EwavSelectedDatasource.DatasourceName;
                    this.gadgetOptions.UseAdvancedDataFilter = applicationViewModel.UseAdvancedFilter;
                    this.xyChartViewModel.GetFrequencyData(this.gadgetOptions);
                }
                this.gadgetExpander.IsExpanded = false;
            }
        }

        /// <summary>
        /// Use to verify if Defined Variables are in Use
        /// </summary>
        private void IsUserDefindVariableInUse()
        {
            Col1 = (cbxSingleField.SelectedIndex > -1) ? (EwavColumn)cbxSingleField.SelectedItem : null;
            Col2 = (cbxWeightField.SelectedIndex > -1) ? (EwavColumn)cbxWeightField.SelectedItem : null;
            Col3 = (cbxStrataField.SelectedIndex > -1) ? (EwavColumn)cbxStrataField.SelectedItem : null;
            Col4 = (cbxColumnXAxisField.SelectedIndex > -1) ? (EwavColumn)cbxColumnXAxisField.SelectedItem : null;
            Col5 = (cbxColumnYAxisField.SelectedIndex > -1) ? (EwavColumn)cbxColumnYAxisField.SelectedItem : null;
            if (Col1 != null && Col1.IsUserDefined == true)
            {
                Col1.IsInUse = true;
            }
            if (Col2 != null && Col2.IsUserDefined == true)
            {
                Col2.IsInUse = true;
            }
            if (Col3 != null && Col3.IsUserDefined == true)
            {
                Col3.IsInUse = true;
            }
            if (Col4 != null && Col4.IsUserDefined == true)
            {
                Col4.IsInUse = true;
            }
            if (Col5 != null && Col5.IsUserDefined == true)
            {
                Col5.IsInUse = true;
            }
        }

        private void btnRun_Click(object sender, RoutedEventArgs e)
        {
            DoChart();
            //Serialize(new XDocument());
        }

        private void EnableDisableButton()
        {
            if ((this.cbxColumnXAxisField.SelectedIndex > 0 && this.cbxColumnYAxisField.SelectedIndex > 0) ||
                this.cbxSingleField.SelectedIndex > 0)
            {
                this.btnRun.IsEnabled = true;
            }
            else
            {
                this.btnRun.IsEnabled = false;
            }
        }

        void IGadget.CloseGadget()
        {
            this.CloseGadget();
        }

        private void CloseButton_Click(object sender, RoutedEventArgs e)
        {
            this.CloseGadget();
        }

        private void ResizeButton_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            // TODO: Add event handler implementation here.
            if (spContent.Visibility == System.Windows.Visibility.Visible)
            {
                spContent.Visibility = System.Windows.Visibility.Collapsed;
            }
            else
            {
                spContent.Visibility = System.Windows.Visibility.Visible;
            }
        }

        private void LayoutRoot_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
        }

        private void LayoutRoot_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            Point p = e.GetSafePosition(cmnClass.GetParentObject<Grid>(this, "LayoutRoot"));
            ((Ewav.DragCanvas)VisualTreeHelper.GetParent(this)).GadgetNameOnRightClick = MyControlName;
            ((Ewav.DragCanvas)VisualTreeHelper.GetParent(this)).StrataList = null;
            ((Ewav.DragCanvas)VisualTreeHelper.GetParent(this)).SelectedGadget = this;
            cmnClass.UpdateZOrder(this, true, cmnClass.GetParentObject<Grid>(this, "LayoutRoot"));
        }

        public string MyControlName
        {
            get
            {
                return "XYChartControl";
            }
        }

        public string MyUIName
        {
            get
            {
                return "XYChartControl Chart";
            }
        }

        public ApplicationViewModel ApplicationViewModel
        {
            get
            {
                return this.applicationViewModel;
            }
        }

        private void cbxField_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.EnableDisableButton();
        }

        //XNode IGadget.Serialize(XDocument doc)
        //{
        //    throw new NotImplementedException();
        //}

        void IGadget.CreateFromXml(XElement element)
        {
            this.LoadingDropDowns = true;
            this.LoadingCanvas = true;

            //foreach (var item in element.Descendants("chartType"))
            //{
            //    this.chartTypeEnum = (XYControlChartTypes)Enum.Parse(typeof(XYControlChartTypes), item.Value.ToString(), true);
            //}

            //InitializeXYControl();

            foreach (XElement child in element.Descendants())
            {
                switch (child.Name.ToString().ToLower())
                {
                    //case "casestatusvariable":
                    //    cbxCaseStatusField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "datevariable":
                    //    cbxDateField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "dateinterval":
                    //    cbxDateInterval.Text = child.InnerText;
                    //    break;
                    case "yaxisvariable":
                        //cbxColumnYAxisField.Text = child.InnerText.Replace("&lt;", "<");
                        cbxColumnYAxisField.SelectedIndex = cmnClass.FindIndexToSelect(cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType, false), child.Value.Replace("&lt;", "<"));
                        break;
                    case "xaxisvariable":
                        cbxColumnXAxisField.SelectedIndex = cmnClass.FindIndexToSelect(cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType), child.Value.Replace("&lt;", "<"));
                        break;
                    //case "yaxisscattervariable":
                    //    cbxScatterYAxisField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "xaxisscattervariable":
                    //    cbxScatterXAxisField.Text = child.InnerText.Replace("&lt;", "<");
                    //    break;
                    //case "allvalues":
                    //    if (child.InnerText.ToLower().Equals("true"))
                    //    {
                    //        checkboxAllValues.IsChecked = true;
                    //    }
                    //    else
                    //    {
                    //        checkboxAllValues.IsChecked = false;
                    //    }
                    //    break;
                    case "singlevariable":
                        cbxSingleField.SelectedIndex = cmnClass.FindIndexToSelect(cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType), child.Value.Replace("&lt;", "<"));
                        break;
                    case "weightvariable":
                        cbxWeightField.SelectedIndex = cmnClass.FindIndexToSelect(cmnClass.GetItemsSource(GetNumericFieldColumnDataType), child.Value.Replace("&lt;", "<"));

                        break;
                    case "stratavariable":
                        cbxStrataField.SelectedIndex = cmnClass.FindIndexToSelect(cmnClass.GetItemsSource(GetPrimaryFieldColumnDataType, false), child.Value.Replace("&lt;", "<"));
                        break;
                    //case "columnaggregatefunction":
                    //    cbxColumnAggregateFunc.Text = child.InnerText;
                    //    break;
                    case "charttype":
                        this.ChartName = child.Value.ToString();
                        break;
                    case "customheading":
                    //    this.CustomOutputHeading = child.Value;
                    //    break;
                    //case "customdescription":
                    //    this.CustomOutputDescription = child.InnerText;
                    //    break;
                    //case "customcaption":
                    //    this.CustomOutputCaption = child.InnerText;
                    //    break;
                    case "chartsize":
                        cbxChartSize.SelectedItem = child.Value.ToString();
                        break;
                    case "charttitle":
                        ChartTitle = child.Value.ToString();
                        break;
                    case "chartlegendtitle":
                        LegendTitle = child.Value.ToString();
                        break;
                    case "xaxislabel":
                        XAxisLabel = child.Value.ToString();
                        break;
                    case "yaxislabel":
                        YAxisLabel = child.Value.ToString();
                        break;
                    //case "xaxisstartvalue":
                    //    txtXAxisStartValue.Text = child.InnerText;
                    //    break;
                    //case "xaxisendvalue":
                    //    txtXAxisEndValue.Text = child.InnerText;
                    //    break;
                    //case "xaxisrotation":
                    //    switch (child.Value)
                    //    {
                    //        case "0":
                    //            cbxXAxisLabelRotation.SelectedIndex = 0;
                    //            break;
                    //        case "45":
                    //            cbxXAxisLabelRotation.SelectedIndex = 1;
                    //            break;
                    //        default:
                    //            cbxXAxisLabelRotation.SelectedIndex = 2;
                    //            break;
                    //    }
                    //    break;
                }
            }
            this.LoadingDropDowns = false;
            this.LoadingCanvas = false;
            double mouseVerticalPosition = 0.0, mouseHorizontalPosition = 0.0;
            foreach (XAttribute attribute in element.Attributes())
            {
                switch (attribute.Name.ToString().ToLower())
                {
                    case "top":
                        mouseVerticalPosition = double.Parse(attribute.Value);
                        break;
                    case "left":
                        mouseHorizontalPosition = double.Parse(attribute.Value);
                        break;
                }
            }

            ShowAppropriateChart();
            DoChart();
            cmnClass.AddControlToCanvas(this, mouseVerticalPosition, mouseHorizontalPosition, applicationViewModel.LayoutRoot);
            //CollapseExpandConfigPanel();
        }
    }
}

namespace Ewav.Web.Services
{
    public partial class XYChartDomainContext
    {
        //This is the place to set RIA Services query timeout. 
        //TimeSpan(0, 5, 0) means five minutes vs the 
        //default 60 sec
        partial void OnCreated()
        {
            if (!DesignerProperties.GetIsInDesignMode(Application.Current.RootVisual))
            {
                ((WebDomainClient<IXYChartDomainServiceContract>)DomainClient).ChannelFactory.Endpoint.Binding.SendTimeout =
                    new TimeSpan(0, 120, 0);
            }
        }
    }
}