﻿/*  ----------------------------------------------------------------------------
 *  Emergint Technologies, Inc.
 *  ----------------------------------------------------------------------------
 *  Epi Info™ - Web Analysis & Visualization
 *  ----------------------------------------------------------------------------
 *  File:       MainWindow.xaml.cs
 *  Namespace:  EwavEncryptionUtility    
 *
 *  Author(s):  Daniel Shorter, Mohammad Usman, Ninad Date, Sachin Agnihotri 
 *  Created:    22/08/2013    
 *  Summary:	no summary     
 *  ----------------------------------------------------------------------------
 */
using System;
using System.Windows;
using Ewav.Security;
using Microsoft.Win32;

namespace EwavEncryptionUtility
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {


        string clipText;

        /// <summary>
        /// Initializes a new instance of the <see cref="Window1" /> class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
            this.Title = "Ewav Encryption Utility";
       


            labelClipboard.Visibility = System.Windows.Visibility.Hidden;
            buttonCopy.IsEnabled = false;    
            buttonEncrypt.IsEnabled = CheckButons();
            buttonDecrypt.IsEnabled = CheckButons();        

        }

        private void buttonExist_Click(object sender, RoutedEventArgs e)
        {


           //    textClip.IsEnabled = false;
            Cryptography c = new Cryptography(txtPassphrase.Text, txtSaltValue.Text, txtGenerateVector.Text);
            //  txtDatabaseConnectionText.Text = c.Encrypt(txtDatabaseConnectionText.Text);        
            textClip.Text = c.Encrypt(txtDatabaseConnectionText.Text);        



        }
            
        private void buttonFill_Click(object sender, RoutedEventArgs e)
        {


            OpenFileDialog ofd = new OpenFileDialog();

            ofd.FileName = "Web.config"; // Default file name
            ofd.DefaultExt = ".config"; // Default file extension
            ofd.Filter = "Web.config confiles (.config)|*.config"; // Filter files by extension 

            // Show open file dialog box
            Nullable<bool> result = ofd.ShowDialog();

            // Process open file dialog box results 
            if (result == true)
            {
                // Open document 
                string filename = ofd.FileName;

                ConfigReader cfr = new ConfigReader(filename);

                txtSaltKey2.Text = cfr.ReadKey("KeyForUserPasswordSalt");
                txtPassphrase.Text = cfr.ReadKey("KeyForConnectionStringPassphrase");
                txtSaltValue.Text = cfr.ReadKey("KeyForConnectionStringSalt");
                txtGenerateVector.Text = cfr.ReadKey("KeyForConnectionStringVector");

                buttonEncrypt.IsEnabled = CheckButons();
                buttonDecrypt.IsEnabled = CheckButons();

                textClip.Text = " ";    
              

            }
        }
  
        private void ResetTool     ()
        {



            textClip.Text = " ";
            labelClipboard.Visibility = System.Windows.Visibility.Hidden;
            txtDatabaseConnectionText.Text = " ";
            buttonCopy.IsEnabled = false;    


        }

        private void buttonNew_Click(object sender, RoutedEventArgs e)
        {
            textClip.IsEnabled = true;
            WriteKeysToClipboard();
        }

        /// <summary>
        /// Returns flag for buton activation              
        /// </summary>
        /// <returns></returns>
        private bool CheckButons()
        {
            return (txtGenerateVector.Text.Length > 0 &&
                    txtPassphrase.Text.Length > 0 &&
                    txtSaltKey2.Text.Length > 0 &&
                    txtSaltValue.Text.Length > 0);
        }

        private void textClip_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            buttonEncrypt.IsEnabled = CheckButons();
            buttonDecrypt.IsEnabled = CheckButons();    
        }

        private void txtDatabaseConnectionText_textChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            buttonEncrypt.IsEnabled = CheckButons();
            buttonDecrypt.IsEnabled = CheckButons();    
        }

        /// <summary>
        /// Writes the keys to clipboard.
        /// </summary>
        private void WriteKeysToClipboard()
        {
            string saltKey;
            string passPhrase;
            string saltValue;
            string generateVector;

            //  Ewav.Security.PasswordHasher ph = new Ewav.Security.PasswordHasher("aa");        

            // Put a if here    
            saltKey = PasswordHasher.CreateSalt();
            passPhrase = PasswordHasher.CreateSalt();
            saltValue = PasswordHasher.CreateSalt();
            generateVector = PasswordHasher.CreateSalt().Substring(0, 16);

            string addTag;


            string guid = Guid.NewGuid().ToString();

            addTag = "<!-- Encryption keys -->\n";
            addTag += string.Format("<!-- Created {0} {1}  by " + Environment.UserName + "  -->\n", DateTime.Now.ToShortDateString(), DateTime.Now.ToShortTimeString());
            addTag += string.Format("<!-- GUID:  {0} -->\n", guid.ToUpper());
            addTag += "<!-- WARNING:  -->\n";
            addTag += "<!-- Once users, datasources or connection strings have been added to the database these keys *cannot* be modified. --> \n";
            addTag += "<!-- If these keys are modified all existing encrypted data will be invalid.  -->\n";
            addTag += "<!-- WARNING:  -->\n";
            addTag += "<add key=\"KeyForUserPasswordSalt\" value=\"{0}\"    />\n";
            addTag += "<add key=\"KeyForConnectionStringPassphrase\" value=\"{1}\"    />\n";
            addTag += "<add key=\"KeyForConnectionStringSalt\" value=\"{2}\"    />\n";
            addTag += "<add key=\"KeyForConnectionStringVector\" value=\"{3}\"    />";

            if (txtDatabaseConnectionText.Text.Length > 0)
            {
                addTag += "\n\n";
                addTag += "<add key=\"MetaDataConnectionString\" value=\"{4}\"  />";

                Cryptography cy = new Cryptography(passPhrase, saltValue, generateVector);
                string databaseConnection = cy.Encrypt(txtDatabaseConnectionText.Text);

                clipText = string.Format(addTag, saltKey, passPhrase, saltValue,
                    generateVector, databaseConnection).Replace("\n", Environment.NewLine);




            }
            else
            {
                clipText = string.Format(addTag, saltKey, passPhrase, saltValue,
                    generateVector).Replace("\n", Environment.NewLine);
            }

            txtSaltKey2.Text = saltKey;
            txtPassphrase.Text = passPhrase;
            txtSaltValue.Text = saltValue;
            txtGenerateVector.Text = generateVector;



            textClip.Text = clipText;
            buttonCopy.IsEnabled = true;    


        }

        private void buttonDecrypt_Click(object sender, RoutedEventArgs e)
        {



            //  textClip.IsEnabled = false;
            Cryptography c = new Cryptography(txtPassphrase.Text, txtSaltValue.Text, txtGenerateVector.Text);
            //  txtDatabaseConnectionText.Text = c.Decrypt(txtDatabaseConnectionText.Text);

            textClip.Text = c.Decrypt(txtDatabaseConnectionText.Text);  


        }

        private void buttonCopy_Click(object sender, RoutedEventArgs e)
        {



            Clipboard.SetText(textClip.Text);    

            labelClipboard.Visibility = System.Windows.Visibility.Visible;


        }

        private void textClip_TextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {




            buttonCopy.IsEnabled = true;
            labelClipboard.Visibility = System.Windows.Visibility.Hidden;            



        }


    }
}